package mpi.eudico.client.annotator;

import mpi.eudico.client.annotator.commands.BackupCA;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.CommandAction;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.commands.PlayAroundSelectionCA;
import mpi.eudico.client.annotator.commands.PlaybackRateToggleCA;
import mpi.eudico.client.annotator.commands.PlaybackVolumeToggleCA;
import mpi.eudico.client.annotator.commands.ShortcutsUtil;
import mpi.eudico.client.annotator.commands.StoreCommand;
import mpi.eudico.client.annotator.commands.global.*;

import mpi.eudico.client.annotator.grid.GridViewer;

import mpi.eudico.client.annotator.gui.ElanMenuItem;

import mpi.eudico.client.annotator.linkedmedia.LinkedFileDescriptorUtil;
import mpi.eudico.client.annotator.linkedmedia.MediaDescriptorUtil;

import mpi.eudico.client.annotator.player.JMFGraphicMediaPlayer;

import mpi.eudico.client.annotator.recognizer.gui.AudioRecognizerPanel;
import mpi.eudico.client.annotator.recognizer.gui.VideoRecognizerPanel;
import mpi.eudico.client.annotator.svg.SVGPrefs;

import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.util.FileUtility;
import mpi.eudico.client.annotator.util.FrameConstants;

import mpi.eudico.client.annotator.viewer.InterlinearViewer;
import mpi.eudico.client.annotator.viewer.LexiconEntryViewer;
import mpi.eudico.client.annotator.viewer.SubtitleViewer;
import mpi.eudico.client.annotator.viewer.TextViewer;
import mpi.eudico.client.annotator.viewer.TimeLineViewer;
import mpi.eudico.client.annotator.viewer.TimeSeriesViewer;
import mpi.eudico.client.util.TranscriptionECVLoader;

import mpi.eudico.p2p.*;

import mpi.eudico.server.corpora.clom.ExternalReference;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clom.TranscriptionStore;

import mpi.eudico.server.corpora.clomimpl.abstr.MediaDescriptor;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.dobes.ACMTranscriptionStore;
import mpi.eudico.server.corpora.clomimpl.dobes.ECVStore;
import mpi.eudico.util.CVEntry;
import mpi.eudico.util.ControlledVocabulary;
import mpi.eudico.util.ExternalCV;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;
import java.net.URL;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.ComponentInputMap;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;
import javax.swing.text.Keymap;

/**
 * DOCUMENT ME! $Id: ElanFrame2.java 23731 2011-04-27 15:26:06Z hasloe $
 *
 * @author $Author$
 * @version $Revision$
 */
public class ElanFrame2 extends JFrame implements ActionListener,
    ElanLocaleListener, FrameConstants, PreferencesUser {
	/** a list of actions for menu containers and a few transcription independent 
	 * actions */
    private ArrayList menuActions = new ArrayList(20);
    /** a map for actions that are added to the menubar's action map
     * this storage is needed for detaching and re-attaching actions to the menu bar */
    private HashMap registeredActions = new HashMap();
    
    private boolean initialized = false;    
    
    // load some keybindings for the Mac
    static {
        if (System.getProperty("os.name").indexOf("Mac") > -1) {
            JTextComponent.KeyBinding[] bind = new JTextComponent.KeyBinding[] {
                    new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(
                            KeyEvent.VK_C,
                            Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                        DefaultEditorKit.copyAction),
                    new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(
                            KeyEvent.VK_X,
                            Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                        DefaultEditorKit.cutAction),
                    new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(
                            KeyEvent.VK_V,
                            Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                        DefaultEditorKit.pasteAction),
                    new JTextComponent.KeyBinding(KeyStroke.getKeyStroke(
                            KeyEvent.VK_A,
                            Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                        DefaultEditorKit.selectAllAction)
                };

            JTextComponent comp = new JTextField();
            Keymap map = JTextComponent.getKeymap(JTextComponent.DEFAULT_KEYMAP);
            JTextComponent.loadKeymap(map, bind, comp.getActions());
        }
    }

    private Transcription transcriptionForThisFrame;
    private JMenuBar menuBar;
    private JMenu menuFile;
    private JMenuItem menuItemFileNew;
    private JMenuItem menuItemFileOpen;
    private JMenu menuRecentFiles;
    private JMenuItem menuItemFileExit;
    private JMenu menuBackup;
    private JMenu menuImport;
    private JMenu menuExport;
    private JMenu menuExportSMIL;
    private JMenuItem menuItemShoeboxImport;
    private JMenuItem menuItemCHATImport;
    private JMenuItem menuItemTranscriberImport;
    private JMenuItem menuItemDiscoverDoc;
    private JMenuItem menuItemPublishDoc;
    private JMenu menuEdit;
    private JMenu menuPreferences;
    private ElanMenuItem setAuthorMI;
    private JMenu menuAnnotation;
    private JMenu menuTier;
    private JMenu menuType;
    private JMenu menuWindow;
    private ButtonGroup windowsGroup;
    private JMenu menuSearch;
    private JMenu menuView;
    private JMenu menuViewer;
    private JMenu menuOptions;
    private JCheckBoxMenuItem menuItemNativeMedia;
    private JCheckBoxMenuItem menuItemPermanentDetached;
    private JCheckBoxMenuItem menuMacNativeLF;
    private JRadioButtonMenuItem menuItemAnnoMode;
    private JRadioButtonMenuItem menuItemSyncMode;
    private JRadioButtonMenuItem menuItemTranscMode; 
    private JRadioButtonMenuItem menuItemSegmentMode; 
    private JCheckBoxMenuItem menuItemKioskMode;
    private JMenuItem menuItemPlayAround;
    private JMenuItem menuItemRateVol;
    private JMenu menuP2P;
    private JMenu menuHelp;
    private JMenu menuFrameLength;
    private JMenu menuAppLanguage;
    private ButtonGroup languageBG;
    private JMenu menuChangeTimePropMode;
    private JMenuItem menuItemOptionsEnglish;
    private JMenuItem menuItemOptionsDutch;
    private JMenuItem menuItemScrubTrans;
    private JMenu menuMediaPlayer;
    private ElanLayoutManager layoutManager;
    private ViewerManager2 viewerManager;
    private ElanP2P elanP2P;
    private PlayerViewerMenuManager pvMenuManager;
    private boolean fullyInitialized = false;
	private ElanMenuItem closeMI;
	private ElanMenuItem saveMI;
	private ElanMenuItem saveAsMI;
	private ElanMenuItem saveAsTemplateMI;
	private ElanMenuItem saveSelEafMI;
	private ElanMenuItem mergeTransMI;
	private ElanMenuItem pageSetUpMI;
	private ElanMenuItem printPreviewMI;
	private ElanMenuItem printMI;
	private ElanMenuItem undoMI;
	private ElanMenuItem redoMI;
	private ElanMenuItem editCVMI;
	private ElanMenuItem linkedFilesMI;
	private ElanMenuItem importPrefsMI;
	private ElanMenuItem exportPrefsMI;
	private ElanMenuItem searchMI;
	private ElanMenuItem goToMI;
	private ElanMenuItem tierDependenciesMI;
	private ElanMenuItem spreadsheetMI;
	private ElanMenuItem statisticsMI;
	private ElanMenuItem importPraatMI;
	private ElanMenuItem importRecogTiersMI;
	private ElanMenuItem editLexiconServiceMI;
	
	private JCheckBoxMenuItem menuItemGridViewer;
	private JCheckBoxMenuItem menuItemTextViewer;
	private JCheckBoxMenuItem menuItemSubtitleViewer;
	private JCheckBoxMenuItem menuItemLexiconViewer;
	private JCheckBoxMenuItem menuItemAudioRecognizer;
	private JCheckBoxMenuItem menuItemVideoRecognizer;
	private JCheckBoxMenuItem menuItemMetaDataViewer;
	private JCheckBoxMenuItem menuItemSignalViewer;
	private JCheckBoxMenuItem menuItemInterLinearViewer;

	
    /**
     * The no arg constructor creates an empty elan frame containing a menubar
     * with a limited set of menu items.
     */
    public ElanFrame2() {
    	/* disabled for the time being
        String p2p = System.getProperty("P2P");

        if ((p2p != null) && p2p.equals("true")) {
            elanP2P = new ElanP2P(this);
        }
        */
        // set the initial title
        setTitle("ELAN");
        ImageIcon icon = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/ELAN16.png"));
        if (icon != null) {
            setIconImage(icon.getImage());
        } else {
            setIconImage(null);    
        }
        
        initFrame();
    }

    /**
     * Constructor that accepts the path to an .eaf file. This first creates an
     * empty frame and then calls openEAF to open the specified .eaf file.
     *
     * @param path the location of the eaf file
     *
     * @see #openEAF(String)
     */
    public ElanFrame2(final String path) {
        this();
        
        if (path != null) {
	        SwingUtilities.invokeLater(new Runnable() {
	                public void run() {
	                    openEAF(path);
	                }
	            });
        }
    }

    /**
     * Creates a new ElanFrame2 instance
     *
     * @param eafPath DOCUMENT ME!
     * @param mediaFiles DOCUMENT ME!
     */
    public ElanFrame2(final String eafPath, final Vector mediaFiles) {
        this();
        
        if (eafPath != null) {
	        SwingUtilities.invokeLater(new Runnable() {
	                public void run() {
	                    openEAF(eafPath, mediaFiles);
	                }
	            });
        }
    }

    /**
     * Constructor that accepts a Transcription object to use for the current
     * ElanFrame.
     *
     * @param transcription the trancription
     */
    public ElanFrame2(Transcription transcription) {
        this();
        transcriptionForThisFrame = transcription;

        // fill the frame if transcription != null
        if (transcriptionForThisFrame != null) {
            try {
                //new InitThread(transcriptionForThisFrame.getName()).start();
                initElan();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Open an .eaf file defined by a full path string
     * Nov 2006: changed access from private to package
     * @param fullPath
     * @param mediaFiles a vector containing associated media files
     */
    void openEAF(String fullPath, Vector mediaFiles) {
        File fileTemp = new File(fullPath);

        //check if file exists and is a file
        if (!fileTemp.exists() || fileTemp.isDirectory()) {
            String strMessage = ElanLocale.getString("Menu.Dialog.Message1");
            strMessage += fullPath;
            strMessage += ElanLocale.getString("Menu.Dialog.Message2");

            String strError = ElanLocale.getString("Message.Error");
            JOptionPane.showMessageDialog(this, strMessage, strError,
                JOptionPane.ERROR_MESSAGE);

            return;
        }

        //check if file is a '.eaf' file
        String lowerPath = fileTemp.toString().toLowerCase();
        if (!lowerPath.endsWith(".eaf") && !lowerPath.endsWith(".etf")) {
            String strMessage = ElanLocale.getString("Menu.Dialog.Message1");
            strMessage += fullPath;
            strMessage += ElanLocale.getString("Menu.Dialog.Message3");

            String strError = ElanLocale.getString("Message.Error");
            JOptionPane.showMessageDialog(this, strMessage, strError,
                JOptionPane.ERROR_MESSAGE);

            return;
        }

        //open the eaf/etf and get the Transcription from it
        try {
            // When config files are possible check if eaf or configuration file
            //           String path = chooser.getSelectedFile().getAbsolutePath();
            //String path = fullPath;
			String path = fileTemp.getAbsolutePath();
            // replace all backslashes by forward slashes
            path = path.replace('\\', '/');

            //long before = System.currentTimeMillis();
            TranscriptionImpl transcription = new TranscriptionImpl(new File(path).getAbsolutePath());
            //long after = System.currentTimeMillis();
            //System.out.println("open eaf took " + (after - before) + " ms");
            transcription.setUnchanged();
            
	        if (lowerPath.endsWith(".etf")) {
                transcription.setName(TranscriptionImpl.UNDEFINED_FILE_NAME);

                // HS nov 2006: set the pathname to undefined too instead of
                // the path to the template
                transcription.setPathName(TranscriptionImpl.UNDEFINED_FILE_NAME);
                transcription.setChanged();
                // load template preferences
            	String prefPath = path.substring(0, path.length() - 3) + "pfsx";
            	try {
            		File pFile = new File(prefPath);
            		if (pFile.exists()) {
            			Preferences.importPreferences(transcription, prefPath);
            		}
            	} catch (Exception ex) {// catch any exception and continue
            		
            	}
            }

            // in case of eaf and etf files load external controlled vocabularies
            if (transcription.getControlledVocabularies().size() > 0) {
            	new TranscriptionECVLoader().loadExternalCVs(transcription, this);// this can set the changed flag
            }
            // hier.. load a set of lexicon client factories? or delay until needed?
            
            if (mediaFiles != null) {
                //Vector descriptors = createMediaDescriptors(mediaFiles);
                Vector descriptors = MediaDescriptorUtil.createMediaDescriptors(mediaFiles);

                // improve this; check and compare with the medianames from the eaf
                transcription.setMediaDescriptors(descriptors);
                transcription.setChanged();
            }

            int lastSlash = path.lastIndexOf('/');
            String eafPath = path.substring(0, lastSlash);
            boolean validMedia = checkMedia(transcription, eafPath);

            if (!validMedia) {
                // ask if incomplete media session is ok, if not return
                int answer = JOptionPane.showConfirmDialog(this,
                        ElanLocale.getString(
                            "Frame.ElanFrame.IncompleteMediaQuestion"),
                        ElanLocale.getString(
                            "Frame.ElanFrame.IncompleteMediaAvailable"),
                        JOptionPane.YES_NO_OPTION);

                if (answer != JOptionPane.YES_OPTION) {
                    return;
                }
            }

            if (transcriptionForThisFrame != null) {
                // create a new ElanFrame for the Transcription
                //new ElanFrame2(transcription);
            	FrameManager.getInstance().createFrame(transcription);
            } else {
                transcriptionForThisFrame = transcription;

                //new InitThread(transcriptionForThisFrame.getName()).start();
                initElan();
                
                FrameManager.getInstance().updateFrameTitle(this, 
                		transcription.getFullPath());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param fullPath DOCUMENT ME!
     */
    public void openEAF(String fullPath) {
        openEAF(fullPath, null);
    }
    
    /**
     * Sets the transcription (document) for this frame. Only effective if this 
     * frame is an empty frame, if there already is an transcription this 
     * method simply returns.
     * 
     * @param transcription the transcription object
     */
    void setTranscription(Transcription transcription) {
    	if (transcriptionForThisFrame != null || 
    			!(transcription instanceof TranscriptionImpl)) {
    		return;
    	}
    	transcriptionForThisFrame = transcription;
    	
    	initElan();
    }

    /**
     * Returns the viewermanager for this frame
     *
     * @return returns the viewer manager
     */
    public ViewerManager2 getViewerManager() {
        return viewerManager;
    }
    
    /**
     * Returns the Elan layout manager.
     * 
     * @return the layout manager
     */
    public ElanLayoutManager getLayoutManager() {
    	return layoutManager;
    }

    /**
     * Check for existence of media files and fix URLs if needed and possible.
     * The directory from which the .eaf file came is used as an alternative
     * location for the media file if the absolute path does not exist. When
     * the file is not found the user is prompted with a file chooser to
     * locate the file.
     *
     * @param transcription
     * @param eafPath DOCUMENT ME!
     *
     * @return boolean that flags if the media descriptors are valid
     */
    public boolean checkMedia(Transcription transcription, String eafPath) {
        boolean validMedia = true;
        boolean saveChangedMedia = true;
        
        Object val = Preferences.get("MediaLocation.AltLocationSetsChanged", null);
        
        if (val instanceof Boolean) {
        	saveChangedMedia = ((Boolean) val).booleanValue();
        }
        
        File currentDirectory = null;
        // make sure the eaf path is treated the same way as media files,
        // i.e. it starts with file:/// or file://
        String fullEAFURL = FileUtility.pathToURLString(transcription.getFullPath());

        try {
            Vector mediaDescriptors = transcription.getMediaDescriptors();

            for (int i = 0; i < mediaDescriptors.size(); i++) {
                MediaDescriptor md = (MediaDescriptor) mediaDescriptors.elementAt(i);

                // remove the file: part of the URL, leading slashes are no problem
                int colonPos = md.mediaURL.indexOf(':');
                // wwj: if the url begins with rtsp, bypass the file checking
                if (colonPos > 0) {
                	    String urlhead = md.mediaURL.substring(0,colonPos);
                	    if ( urlhead.trim().equalsIgnoreCase("rtsp")) {
                		   continue; 
                	    }
                 }
                String fileName = md.mediaURL.substring(colonPos + 1);

                // replace all back slashes by forward slashes
                fileName = fileName.replace('\\', '/');

                File file = new File(fileName);

                if (!file.exists()) {
                    // look for the file in the local directory
                    int lastSlashPos = fileName.lastIndexOf('/');
                    String localFileName = fileName.substring(lastSlashPos + 1);
                    file = new File(eafPath + "/" + localFileName);

                    if (file.exists()) {
                        // adjust urls, check the user setting whether this should set the changed flag
                        adjustMediaDescriptors(mediaDescriptors, i,
                            file.getAbsolutePath());
                        if (saveChangedMedia) {
                        	transcription.setChanged();
                        }

                        continue;
                    }
                    
                    // look in the relative path stored in the mediadescriptor
                    if (md.relativeMediaURL != null) {
                    	String relUrl = md.relativeMediaURL;
                    	if (relUrl.startsWith("file:/")) {
                    		relUrl = relUrl.substring(6);
                    	}
                    	// resolve relative url and check location
                    	String absPath = FileUtility.getAbsolutePath(fullEAFURL, relUrl);
                    	if (absPath != null) {
                    		file = new File(absPath);
                    		if (file.exists()) {
                                adjustMediaDescriptors(mediaDescriptors, i,
                                    file.getAbsolutePath());
                                if (saveChangedMedia) {
                                	transcription.setChanged();
                                }
                                continue;
                    		}
                    	}
                    }
                    
                    // look in a relative path ../Media
                    file = new File(eafPath + "/../Media/" + localFileName);

                    if (file.exists()) {
                        // adjust urls
                        adjustMediaDescriptors(mediaDescriptors, i,
                            file.getAbsolutePath());
                        if (saveChangedMedia) {
                        	transcription.setChanged();
                        }

                        continue;
                    }

                    // look in a relative path ../media
                    file = new File(eafPath + "/../media/" + localFileName);

                    if (file.exists()) {
                        // adjust urls
                        adjustMediaDescriptors(mediaDescriptors, i,
                            file.getAbsolutePath());
                        if (saveChangedMedia) {
                        	transcription.setChanged();
                        }
                        continue;
                    }
                    // Dec 2008 check a user definable preferred location
                    val = Preferences.get("DefaultMediaLocation", null);
                    
                    if (val instanceof String) {
                    	file = new File(FileUtility.urlToAbsPath((String)val) + "/" + localFileName);
                    	
                        if (file.exists()) {
                            // adjust urls
                            adjustMediaDescriptors(mediaDescriptors, i,
                                file.getAbsolutePath());
                            if (saveChangedMedia) {
                            	transcription.setChanged();
                            }

                            continue;
                        }
                    }
                    
                    // no fallback worked, prompt the user to locate the file
                    JFileChooser chooser = new JFileChooser();

                    if (md.mimeType.equals(MediaDescriptor.WAV_MIME_TYPE)) {
                        chooser.setFileFilter(ElanFileFilter.createFileFilter(
                                ElanFileFilter.WAV_TYPE));
                    } else if (md.mimeType.equals(MediaDescriptor.MPG_MIME_TYPE)) {
                        chooser.setFileFilter(ElanFileFilter.createFileFilter(
                                ElanFileFilter.MPEG_TYPE));
                    } else {
            			   FileFilter mediaFilter = ElanFileFilter.createFileFilter(
            			           ElanFileFilter.MEDIA_TYPE);
            			   chooser.addChoosableFileFilter(mediaFilter);
            			   chooser.addChoosableFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.MP4_TYPE));
            			   chooser.addChoosableFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.QT_TYPE));
            			   chooser.setFileFilter(mediaFilter);
                    }

                    chooser.setDialogTitle(ElanLocale.getString(
                            "Frame.ElanFrame.LocateMedia") + ": " + localFileName);

                    boolean found = false;

                    while (!found) {
                        if (currentDirectory != null) {
                            chooser.setCurrentDirectory(currentDirectory);
                        }

                        chooser.setSelectedFile(new File(file.getName()));

                        int returnVal = chooser.showOpenDialog(this);

                        if (returnVal == JFileChooser.APPROVE_OPTION) {
                            currentDirectory = chooser.getCurrentDirectory();

                            //String name = chooser.getSelectedFile().getName();

                            //if (name.equals(localFileName)) {
                            if (!chooser.getSelectedFile().exists()) {
                                JOptionPane.showMessageDialog(this,
                                    ElanLocale.getString(
                                        "Frame.ElanFrame.LocateMedia"),
                                    ElanLocale.getString("Message.Error"),
                                    JOptionPane.ERROR_MESSAGE);
                                found = false;

                                continue;
                            }

                            // adjust urls
                            adjustMediaDescriptors(mediaDescriptors, i,
                                chooser.getSelectedFile().getAbsolutePath());
                            //if (saveChangedMedia) {// in this case prompt for save
                            	transcription.setChanged();
                            //}

                            found = true;

                            /*} else {
                               // we do not allow another media file name than the original
                               JOptionPane.showMessageDialog(this,
                                   ElanLocale.getString(
                                       "Frame.ElanFrame.MediaFileRenamed"),
                                   ElanLocale.getString("Message.Error"),
                                   JOptionPane.ERROR_MESSAGE);
                               }
                             */
                        } else {
                            // the user did choose cancel and thereby gave up locating the file
                            md.isValid = false;
                            validMedia = false;

                            break;
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return validMedia;
    }

    /**
     * Replace the current path in a media descriptor and all the media
     * descriptors that are derived from it
     *
     * @param mediaDescriptors
     * @param index position in the Vector of descriptors of the descriptor
     *        which media URL must be changed
     * @param newPath the new file path that must become the new media URL
     */
    private void adjustMediaDescriptors(Vector mediaDescriptors, int index,
        String newPath) {
        // remember the old URL
        String oldURL = ((MediaDescriptor) mediaDescriptors.elementAt(index)).mediaURL;

        //String newURL = pathToURLString(newPath);
        String newURL = FileUtility.pathToURLString(newPath);
        String newExt = null;

        if (newURL.indexOf('.') > -1) {
            newExt = newURL.substring(newURL.lastIndexOf('.') + 1);
        }

        // replace the old URL and mime type
        ((MediaDescriptor) mediaDescriptors.elementAt(index)).mediaURL = newURL;
        ((MediaDescriptor) mediaDescriptors.elementAt(index)).mimeType = MediaDescriptorUtil.mimeTypeForExtension(newExt);

        // replace the extracted from URL's
        for (int i = 0; i < mediaDescriptors.size(); i++) {
            String extractedFrom = ((MediaDescriptor) mediaDescriptors.elementAt(i)).extractedFrom;

            if (oldURL.equals(extractedFrom)) {
                ((MediaDescriptor) mediaDescriptors.elementAt(i)).extractedFrom = newURL;
            }
        }
    }

    /**
     * Init Elan for a Transcription mediaDecriptors should be contained in
     * Transcription Is this the place to create all the viewers, must there
     * be getters for these viewers who needs to know about them?
     */
    private void initElan() {
        setTitle("Initializing....");
        // before creating viewers apply preferences to controlled vocabularies
        loadCVPreferences();   
        
        //set viewer preferences        
        setPreference(ELANCommandFactory.GRID_VIEWER, menuItemGridViewer.isSelected(), null);
        setPreference(ELANCommandFactory.TEXT_VIEWER, menuItemTextViewer.isSelected(), null);
        setPreference(ELANCommandFactory.SUBTITLE_VIEWER, menuItemSubtitleViewer.isSelected(), null);
        setPreference(ELANCommandFactory.LEXICON_VIEWER, menuItemLexiconViewer.isSelected(), null);
        setPreference(ELANCommandFactory.AUDIO_RECOGNIZER, menuItemAudioRecognizer.isSelected(), null);
        setPreference(ELANCommandFactory.VIDEO_RECOGNIZER, menuItemVideoRecognizer.isSelected(), null);
        setPreference(ELANCommandFactory.METADATA_VIEWER, menuItemMetaDataViewer.isSelected(), null);
        setPreference(ELANCommandFactory.INTERLINEAR_VIEWER, menuItemInterLinearViewer.isSelected(), null);
        setPreference(ELANCommandFactory.SIGNAL_VIEWER, menuItemSignalViewer.isSelected(), null); 
        
        viewerManager = new ViewerManager2(transcriptionForThisFrame);  
        layoutManager = new ElanLayoutManager(this, viewerManager);
        
        ELANCommandFactory.addDocument(this, viewerManager, layoutManager); 
        pvMenuManager = new PlayerViewerMenuManager(this, transcriptionForThisFrame);
        //long time = System.currentTimeMillis();
        //System.out.println("B: " + 0);
        //MediaDescriptorUtil.createMediaPlayers((TranscriptionImpl) transcriptionForThisFrame,
        //    transcriptionForThisFrame.getMediaDescriptors());
        MediaDescriptorUtil.createMediaPlayers((TranscriptionImpl) transcriptionForThisFrame,
        		pvMenuManager.getStoredVisiblePlayers());

        // if there is a signal viewer its mediafile is the first in the list       
        ArrayList<String> audioPaths = new ArrayList<String>(4);   
       	if (layoutManager.getSignalViewer() != null) {
       		audioPaths.add(layoutManager.getSignalViewer().getMediaPath());
       	}        	
      
        if(!menuItemSignalViewer.isSelected()){
        	if (layoutManager.getSignalViewer() != null) {
        		layoutManager.remove(layoutManager.getSignalViewer());        		
        	} 
        }
        
        ArrayList<String> videoPaths = new ArrayList<String>(6);
        // there may be other audio files associated with the transcription
        MediaDescriptor md;
        String path;

        for (int i = 0; i < transcriptionForThisFrame.getMediaDescriptors().size(); i++) {
        	md = (MediaDescriptor) transcriptionForThisFrame.getMediaDescriptors().get(i);
    		path = md.mediaURL;
    		if (path.startsWith("file:")) {
    			path = path.substring(5);
    		}
        	if (!md.mimeType.equals(MediaDescriptor.GENERIC_AUDIO_TYPE) && !md.mimeType.equals(MediaDescriptor.WAV_MIME_TYPE)) {

        		if (!videoPaths.contains(path)) {
        		    videoPaths.add(path);
        		}
        	}
        	// audio detectors may work with video...
    		if (!audioPaths.contains(path)) {
    		    audioPaths.add(path);
    		}
        }
        
        viewerManager.setAudioPaths(audioPaths); 
        viewerManager.setVideoPaths(videoPaths);    	
    	
        if (transcriptionForThisFrame.getLinkedFileDescriptors().size() > 0) {      
        	LinkedFileDescriptorUtil.initLinkedFiles(
        		(	TranscriptionImpl) transcriptionForThisFrame);
        }
        
//        //		layoutManager.add(new mpi.eudico.p2p.CollaborationPanel());
//        if (elanP2P != null) {
//            elanP2P.setManagers(viewerManager, layoutManager);
//        }

//        // temp block for svg
//        if (SVGPrefs.getUseSVG() ||
//                (((TranscriptionImpl) transcriptionForThisFrame).getSVGFile() != null)) {
//            // lightweight svgviewer
//	
//            if (viewerManager.getMasterMediaPlayer() instanceof JMFGraphicMediaPlayer) {
//                // don't need to add to layout
//                menuItemFileOpen.setEnabled(false);
//                menuItemFileNew.setEnabled(false);
//                menuImport.setEnabled(false);
//            }
//        }

        ElanLocale.addElanLocaleListener(transcriptionForThisFrame, this);
        
        setFrameTitle();
        
        initMenusAndCommands();
        
        pvMenuManager.initPlayerMenu();

        if (viewerManager.getMasterMediaPlayer().isFrameRateAutoDetected()) {
            // disable the menu to change the video standard, i.e. the frame length
            setMenuEnabled(FrameConstants.FRAME_LENGTH, false);
        }       
        
		int mode = ElanLayoutManager.NORMAL_MODE;
		Integer lastMode = (Integer) Preferences.get("LayoutManager.CurrentMode", viewerManager.getTranscription()); 
		if(lastMode != null && mode != lastMode.intValue()){
			mode = lastMode;
		}
		layoutManager.changeMode(mode);
		
		Preferences.addPreferencesListener(transcriptionForThisFrame, layoutManager);
	    Preferences.addPreferencesListener(transcriptionForThisFrame, this);
	    Preferences.notifyListeners(transcriptionForThisFrame);
		
        // a few prefs to load only on load
        loadPreferences();

        // instantiate Viewer components:
        //  use CommandActions for relevant toolbar buttons and combobox menu items
        //  position and size components using LayoutManager
        layoutManager.doLayout();

        //  this sucks but is needed to ensure visible video on the mac
        //viewerManager.getMasterMediaPlayer().setMediaTime(0);
        initialized = true;
    }

    private void setFrameTitle() {
        try {
            if (transcriptionForThisFrame != null) {
                if (transcriptionForThisFrame.getName().equals(TranscriptionImpl.UNDEFINED_FILE_NAME)) {
                    setTitle("ELAN - " +
                        ElanLocale.getString(
                            "Frame.ElanFrame.UndefinedFileName"));
                } else {
                    setTitle("ELAN - " + transcriptionForThisFrame.getName());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Performs some general initialization for an empty frame.
     */
    private void initFrame() {
    	//getRootPane().putClientProperty("Window.alpha", new Float(0.50));// on MacOS a window can be (semi) transparent
        Locale savedLocale = (Locale) Preferences.get("Locale", null);

        if (savedLocale != null) {
            ElanLocale.setLocale(savedLocale);
        }
        // create the initial menu items
        initMenuBar();

        // listen for WindowEvents events on the ElanFrame
        addWindowListener(new ElanFrameWindowListener());

        // require the program to handle the operation in the
        // windowClosing method of a registered WindowListener object.
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // add this elanframe to locale listener
        //ElanLocale.addElanLocaleListener(this);
        pack();

        /*
           Dimension dimSize = new Dimension(800, 650);
           setSize((int)dimSize.getWidth(), (int)dimSize.getHeight());
           Dimension dimLocation = Toolkit.getDefaultToolkit().getScreenSize();
           setLocation((int)(dimLocation.getWidth()/2-getWidth()/2),(int)(dimLocation.getHeight()/2-getHeight()/2));
         */
        Dimension d = (Dimension) Preferences.get("FrameSize", null);

//        if (d != null) {
//            setSize(d);
//        } else {
//            Dimension dimSize = new Dimension(800, 650);
//            setSize((int) dimSize.getWidth(), (int) dimSize.getHeight());
//        }
//
//        Point p = (Point) Preferences.get("FrameLocation", null);
//
//        if (p != null) {
//            setLocation(p);
//        } else {
//            Dimension dimLocation = Toolkit.getDefaultToolkit().getScreenSize();
//            setLocation((int) ((dimLocation.getWidth() / 2) - (getWidth() / 2)),
//                (int) ((dimLocation.getHeight() / 2) - (getHeight() / 2)));
//        }
        // HS June 2010 correct dimension and location if they don't fit on the screen?
        // or make this a user preference?
        Rectangle wRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
        
        if (d != null) {      	
        	if (d.width > wRect.width) {
        		d.setSize(wRect.width, d.height);
        	}
        	if (d.height > wRect.getHeight()) {
        		d.setSize(d.width, wRect.height);
        	}
            setSize(d);
        } else {
        	setSize(800, 600);
        }
        Point p = (Point) Preferences.get("FrameLocation", null);

        if (p != null) {
        	int mpx = wRect.width - getWidth();
        	if (p.x > mpx && mpx > 0) {
        		p.x = mpx;
        	}
        	int mph = wRect.height - getHeight();
        	if (p.y > mph && mph > 0) {
        		p.y = mph;
        	}
            setLocation(p);
        } else {
        	setLocation((int) ((wRect.getWidth() / 2) - (getWidth() / 2)), 
        			(int) ((wRect.getHeight() / 2) - (getHeight() / 2)));
        }
        // the call to setVisible should be moved to (a new thread created in) 
        // the class that called a constructor....
        setVisible(true);
    }

    private void initMenuBar() {
        menuBar = new JMenuBar();

        setJMenuBar(menuBar);

        //make menu visible / appear above heavyweight video
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        // HS 07-2009 make a complete menu layout except for Annotation, Tier and Type menu.
        // add Actions for most menu items later in initMenusAndCommands()
        MenuAction ma;
        ma = new MenuAction("Menu.File");
        menuFile = new JMenu(ma);
        menuActions.add(ma);
        menuBar.add(menuFile);

        ma = new NewMA(ELANCommandFactory.NEW_DOC, this);
        menuActions.add(ma);
        menuItemFileNew = new JMenuItem(ma);
        menuFile.add(menuItemFileNew);

        ma = new OpenMA(ELANCommandFactory.OPEN_DOC, this);
        menuActions.add(ma);
        menuItemFileOpen = new JMenuItem(ma);
        menuFile.add(menuItemFileOpen);

        ma = new MenuAction("Menu.File.OpenRecent");
        menuActions.add(ma);
        menuRecentFiles = new JMenu(ma);
        menuFile.add(menuRecentFiles);
        
        closeMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.CLOSE), false);
        menuFile.add(closeMI);
		menuFile.addSeparator();
       
		saveMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.SAVE), false);
		menuFile.add(saveMI);
		
		saveAsMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.SAVE_AS), false);
        menuFile.add(saveAsMI);
        
        saveAsTemplateMI = new ElanMenuItem(ElanLocale.getString(
                ELANCommandFactory.SAVE_AS_TEMPLATE), false);
		menuFile.add(saveAsTemplateMI);
		
		saveSelEafMI = new ElanMenuItem(ElanLocale.getString(
                ELANCommandFactory.SAVE_SELECTION_AS_EAF), false);
        menuFile.add(saveSelEafMI);
        
        ma = new MergeTranscriptionsMA(ELANCommandFactory.MERGE_TRANSCRIPTIONS, this);
        menuActions.add(ma);
        mergeTransMI = new ElanMenuItem(ma);
		menuFile.add(mergeTransMI);
		
        ma = new MenuAction("Menu.File.Backup.Auto");
        menuActions.add(ma);
        menuBackup = new JMenu(ma);
        menuBackup.setEnabled(false);
        menuFile.add(menuBackup);
        menuFile.addSeparator();
        
        pageSetUpMI = new ElanMenuItem(ElanLocale.getString(
        		ELANCommandFactory.PAGESETUP), false);
		menuFile.add(pageSetUpMI);
		printPreviewMI = new ElanMenuItem(ElanLocale.getString(
				ELANCommandFactory.PREVIEW), false);
		menuFile.add(printPreviewMI);
		printMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.PRINT), false);
		menuFile.add(printMI);
		menuFile.addSeparator();
		
		ma = new MenuAction("Menu.File.ProcessMulti");
		menuActions.add(ma);
		JMenu mfProcessMenu = new JMenu(ma);
		menuFile.add(mfProcessMenu);
		ma = new MultipleFilesEditMA("Menu.File.Process.EditMF", this);
		menuActions.add(ma);
		mfProcessMenu.add(new JMenuItem(ma));
        
        ma = new MultiEAFScrubberMA("Menu.File.ScrubTranscriptions", this);
        menuActions.add(ma);
        menuItemScrubTrans = new JMenuItem(ma);
        mfProcessMenu.add(menuItemScrubTrans);
        
        ma = new MultiFileAnnotationsFromOverlapsMA("Menu.File.MultipleFileAnnotationFromOverlaps", this);
        menuActions.add(ma);
        JMenuItem menuItemMultipleFileAnnotationsFromOverlaps = new JMenuItem(ma);
        mfProcessMenu.add(menuItemMultipleFileAnnotationsFromOverlaps);
        
        ma = new StatisticsMultipleFilesMA("Menu.File.MultiFileStatistics", this);
        menuActions.add(ma);
        mfProcessMenu.add(new JMenuItem(ma));
        
		menuFile.addSeparator();
		
        ma = new MenuAction("Menu.File.Export");
        menuActions.add(ma);
        menuExport = new JMenu(ma);
        menuExport.setEnabled(false);
        menuFile.add(menuExport);
        
        ma = new MenuAction("Menu.File.Export.MultipleFiles");
        menuActions.add(ma);
        JMenu exportMenuMulti = new JMenu(ma);
        menuFile.add(exportMenuMulti);
        
        ma = new ExportTabMultiMA(ELANCommandFactory.EXPORT_TAB_MULTI, this);
        menuActions.add(ma);
        exportMenuMulti.add(new JMenuItem(ma));
        
        ma = new ExportAnnotationsMultiMA(ELANCommandFactory.EXPORT_ANNLIST_MULTI, this);
        menuActions.add(ma);
        exportMenuMulti.add(new JMenuItem(ma));
        
        ma = new ExportWordsMultiMA(ELANCommandFactory.EXPORT_WORDLIST_MULTI, this);
        menuActions.add(ma);
        exportMenuMulti.add(new JMenuItem(ma));
        
        ma = new ExportTiersMA(ELANCommandFactory.EXPORT_TIERS_MULTI, this);
        menuActions.add(ma);
        exportMenuMulti.add(new JMenuItem(ma));
        
        ma = new MenuAction("Menu.File.Import");
        menuActions.add(ma);
        menuImport = new JMenu(ma);
        menuFile.add(menuImport);

        ma = new ImportShoeboxMA(ELANCommandFactory.IMPORT_SHOEBOX, this);
        menuActions.add(ma);
        menuItemShoeboxImport = new JMenuItem(ma);
        menuImport.add(menuItemShoeboxImport);
        
        ma = new ImportToolboxMA(ELANCommandFactory.IMPORT_TOOLBOX, this);
        menuActions.add(ma);
        JMenuItem toolboxImportMI = new JMenuItem(ma);
        menuImport.add(toolboxImportMI);

        ma = new ImportFlexMA(ELANCommandFactory.IMPORT_FLEX, this);
        menuActions.add(ma);
        JMenuItem flexImportMI = new JMenuItem(ma);
        menuImport.add(flexImportMI);
        
        ma = new ImportCHATMA(ELANCommandFactory.IMPORT_CHAT, this);
        menuActions.add(ma);
        menuItemCHATImport = new JMenuItem(ma);
        menuImport.add(menuItemCHATImport);

        ma = new ImportTranscriberMA(ELANCommandFactory.IMPORT_TRANS, this);
        menuActions.add(ma);
        menuItemTranscriberImport = new JMenuItem(ma);
        menuImport.add(menuItemTranscriberImport);

        ma = new ImportDelimitedTextMA(ELANCommandFactory.IMPORT_TAB, this);
        menuActions.add(ma);
        menuImport.add(new JMenuItem(ma));
        
        ma = new ImportPraatMA(ELANCommandFactory.IMPORT_PRAAT_GRID, this);
        importPraatMI = new ElanMenuItem(ma);
		menuActions.add(ma);
        menuImport.add(importPraatMI);
        
        ma = new ImportRecognizerTiersMA(ELANCommandFactory.IMPORT_RECOG_TIERS, this);
        importRecogTiersMI = new ElanMenuItem(ma);
		menuActions.add(ma);
		menuImport.add(importRecogTiersMI);
		
        menuFile.addSeparator();

        ma = new ExitMA(ELANCommandFactory.EXIT);
        menuActions.add(ma);
        menuItemFileExit = new JMenuItem(ma);
        menuFile.add(menuItemFileExit);

        ma = new MenuAction("Menu.Edit");
        menuActions.add(ma);
        menuEdit = new JMenu(ma);
        menuBar.add(menuEdit); 
 
        undoMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.UNDO), false);
        menuEdit.add(undoMI);
        redoMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.REDO), false);
		menuEdit.add(redoMI);
        menuEdit.addSeparator();
        editCVMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.EDIT_CV_DLG), false);
		menuEdit.add(editCVMI);
		// For opening a Edit Lexicon Service Dialog: 
		editLexiconServiceMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.EDIT_LEX_SRVC_DLG), false);
		menuEdit.add(editLexiconServiceMI);
		setAuthorMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.SET_AUTHOR), false);
		menuEdit.add(setAuthorMI);
		menuEdit.addSeparator();
		linkedFilesMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.LINKED_FILES_DLG), false);
		menuEdit.add(linkedFilesMI);
        menuEdit.addSeparator();
        
        ma = new MenuAction("Menu.Edit.Preferences");
        menuActions.add(ma);
        menuPreferences = new JMenu(ma);
        menuEdit.add(menuPreferences);
        ma = new EditPreferencesMA(ELANCommandFactory.EDIT_PREFS, this);
        menuActions.add(ma);
        menuPreferences.add(new JMenuItem(ma));
        ma = new EditShortcutsMA("Menu.Edit.Preferences.Shortcut", this);
        menuActions.add(ma);
        menuPreferences.add(new JMenuItem(ma));
        menuPreferences.addSeparator();
        importPrefsMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.IMPORT_PREFS), false);
        menuPreferences.add(importPrefsMI);
        exportPrefsMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.EXPORT_PREFS), false);
		menuPreferences.add(exportPrefsMI);

		ma = new MenuAction("Menu.Annotation");
        menuActions.add(ma);
        menuAnnotation = new JMenu(ma);
        menuBar.add(menuAnnotation);
        menuAnnotation.setEnabled(false);
        
        ma = new MenuAction("Menu.Tier");
        menuActions.add(ma);
        menuTier = new JMenu(ma);
        menuBar.add(menuTier);
        menuTier.setEnabled(false);
        
        ma = new MenuAction("Menu.Type");
        menuActions.add(ma);
        menuType = new JMenu(ma);
        menuBar.add(menuType);
        menuType.setEnabled(false);
        
        ma = new MenuAction("Menu.Search");
        menuActions.add(ma);
        menuSearch = new JMenu(ma);
        menuBar.add(menuSearch);
        //menuSearch.setEnabled(false);
        
        searchMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.SEARCH_DLG), false);
		menuSearch.add(searchMI);
		ma = new MultiFindReplaceMA(ELANCommandFactory.REPLACE_MULTIPLE, this);
		menuActions.add(ma);
		menuSearch.add(ma);
		
		ma = new SearchMultipleMA(ELANCommandFactory.SEARCH_MULTIPLE_DLG, this);
		menuActions.add(ma);
		menuSearch.add(ma);
		
		ma = new StructuredSearchMultipleMA(ELANCommandFactory.STRUCTURED_SEARCH_MULTIPLE_DLG, this);
		menuActions.add(ma);
		menuSearch.add(ma);
		goToMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.GOTO_DLG), false);
		menuSearch.add(goToMI);
	
        ma = new MenuAction("Menu.View");
        menuActions.add(ma);
        menuView = new JMenu(ma);
        menuBar.add(menuView);

        ma = new MenuAction("Menu.View.MediaPlayer");
        menuActions.add(ma);
        menuMediaPlayer = new JMenu(ma);
        menuView.add(menuMediaPlayer);
        
        ma = new MenuAction("Menu.View.Viewers");
        menuActions.add(ma);
        menuViewer = new JMenu(ma);
        menuView.add(menuViewer);
        menuView.addSeparator();
        
        menuItemGridViewer = new JCheckBoxMenuItem(new CreateGridViewerMA(ELANCommandFactory.GRID_VIEWER,this));      
        menuItemGridViewer.setSelected(true);  
        menuViewer.add(menuItemGridViewer);   
        
        
        menuItemTextViewer = new JCheckBoxMenuItem(new CreateTextViewerMA(ELANCommandFactory.TEXT_VIEWER,this));      
        menuItemTextViewer.setSelected(true);
        menuViewer.add(menuItemTextViewer);   
        
        menuItemSubtitleViewer = new JCheckBoxMenuItem(new CreateSubtitleViewerMA(ELANCommandFactory.SUBTITLE_VIEWER,this));      
        menuItemSubtitleViewer.setSelected(true); 
        menuViewer.add(menuItemSubtitleViewer);   
        
        menuItemLexiconViewer = new JCheckBoxMenuItem(new CreateLexiconViewerMA(ELANCommandFactory.LEXICON_VIEWER,this));      
        menuItemLexiconViewer.setSelected(true); 
        menuViewer.add(menuItemLexiconViewer);
        
        menuItemAudioRecognizer = new JCheckBoxMenuItem(new CreateAudioRecognizerMA(ELANCommandFactory.AUDIO_RECOGNIZER,this));  
        menuItemAudioRecognizer.setSelected(true); 
        menuViewer.add(menuItemAudioRecognizer);   
        
        menuItemVideoRecognizer = new JCheckBoxMenuItem(new CreateVideoRecognizerMA(ELANCommandFactory.VIDEO_RECOGNIZER,this));      
        menuItemVideoRecognizer.setSelected(true); 
        menuViewer.add(menuItemVideoRecognizer);         
        
        menuItemMetaDataViewer = new JCheckBoxMenuItem(new CreateMetadataViewerMA(ELANCommandFactory.METADATA_VIEWER,this));      
        menuItemMetaDataViewer.setSelected(true);
        menuViewer.add(menuItemMetaDataViewer); 
        
        menuItemSignalViewer = new JCheckBoxMenuItem(new CreateSignalViewerMA(ELANCommandFactory.SIGNAL_VIEWER,this));      
        menuItemSignalViewer.setSelected(true);        
        menuViewer.add(menuItemSignalViewer); 
        
        menuItemInterLinearViewer = new JCheckBoxMenuItem(new CreateInterlinearViewerMA(ELANCommandFactory.INTERLINEAR_VIEWER,this));      
        menuItemInterLinearViewer.setSelected(true);          
        menuViewer.add(menuItemInterLinearViewer); 
        
        loadViewerPreferences();
        
        tierDependenciesMI = new ElanMenuItem(ElanLocale.getString(
        		ELANCommandFactory.TIER_DEPENDENCIES), false);
		menuView.add(tierDependenciesMI);
		// replace by transcription independent actions
		ma = new ShortcutsMA(ELANCommandFactory.SHORTCUTS);
		menuActions.add(ma);
        menuView.add(ma);
        ma = new FontBrowserMA(ELANCommandFactory.FONT_BROWSER);
        menuActions.add(ma);
        menuView.add(ma);
        
        ma = new ShowLogMA("Menu.View.LogView", this);
        menuActions.add(ma);
        menuView.add(ma);
        
        menuView.addSeparator();
        spreadsheetMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.SPREADSHEET), false);
        menuView.add(spreadsheetMI);
        statisticsMI = new ElanMenuItem(ElanLocale.getString(ELANCommandFactory.STATISTICS), false);
        menuView.add(statisticsMI);
        
        ma = new MenuAction("Menu.Options");
        menuActions.add(ma);
        menuOptions = new JMenu(ma);
        menuBar.add(menuOptions);

        ma = new MenuAction("Menu.Options.TimeChangePropagationMode");
        menuActions.add(ma);
        menuChangeTimePropMode = new JMenu(ma);
        menuChangeTimePropMode.setEnabled(false);
        menuOptions.add(menuChangeTimePropMode);
        menuOptions.addSeparator();
        
        menuItemAnnoMode = new JRadioButtonMenuItem(ElanLocale.getString(ELANCommandFactory.ANNOTATION_MODE));
        menuItemAnnoMode.setEnabled(false);
        menuItemAnnoMode.setSelected(true);
        menuOptions.add(menuItemAnnoMode);

        menuItemSyncMode = new JRadioButtonMenuItem(ElanLocale.getString(ELANCommandFactory.SYNC_MODE));
        menuItemSyncMode.setEnabled(false);
        menuOptions.add(menuItemSyncMode);
        
        menuItemTranscMode = new JRadioButtonMenuItem(ElanLocale.getString(ELANCommandFactory.TRANSCRIPTION_MODE));
        menuItemTranscMode.setEnabled(false);
        menuOptions.add(menuItemTranscMode);
        
        menuItemSegmentMode = new JRadioButtonMenuItem(ElanLocale.getString(ELANCommandFactory.SEGMENTATION_MODE));
        menuItemSegmentMode.setEnabled(false);
        menuOptions.add(menuItemSegmentMode);
        menuOptions.addSeparator();
        
//        menuItemPlayAround = new ElanMenuItem(ElanLocale.getString(
//        		ELANCommandFactory.PLAY_AROUND_SELECTION_DLG), false);
//        menuOptions.add(menuItemPlayAround);
        ma = new SetPlayAroundSelectionMA(ELANCommandFactory.PLAY_AROUND_SELECTION_DLG, this);
        menuActions.add(ma);
        menuOptions.add(new JMenuItem(ma));
        ma = new SetPlaybackToggleMA(ELANCommandFactory.PLAYBACK_TOGGLE_DLG, this);
        menuActions.add(ma);
        menuOptions.add(new JMenuItem(ma));
//        menuItemRateVol = new ElanMenuItem(ElanLocale.getString(
//                ELANCommandFactory.PLAYBACK_TOGGLE_DLG), false);
//        menuOptions.add(menuItemRateVol);

        menuOptions.addSeparator();
        ma = new MenuAction("Menu.Options.FrameLength");
        menuActions.add(ma);
        menuFrameLength = new JMenu(ma);
        menuFrameLength.setEnabled(false);
        menuOptions.add(menuFrameLength);
        
        menuOptions.addSeparator();
        ma = new MenuAction("Menu.Options.Language");
        menuActions.add(ma);
        menuAppLanguage = new JMenu(ma);
        languageBG = new ButtonGroup();
        JRadioButtonMenuItem langRBMI;
        // add languages
        ma = new SetLocaleMA(ELANCommandFactory.CATALAN, this, ElanLocale.CATALAN);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.GERMAN, this, ElanLocale.GERMAN);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.ENGLISH, this, ElanLocale.ENGLISH);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.SPANISH, this, ElanLocale.SPANISH);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.FRENCH, this, ElanLocale.FRENCH);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.JAPANESE, this, ElanLocale.JAPANESE);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.DUTCH, this, ElanLocale.DUTCH);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.PORTUGUESE, this, ElanLocale.PORTUGUESE);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.SWEDISH, this, ElanLocale.SWEDISH);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);
        
        ma = new SetLocaleMA(ELANCommandFactory.CUSTOM_LANG, this, ElanLocale.CUSTOM);
        langRBMI = new JRadioButtonMenuItem(ma);
        languageBG.add(langRBMI);
        menuAppLanguage.add(langRBMI);

        menuOptions.add(menuAppLanguage);
        
        ma = new MenuAction("Menu.Window");
        menuActions.add(ma);
        menuWindow = new JMenu(ma);
        windowsGroup = new ButtonGroup();
        menuBar.add(menuWindow);
        
        ma = new MenuAction("Menu.Help");
        menuActions.add(ma);
        menuHelp = new JMenu(ma);
        menuBar.add(menuHelp);
        //menuHelp.setVisible(false);
        ma = new HelpMA(ELANCommandFactory.HELP, this);
        menuActions.add(ma);
        menuHelp.add(new JMenuItem(ma));
        menuHelp.addSeparator();
        
        ma = new MenuAction("Menu.Help.Website");
        menuActions.add(ma);
        JMenu websiteMenu = new JMenu(ma);
        menuHelp.add(websiteMenu);
        
        ma = new WebMA("Menu.Help.Website.ReleaseNotes", this, "http://www.lat-mpi.eu/tools/elan/release-notes.html");
        menuActions.add(ma);
        websiteMenu.add(new JMenuItem(ma));
        
        ma = new WebMA("Menu.Help.Website.Download", this, "http://www.lat-mpi.eu/tools/elan/download");
        menuActions.add(ma);
        websiteMenu.add(new JMenuItem(ma));
   
        ma = new WebMA("Menu.Help.Website.Forum", this, "http://www.lat-mpi.eu/tools/elan/elanforum/simpleforum_view");
        menuActions.add(ma);
        websiteMenu.add(new JMenuItem(ma));
        
        websiteMenu.addSeparator();
        
        ma = new WebMA("Menu.Help.Website.Subscibe", this, "mailto:majordomo@mpi.nl?body=subscribe%20Elan&amp;subject=subscribe%20to%20Elan");
        menuActions.add(ma);
        websiteMenu.add(new JMenuItem(ma));
        
        ma = new AboutMA(ELANCommandFactory.ABOUT, this);
        menuActions.add(ma);
        menuHelp.add(new JMenuItem(ma));
        
        // temporary and therefore old fashioned menu item to allow fallback to JMF media players
        // on windows machines. As soon as native media is succesfully used in a few releases
        // this menu item can be deleted. The only action performed while toggling this item is
        // setting the PreferredMediaFramework property that is used by the player factory
        /* use either with a -D startup option (-DPreferredMediaFramework=JMF) or add to a preference panel 
        if (System.getProperty("os.name").toLowerCase().indexOf("windows") > -1) {
            menuItemNativeMedia = new JCheckBoxMenuItem();
            menuItemNativeMedia.setText("Use Native Media Platform");
            menuItemNativeMedia.addActionListener(this);

            // haal de state uit de property
            boolean nativePrefered = true;
            String preferredMF = System.getProperty("PreferredMediaFramework");

            // set here default choice if nothing defined
            if (preferredMF == null) {
                System.setProperty("PreferredMediaFramework", "NativeWindows");

                //System.setProperty("PreferredMediaFramework", "JMF");
            } else if (!preferredMF.equals("NativeWindows")) {
                nativePrefered = false;
            }

            menuItemNativeMedia.setState(nativePrefered);

            // optionally suppress jmf support for the standalone version
            String suppressJMF = System.getProperty("suppressJMF");

            if ((suppressJMF == null) ||
                    !suppressJMF.toLowerCase().equals("true")) {
                menuOptions.add(menuItemNativeMedia);
            }
        }
		*/
        if (elanP2P != null) {
            ma = new MenuAction("Menu.P2P");
            menuActions.add(ma);       	
            menuP2P = new JMenu(ma);
            menuBar.add(menuP2P);
            
            menuItemDiscoverDoc = new JMenuItem();
            menuItemDiscoverDoc.addActionListener(this);
            menuItemDiscoverDoc.setActionCommand("DiscoverDoc");
            menuP2P.add(menuItemDiscoverDoc);
        }
        /*
        if (System.getProperty("os.name").startsWith("Mac OS")) {
            // inserted by AR to allow choice for permanent detached video at startup
            menuItemPermanentDetached = new JCheckBoxMenuItem();
            menuItemPermanentDetached.setText("Use detached media window");
            menuItemPermanentDetached.addActionListener(this);

            // haal de state uit de preferences
            Boolean permanentDetached = (Boolean) Preferences.get("PreferredMediaWindow",
                    null);

            if (permanentDetached == null) {
                permanentDetached = new Boolean(false); // default usage is attached media window
            }

            menuItemPermanentDetached.setState(permanentDetached.booleanValue());
            menuOptions.add(menuItemPermanentDetached);

            // end of insertion by AR
        }
        */

        updateLocale();
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        if (command.equals("DiscoverDoc")) {
            if (elanP2P != null) {
                Command c = ELANCommandFactory.createCommand(null,
                        ELANCommandFactory.DISCOVER_DOC);
                c.execute(elanP2P, new Object[] { this });
            }
        } else if (command.equals("PublishDoc")) {
            if (elanP2P != null) {
                Command c = ELANCommandFactory.createCommand(null,
                        ELANCommandFactory.PUBLISH_DOC);
                c.execute(elanP2P,
                    new Object[] { this, transcriptionForThisFrame });
            }
        } else if (command.equals("Use Native Media Platform")) {
            boolean useNativeMedia = menuItemNativeMedia.getState();

            if (useNativeMedia) {
                System.out.println(
                    "Setting preferred media framework to Native");
                System.setProperty("PreferredMediaFramework", "NativeWindows");
            } else {
                System.out.println("Setting preferred media framework to JMF");
                System.setProperty("PreferredMediaFramework", "JMF");
            }
        } /*else if (command.equals("Use detached media window")) {
            Preferences.set("PreferredMediaWindow",
                new Boolean(menuItemPermanentDetached.getState()), null);
        } else if (command.equals("MacNativeLF")) {
            switchMacLF();
        }*/  
    }

    private void initMenusAndCommands() {
        // instantiate CommandActions, also UndoCA and RedoCA

        /* sample:
           CommandAction playSelectionCA = ELANCommandFactory.getCommandAction(ELANCommandFactory.PLAY_SELECTION);
         */

        // instantiate JMenuItems, where possible with CommandActions as args for constructor

        /* sample:
           playSelectionItem = new JMenuItem(playSelectionCA);
           playSelectionItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
           menuInvisible.add(playSelectionItem);   // add to menu, if only for button then to invisible menu
         */

        // add actions with accelerator keys and without a menu item to the input
        // and action map
        InputMap inputMap = menuBar.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        ActionMap actionMap = menuBar.getActionMap();

        if (inputMap instanceof ComponentInputMap && (actionMap != null)) {
            String id = "Act-";
            String nextId;
            Object key;
            List<String> vals;
            int index = 0;
            Action act;
            KeyStroke ks;
            Map<String, KeyStroke> shortMap = ShortcutsUtil.getInstance().getCurrentShortcuts();
            Iterator<String> keyIt = shortMap.keySet().iterator();
            while (keyIt.hasNext()) {
            	act = ELANCommandFactory.getCommandAction(transcriptionForThisFrame, keyIt.next());
            	if (act != null) {
            		ks = (KeyStroke) act.getValue(Action.ACCELERATOR_KEY);
            		if (ks != null) {
            			nextId = id + index++;
            			inputMap.put(ks, nextId);
            			actionMap.put(nextId, act);
            		}
            	}
            }
            // add 2 more actions that are not in a menu and that are not transcription dependent
            act = new NextWindowMA(ELANCommandFactory.NEXT_WINDOW);
            ks = (KeyStroke) act.getValue(Action.ACCELERATOR_KEY);
            nextId = id + index++;
			inputMap.put(ks, nextId);
			actionMap.put(nextId, act);
            act = new PrevWindowMA(ELANCommandFactory.PREV_WINDOW);
            ks = (KeyStroke) act.getValue(Action.ACCELERATOR_KEY);
            nextId = id + index++;
			inputMap.put(ks, nextId);
			actionMap.put(nextId, act);
        }

        MenuAction ma;
        //menuFile.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        //        transcriptionForThisFrame, ELANCommandFactory.CLOSE)), 3);
        closeMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        		ELANCommandFactory.CLOSE), true);
        saveMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SAVE), true);
        saveAsMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SAVE_AS), true);
        saveAsTemplateMI.setAction(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SAVE_AS_TEMPLATE), true);
        saveSelEafMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame,
                ELANCommandFactory.SAVE_SELECTION_AS_EAF), true);
        menuActions.remove(mergeTransMI.getAction());
        mergeTransMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        		ELANCommandFactory.MERGE_TRANSCRIPTIONS), true);
        
        menuBackup.setEnabled(true);
        ButtonGroup backupGroup = new ButtonGroup();

        // retrieve the stored value for backup interval
        Integer buDelay = (Integer) Preferences.get("BackUpDelay", null);
        JRadioButtonMenuItem neverMI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.BACKUP_NEVER));

        if ((buDelay == null) ||
                ((buDelay != null) &&
                (buDelay.compareTo(Constants.BACKUP_NEVER) == 0))) {
            neverMI.setSelected(true);
        }

        JRadioButtonMenuItem backup1MI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.BACKUP_1));

        if ((buDelay != null) && (buDelay.compareTo(Constants.BACKUP_1) == 0)) {
            backup1MI.setSelected(true);
        }
        
        JRadioButtonMenuItem backup5MI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.BACKUP_5));

        if ((buDelay != null) && (buDelay.compareTo(Constants.BACKUP_5) == 0)) {
            backup5MI.setSelected(true);
        }

        JRadioButtonMenuItem backup10MI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.BACKUP_10));

        if ((buDelay != null) && (buDelay.compareTo(Constants.BACKUP_10) == 0)) {
            backup10MI.setSelected(true);
        }

        JRadioButtonMenuItem backup20MI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.BACKUP_20));

        if ((buDelay != null) && (buDelay.compareTo(Constants.BACKUP_20) == 0)) {
            backup20MI.setSelected(true);
        }

        JRadioButtonMenuItem backup30MI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.BACKUP_30));

        if ((buDelay != null) && (buDelay.compareTo(Constants.BACKUP_30) == 0)) {
            backup30MI.setSelected(true);
        }

        backupGroup.add(neverMI);
        backupGroup.add(backup1MI);
        backupGroup.add(backup5MI);
        backupGroup.add(backup10MI);
        backupGroup.add(backup20MI);
        backupGroup.add(backup30MI);
        menuBackup.add(neverMI);
        menuBackup.add(backup1MI);
        menuBackup.add(backup5MI);
        menuBackup.add(backup10MI);
        menuBackup.add(backup20MI);
        menuBackup.add(backup30MI);

        pageSetUpMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.PAGESETUP), true);
        printPreviewMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.PREVIEW), true);
        printMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.PRINT), true);

        menuExport.setEnabled(true);

        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.EXPORT_SHOEBOX)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.EXPORT_TOOLBOX)));
        
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.EXPORT_CHAT)));

        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.EXPORT_TAB)));

        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.EXPORT_TIGER)));
        
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.EXPORT_INTERLINEAR)));

        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                ELANCommandFactory.EXPORT_HTML)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.EXPORT_TRAD_TRANSCRIPT)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.EXPORT_PRAAT_GRID)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.EXPORT_WORDS)));
        
        ma = new MenuAction("Menu.File.Export.Smil");
        menuActions.add(ma);
        menuExportSMIL = new JMenu(ma);        
        menuExport.add(menuExportSMIL);
        menuExportSMIL.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                   transcriptionForThisFrame, ELANCommandFactory.EXPORT_SMIL_RT)));
        menuExportSMIL.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.EXPORT_SMIL_QT)));        

        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.EXPORT_QT_SUB)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.EXPORT_SUBTITLES)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.EXPORT_RECOG_TIER)));

        //		menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        //				 transcriptionForThisFrame, ELANCommandFactory.EXPORT_TEX)));
        // maybe test here if available and only show menu item if available:
        //
        //ExportMediaCA ca = (ExportMediaCA) ELANCommandFactory.getCommandAction(transcriptionForThisFrame, ELANCommandFactory.EXPORT_MEDIA);
        //if (ca.isAvailable()) {
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.EXPORT_MEDIA)));

        //}
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.CLIP_MEDIA)));
        
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.EXPORT_IMAGE_FROM_WINDOW)));
        menuExport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.EXPORT_FILMSTRIP)));
        //menuImport.addSeparator();
        //menuImport.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        //        transcriptionForThisFrame, ELANCommandFactory.IMPORT_PRAAT_GRID)));
        menuActions.remove(importPraatMI.getAction());
        importPraatMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.IMPORT_PRAAT_GRID), true);
        menuActions.remove(importRecogTiersMI.getAction());
        importRecogTiersMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        		ELANCommandFactory.IMPORT_RECOG_TIERS), true);
        // menu items with command actions
        CommandAction undoCA = ELANCommandFactory.getUndoCA(transcriptionForThisFrame);
        undoMI.setAction(undoCA);

        CommandAction redoCA = ELANCommandFactory.getRedoCA(transcriptionForThisFrame);
        redoMI.setAction(redoCA);

        editCVMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        		ELANCommandFactory.EDIT_CV_DLG), true);
		// For opening a Edit Lexicon Service Dialog
        editLexiconServiceMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        		ELANCommandFactory.EDIT_LEX_SRVC_DLG), true);
        setAuthorMI.setAction(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SET_AUTHOR), true);
        linkedFilesMI.setAction(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.LINKED_FILES_DLG), true);
        importPrefsMI.setAction(ELANCommandFactory.getCommandAction(
    			transcriptionForThisFrame, ELANCommandFactory.IMPORT_PREFS), true);
        exportPrefsMI.setAction(ELANCommandFactory.getCommandAction(
    			transcriptionForThisFrame, ELANCommandFactory.EXPORT_PREFS), true);

        menuAnnotation.setEnabled(true);
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.NEW_ANNOTATION)));        
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.NEW_ANNOTATION_BEFORE)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.NEW_ANNOTATION_AFTER)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.MODIFY_ANNOTATION)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.MODIFY_ANNOTATION_DC_DLG)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.MERGE_ANNOTATION_WN)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.MERGE_ANNOTATION_WB)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.REMOVE_ANNOTATION_VALUE)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.SPLIT_ANNOTATION)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.DELETE_ANNOTATION)));
        ma = new MenuAction("Menu.Annotation.Delete");
        menuActions.add(ma);
        JMenu delMenu = new JMenu(ma);
        menuAnnotation.add(delMenu);
        delMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, 
        		ELANCommandFactory.DELETE_ANNOS_IN_SELECTION)));
        delMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, 
        		ELANCommandFactory.DELETE_ANNOS_LEFT_OF)));
        delMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, 
        		ELANCommandFactory.DELETE_ANNOS_RIGHT_OF)));
        delMenu.addSeparator();
        delMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, 
        		ELANCommandFactory.DELETE_ALL_ANNOS_LEFT_OF)));
        delMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, 
        		ELANCommandFactory.DELETE_ALL_ANNOS_RIGHT_OF)));
        
        if (SVGPrefs.getUseSVG() ||
            (((TranscriptionImpl) transcriptionForThisFrame).getSVGFile() != null)) {
        	menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.MODIFY_GRAPHIC_ANNOTATION)));
        }
        menuAnnotation.addSeparator();
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
            transcriptionForThisFrame,
            ELANCommandFactory.COPY_ANNOTATION)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
            transcriptionForThisFrame,
            ELANCommandFactory.COPY_ANNOTATION_TREE)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
    		transcriptionForThisFrame, 
		    ELANCommandFactory.DUPLICATE_ANNOTATION)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
            transcriptionForThisFrame,
            ELANCommandFactory.PASTE_ANNOTATION)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
            transcriptionForThisFrame,
            ELANCommandFactory.PASTE_ANNOTATION_TREE)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
            transcriptionForThisFrame,
            ELANCommandFactory.PASTE_ANNOTATION_HERE)));
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
            transcriptionForThisFrame,
            ELANCommandFactory.PASTE_ANNOTATION_TREE_HERE)));
        menuAnnotation.addSeparator();
        ma = new MenuAction("Menu.Annotation.Shift");
        menuActions.add(ma);
        JMenu shiftMenu = new JMenu(ma);
        menuAnnotation.add(shiftMenu);
        shiftMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SHIFT_ACTIVE_ANNOTATION)));
        shiftMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SHIFT_ANNOS_IN_SELECTION)));
        shiftMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SHIFT_ANNOS_LEFT_OF)));
        shiftMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SHIFT_ANNOS_RIGHT_OF)));
        shiftMenu.addSeparator();
        shiftMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SHIFT_ALL_ANNOS_LEFT_OF)));
        shiftMenu.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SHIFT_ALL_ANNOS_RIGHT_OF)));
        
        menuAnnotation.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SHIFT_ALL_ANNOTATIONS)));
    
        menuTier.setEnabled(true);
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.ADD_TIER)));
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.CHANGE_TIER)));
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
    		transcriptionForThisFrame, ELANCommandFactory.REPARENT_TIER)));
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.DELETE_TIER)));
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.IMPORT_TIERS)));
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.ADD_PARTICIPANT)));
        menuTier.addSeparator();
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.TOKENIZE_DLG)));

        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.FILTER_TIER_DLG)));

        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
				transcriptionForThisFrame,
				ELANCommandFactory.COPY_TIER_DLG)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
				transcriptionForThisFrame,
				ELANCommandFactory.MERGE_TIERS)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
				transcriptionForThisFrame,
				ELANCommandFactory.MERGE_TIER_GROUP)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.ANN_ON_DEPENDENT_TIER)));

        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.ANN_FROM_OVERLAP)));
        // temp
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.ANN_FROM_OVERLAP_CLAS)));
        // temp
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.ANN_FROM_GAPS)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.REGULAR_ANNOTATION_DLG)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.REMOVE_ANNOTATIONS_OR_VALUES)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.LABEL_AND_NUMBER)));
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.CHANGE_CASE)));
        
//        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
//                transcriptionForThisFrame,
//                ELANCommandFactory.SEGMENTATION_DLG)));

        menuTier.addSeparator();
        
        menuTier.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.COMPARE_ANNOTATORS_DLG)));
    
        menuType.setEnabled(true);
        
        menuType.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.ADD_TYPE)));
        menuType.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.CHANGE_TYPE)));
        menuType.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.DELETE_TYPE)));
        menuType.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame,
                ELANCommandFactory.IMPORT_TYPES)));
        
        //menuSearch.setEnabled(true);
        searchMI.setAction(ELANCommandFactory.getCommandAction(
        		transcriptionForThisFrame, ELANCommandFactory.SEARCH_DLG), true);
        goToMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.GOTO_DLG), true);

        //menuView.setEnabled(true);
        tierDependenciesMI.setAction(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.TIER_DEPENDENCIES), true);
        spreadsheetMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SPREADSHEET), true);
        statisticsMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.STATISTICS), true);
        
//        CommandAction syntaxViewerAction = ELANCommandFactory.getCommandAction(
//        		transcriptionForThisFrame, ELANCommandFactory.SYNTAX_VIEWER);
//        if(syntaxViewerAction != null){
//        	menuView.add(new ElanMenuItem(syntaxViewerAction));
//        }
        
        menuChangeTimePropMode.setEnabled(true);

        ButtonGroup timePropGroup = new ButtonGroup();
        JRadioButtonMenuItem normalModeMI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.TIMEPROP_NORMAL));
        JRadioButtonMenuItem bulldozerModeMI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.BULLDOZER_MODE));
        JRadioButtonMenuItem shiftModeMI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.SHIFT_MODE));
        timePropGroup.add(normalModeMI);
        normalModeMI.setSelected(true);
        timePropGroup.add(bulldozerModeMI);
        timePropGroup.add(shiftModeMI);
        menuChangeTimePropMode.add(normalModeMI);
        menuChangeTimePropMode.add(bulldozerModeMI);
        menuChangeTimePropMode.add(shiftModeMI);

        ButtonGroup modeGroup = new ButtonGroup();
        menuItemAnnoMode.setAction(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame,
                    ELANCommandFactory.ANNOTATION_MODE));
        menuItemAnnoMode.setEnabled(true);
        menuItemSyncMode.setAction(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.SYNC_MODE));
        menuItemSyncMode.setEnabled(true);
       
        menuItemTranscMode.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.TRANSCRIPTION_MODE));
        menuItemTranscMode.setEnabled(true);
        
        menuItemSegmentMode.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SEGMENTATION_MODE));
        menuItemSegmentMode.setEnabled(true);
        
        modeGroup.add(menuItemAnnoMode);
        modeGroup.add(menuItemSyncMode);
        modeGroup.add(menuItemTranscMode);
        modeGroup.add(menuItemSegmentMode);
        
        Object val = Preferences.get("LayoutManager.CurrentMode", viewerManager.getTranscription()); 
        if(val != null){
        	switch((Integer)val){
        	case ElanLayoutManager.NORMAL_MODE:
        		menuItemAnnoMode.setSelected(true);
        		break;
        	case ElanLayoutManager.SYNC_MODE:
        		menuItemSyncMode.setSelected(true);
        		break;
        	case ElanLayoutManager.TRANSC_MODE:
        		menuItemTranscMode.setSelected(true);
        		
        		break;
        	case ElanLayoutManager.SEGMENT_MODE:
        		menuItemSegmentMode.setSelected(true);
        		break;
        	}        	
        }        
        
        
//        menuOptions.add(menuItemAnnoMode);
//        menuOptions.add(menuItemSyncMode);
        //menuOptions.addSeparator();
        //menuItemKioskMode = new JCheckBoxMenuItem(ELANCommandFactory.getCommandAction(
        //        transcriptionForThisFrame,
        //        ELANCommandFactory.KIOSK_MODE));
        //menuOptions.add(menuItemKioskMode);
        //menuOptions.addSeparator();

//        menuItemPlayAround.setAction(ELANCommandFactory.getCommandAction(
//                transcriptionForThisFrame,
//                ELANCommandFactory.PLAY_AROUND_SELECTION_DLG));
//        menuItemPlayAround.setEnabled(true);

//        menuItemRateVol.setAction(ELANCommandFactory.getCommandAction(
//                transcriptionForThisFrame,
//                ELANCommandFactory.PLAYBACK_TOGGLE_DLG));
//        menuItemRateVol.setEnabled(true);

        menuFrameLength.setEnabled(true);
        ButtonGroup videoGroup = new ButtonGroup();
        JRadioButtonMenuItem palMI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.SET_PAL));
        JRadioButtonMenuItem ntscMI = new JRadioButtonMenuItem(ELANCommandFactory.getCommandAction(
                    transcriptionForThisFrame, ELANCommandFactory.SET_NTSC));
        videoGroup.add(palMI);
        palMI.setSelected(true);
        videoGroup.add(ntscMI);
        menuFrameLength.add(palMI);
        menuFrameLength.add(ntscMI);
        
        if (elanP2P != null) {
            menuItemPublishDoc = new JMenuItem();
            menuItemPublishDoc.addActionListener(this);
            menuItemPublishDoc.setActionCommand("PublishDoc");
            menuP2P.add(menuItemPublishDoc);

            //menuP2P.add(new ElanMenuItem(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, ELANCommandFactory.PUBLISH_DOC)), 0);
        }
        updateLocale();
    }

    /**
     * DOCUMENT ME!
     */
    public void updateLocale() {
        setFrameTitle();

        // update the language menu items
        if (languageBG != null) {
            if (languageBG.getButtonCount() > 0) {
                Enumeration en = languageBG.getElements();
                Object item;
                JRadioButtonMenuItem rbItem;

                while (en.hasMoreElements()) {
                    item = en.nextElement();

                    if (item instanceof JRadioButtonMenuItem) {
                        rbItem = (JRadioButtonMenuItem) item;

                        if (rbItem.getAction() instanceof SetLocaleMA) {
                        	if ( ((SetLocaleMA)rbItem.getAction()).getLocale().equals(ElanLocale.getLocale())) {
	                            rbItem.setSelected(true);                            
	
	                            break;
                        	}
                        }
                    }
                }
            }
        }

        if (menuP2P != null) {
            menuItemDiscoverDoc.setText(ElanLocale.getString(
                    "Menu.P2P.DiscoverDocument"));

            if (menuItemPublishDoc != null) {
                menuItemPublishDoc.setText(ElanLocale.getString(
                        "Menu.P2P.PublishDocument"));
            }
        }

        // update the special, document independent actions
        MenuAction ma;
        for (int i = 0; i < menuActions.size(); i++) {
        	ma = (MenuAction) menuActions.get(i);
        	ma.updateLocale();
        }
    }

    private void savePreferences() {
        // save some transcription specific preference values
    	/*
        if (layoutManager != null) {
            if (layoutManager.getMultiTierControlPanel() != null) {
                List tierOrder = layoutManager.getMultiTierControlPanel()
                                                .getTierOrder();

                if (tierOrder != null) {
                    Preferences.set("TierOrder", tierOrder,
                        transcriptionForThisFrame);
                }

                String activeTierName = layoutManager.getMultiTierControlPanel()
                                                     .getActiveTierName();

                if (activeTierName != null) {
                    Preferences.set("ActiveTierName", activeTierName,
                        transcriptionForThisFrame);
                }
            }

            Preferences.set("LayoutManagerState", layoutManager.getState(),
                transcriptionForThisFrame);
        }
        */
        if (viewerManager != null) {
            Preferences.set("MediaTime",
                viewerManager.getMasterMediaPlayer().getMediaTime(),
                transcriptionForThisFrame, false, false);
            Preferences.set("SelectionBeginTime",
                viewerManager.getSelection().getBeginTime(),
                transcriptionForThisFrame, false, false);
            Preferences.set("SelectionEndTime",
                viewerManager.getSelection().getEndTime(),
                transcriptionForThisFrame, false, false);
            Preferences.set("TimeScaleBeginTime",
                viewerManager.getTimeScale().getBeginTime(),
                transcriptionForThisFrame, false, true);// forces writing
        }
        // replace by setPreference();
        setPreference("Locale", ElanLocale.getLocale(), null);
        setPreference("FrameSize", getSize(), null);
        //setPreference("FrameLocation", getLocation(), null);
        Preferences.set("FrameLocation", getLocation(), null, false, true);// forces writing
    }
    
    private void loadViewerPreferences(){
    	
    	Object val = Preferences.get(ELANCommandFactory.GRID_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemGridViewer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.TEXT_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemTextViewer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.SUBTITLE_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemSubtitleViewer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.LEXICON_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemLexiconViewer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.AUDIO_RECOGNIZER, null);
    	if (val instanceof Boolean) {
    		menuItemAudioRecognizer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.VIDEO_RECOGNIZER, null);
    	if (val instanceof Boolean) {
    		menuItemVideoRecognizer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.METADATA_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemMetaDataViewer.setSelected((Boolean)val);    	
    	}
    	
    	val = Preferences.get(ELANCommandFactory.SIGNAL_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemSignalViewer.setSelected((Boolean)val);    	
    	}    
    	val = Preferences.get(ELANCommandFactory.INTERLINEAR_VIEWER, null);
    	if (val instanceof Boolean) {
    		menuItemInterLinearViewer.setSelected((Boolean)val);    	
    	}  
    }

    private void loadPreferences() {
        // initialize some transcription specific values
        // invokeLater ensures the viewers are initialized properly before setting values
        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                	/*
                    Locale savedLocale = (Locale) Preferences.get("Locale", null);

                    if (savedLocale != null) {
                        ElanLocale.setLocale(savedLocale);
                    }
                    Object toObject = Preferences.get("TierOrder",
                            transcriptionForThisFrame);
                    Vector tierOrder;
                    if (toObject instanceof ArrayList) {
                    	tierOrder = new Vector((ArrayList) toObject);
                    } else {
                    	tierOrder = (Vector) toObject;
                    }

                    if (tierOrder != null) {
                        if (tierOrder.size() > 0) {
                            layoutManager.getMultiTierControlPanel()
                                         .setTierOrder(tierOrder);
                        } else if (transcriptionForThisFrame.getTiers()
                                                                .size() > 0) {
                            Vector tiers = transcriptionForThisFrame.getTiers();
                            Vector tierNames = new Vector(tiers.size());

                            for (int i = 0; i < tiers.size(); i++) {
                                Tier t = (Tier) tiers.get(i);
                                tierNames.add(t.getName());
                            }

                            layoutManager.getMultiTierControlPanel()
                                         .setTierOrder(tierNames);
                        }
                    }

                    String activeTierName = (String) Preferences.get("ActiveTierName",
                            transcriptionForThisFrame);

                    if (activeTierName != null) {
                        layoutManager.getMultiTierControlPanel()
                                     .setActiveTierForName(activeTierName);
                    }
                    */
                    Long beginTime = (Long) Preferences.get("SelectionBeginTime",
                            transcriptionForThisFrame);
                    Long endTime = (Long) Preferences.get("SelectionEndTime",
                            transcriptionForThisFrame);

                    if ((beginTime != null) && (endTime != null)) {
                        viewerManager.getSelection().setSelection(beginTime.longValue(),
                            endTime.longValue());
                    }

                    Long mediaTime = (Long) Preferences.get("MediaTime",
                            transcriptionForThisFrame);

                    if (mediaTime != null) {
                        viewerManager.getMasterMediaPlayer().setMediaTime(mediaTime.longValue());
                    }

                    Long timeScaleBeginTime = (Long) Preferences.get("TimeScaleBeginTime",
                            transcriptionForThisFrame);

                    if (timeScaleBeginTime != null) {
                        viewerManager.getTimeScale().setBeginTime(timeScaleBeginTime.longValue());
                    }
                    
        	            Object val = Preferences.get("MediaNavigation.FrameStepToFrameBegin",
        	                null);
        	
        	            if (val instanceof Boolean) {
        	                viewerManager.setFrameStepsToBeginOfFrame(((Boolean) val).booleanValue());
        	            }
                    /*
                    HashMap layoutMap = (HashMap) Preferences.get("LayoutManagerState",
                           transcriptionForThisFrame);

                    if (layoutMap != null) {
                        layoutManager.setState(layoutMap);
                    }
					*/
                    // start the backup thread
                    Integer backupDelay = (Integer) Preferences.get("BackUpDelay",
                            null);

                    if ((backupDelay != null) && (backupDelay.intValue() > 0)) {
                        Command c = ELANCommandFactory.createCommand(transcriptionForThisFrame,
                                ELANCommandFactory.BACKUP);
                        c.execute(ELANCommandFactory.getCommandAction(
                                transcriptionForThisFrame,
                                ELANCommandFactory.BACKUP),
                            new Object[] { backupDelay });
                    }
                    fullyInitialized = true;
                    
                    layoutManager.doLayout();
                }
            });

        //layoutManager.doLayout();
    }

    /**
     * A separate treatment for preferences for controlled vocabulary entries.
     * The preferences are (currently) stored in the CVEntry themselves, in contrast
     * to most other preferences that are fetched from the Preferences each time when needed.
     */
    private void loadCVPreferences() {
    	if (transcriptionForThisFrame == null) {
    		return;
    	}
    	Object cvPrefs = Preferences.get("CV.Prefs", transcriptionForThisFrame);
    	if (cvPrefs instanceof HashMap) {
    		HashMap cvPrefMap = (HashMap) cvPrefs;
    		List allCV = ((TranscriptionImpl) transcriptionForThisFrame).getControlledVocabularies();
    		if (allCV == null) {
    			return;
    		}
			String color = "Color";
			String keyCode = "KeyCode";
    		ControlledVocabulary cv;
    		CVEntry[] cve;
    		HashMap cvMap;
    		HashMap cveMap;
    		for (int i = 0; i < allCV.size(); i++) {
    			cv = (ControlledVocabulary) allCV.get(i);
    			Object cvMapObj = cvPrefMap.get(cv.getName());
    			if (cvMapObj instanceof HashMap) {
    				cvMap = (HashMap) cvMapObj;
    				cve = cv.getEntries();
    				
    				for (int j = 0; j < cve.length; j++) {
    					if (cvMap.containsKey(cve[j].getValue())) {
    						cveMap = (HashMap) cvMap.get(cve[j].getValue());
    						if (cveMap.containsKey(color)) {
    							cve[j].setPrefColor((Color)cveMap.get(color));
    						}
    						if (cveMap.containsKey(keyCode)) {
    							cve[j].setShortcutKeyCode(((Integer)cveMap.get(keyCode)).intValue());
    						}
    					}
    				}
    			}
    		}
    	}
    }
    
    /**
     * @see PreferencesUser#setPreference(String, Object, Object)
     */
	public void setPreference(String key, Object value, Object document) {
		if (document instanceof Transcription) {
			Preferences.set(key, value, (Transcription)document, false, false);
		} else {
			Preferences.set(key, value, null, false, false);
		}
	}

	/**
	 * Should these preferences be part of the loading of doc. preferences??
	 * @see PreferencesListener#preferencesChanged()
	 */
	public void preferencesChanged() {
        Dimension d = (Dimension) Preferences.get("FrameSize", null);
        // HS June 2010 correct dimension and location if they don't fit on the screen?
        // or make this a user preference?
        Rectangle wRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
        
        if (d != null) {      	
        	if (d.width > wRect.width) {
        		d.setSize(wRect.width, d.height);
        	}
        	if (d.height > wRect.getHeight()) {
        		d.setSize(d.width, wRect.height);
        	}
            setSize(d);
        }
        Point p = (Point) Preferences.get("FrameLocation", null);

        if (p != null) {
        	int mpx = wRect.width - getWidth();
        	if (p.x > mpx && mpx > wRect.x) {
        		p.x = mpx;
        	}
        	int mph = wRect.height - getHeight();
        	if (p.y > mph && mph > wRect.y) {
        		p.y = mph;
        	}
            setLocation(p);
        }
        // ?? 
        Locale savedLocale = (Locale) Preferences.get("Locale", null);

        if (savedLocale != null) {
            ElanLocale.setLocale(savedLocale);
        }
        
        if (viewerManager != null) {
	        Object val = Preferences.get("MediaNavigation.FrameStepToFrameBegin",
	                null);
	
	        if (val instanceof Boolean) {
	            viewerManager.setFrameStepsToBeginOfFrame(((Boolean) val).booleanValue());
	        }
        }
        // update values in playbacktoggle and play around selection actions, not really elegant
        Object val = Preferences.get("PlayAroundSelection.Mode", null);
        boolean msMode = true;
        if (val instanceof String) {
        	if ("frames".equals((String)val)) {
        		msMode = false;
        	}
        }
        val = Preferences.get("PlayAroundSelection.Value", null);
        if (val instanceof Integer && transcriptionForThisFrame != null) {
        	int playaroundVal = ((Integer) val).intValue();
        	if (!msMode) {
        		playaroundVal *= viewerManager.getMasterMediaPlayer().getMilliSecondsPerSample();
        		((PlayAroundSelectionCA) ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        				ELANCommandFactory.PLAY_AROUND_SELECTION)).setPlayAroundSelectionValue(playaroundVal);
        	}
        }
        val = Preferences.get("PlaybackRateToggleValue", null);
        if (val instanceof Float && transcriptionForThisFrame != null) {
        	float rate = ((Float) val).floatValue();
        	((PlaybackRateToggleCA)ELANCommandFactory.getCommandAction(
                	transcriptionForThisFrame, 
                	ELANCommandFactory.PLAYBACK_RATE_TOGGLE)).setToggleValue(rate);
        }
        val = Preferences.get("PlaybackVolumeToggleValue", null);
        if (val instanceof Float && transcriptionForThisFrame != null) {
        	float vol = ((Float) val).floatValue();
        	((PlaybackVolumeToggleCA)ELANCommandFactory.getCommandAction(
                	transcriptionForThisFrame, 
                	ELANCommandFactory.PLAYBACK_VOLUME_TOGGLE)).setToggleValue(vol);
        }
        val = Preferences.get("NumberOfBackUpFiles", null);
        if (val instanceof Integer && transcriptionForThisFrame != null) {
        	int nf = ((Integer) val).intValue();
        	((BackupCA) ELANCommandFactory.getCommandAction(
        			transcriptionForThisFrame, 
        			ELANCommandFactory.BACKUP)).setNumBuFiles(nf);
        } 
	}
	
    /**
     * Checks whether there are any changes to save and starts the saving and or 
     * closing sequence.
     * 
     * @see #saveAndClose(boolean)
     * @see #doClose(boolean)
     */
    public void checkSaveAndClose() {
    	if (transcriptionForThisFrame != null) {
    		
    		if (transcriptionForThisFrame.isChanged()) {

	            int response = JOptionPane.showConfirmDialog(null,
	                    ElanLocale.getString("Frame.ElanFrame.UnsavedData"),
	                    ElanLocale.getString("Message.Warning"),
	                    JOptionPane.YES_NO_CANCEL_OPTION);
	
	            if (response == JOptionPane.YES_OPTION) {
	            	saveAndClose(true);
	            } else if ((response == JOptionPane.CANCEL_OPTION) ||
	                    (response == JOptionPane.CLOSED_OPTION)) {
	                // the user does not want to close
	                return;
	            } else {
	            	doClose(true);
	            }
    		} else {
            	doClose(true);
            }
    	}
    }
    
    /**
     * Saves the document and starts the window closing sequence. 
     * <b>Caution: </b>does not ask the user whether or not to save changes; 
     * it is assumed that this has been done elsewhere.
     * Called from the Close Command or from the FrameManager.
     * 
     * @param unregister if true this frame should be unregistered with the 
     * FrameManager
     * @see #checkSaveAndClose()
     * @see #doClose(boolean)
     */
    public void saveAndClose(boolean unregister) {
    	if (unregister) {
    		FrameManager.getInstance().closeFrame(this);	
    	}
    	
        if ((transcriptionForThisFrame != null) &&
                transcriptionForThisFrame.isChanged()) {
            boolean saveNewCopy = false;

            if (transcriptionForThisFrame.getName().equals(TranscriptionImpl.UNDEFINED_FILE_NAME)) {
                // save as dialog
                saveNewCopy = true;
            }

            TranscriptionStore ets = ACMTranscriptionStore.getCurrentTranscriptionStore();
            StoreCommand storeComm = new StoreCommand(ELANCommandFactory.STORE);
            if (viewerManager.getMultiTierControlPanel() != null) {
	            storeComm.execute(transcriptionForThisFrame,
	                new Object[] {
	                    ets, new Boolean(false), new Boolean(saveNewCopy),
	                    viewerManager.getMultiTierControlPanel()
	                                 .getVisibleTiers()
	                });
            } else {
	            storeComm.execute(transcriptionForThisFrame,
		                new Object[] {
		                    ets, new Boolean(false), new Boolean(saveNewCopy),
		                    new ArrayList()
		                });
            }
            // HS nov 2006: check if the file actually has been saved
            // if not e.g. a Save As dialog has been canceled, return
            if (transcriptionForThisFrame.isChanged()) {
                System.out.println("Save (as) cancelled");
                return;
            }
        }
        doClose(unregister);
    }
    
    /**
     * Unregisteres with the FrameManager if necessary and closes 
     * the frame (without saving). Performs clean up and disposes the frame.<br>
     * Called from from Close Command or from the FrameManager.<br>
     * <b>Caution: </b>does not check whether the document should be saved or not; 
     * it is assumed that this has been done elsewhere.
     * @param unregister if true this frame should be unregistered with the 
     * FrameManager
     * @see #checkSaveAndClose()
     * @see #saveAndClose(boolean)
     */
    public void doClose(boolean unregister) {
    	if (unregister) {
    		FrameManager.getInstance().closeFrame(this);	
    	}
    	
    	savePreferences();
    	
    	//remove document from ELANCommandFactory, unregister as listener etc.
        if (transcriptionForThisFrame != null) {
            //stop the backup task, just to be sure

        	    BackupCA ca = ((BackupCA) ELANCommandFactory.getCommandAction(transcriptionForThisFrame,
                ELANCommandFactory.BACKUP));
        	    if (ca != null) {
        		    ca.stopBackUp();
        	    }
        	
            ELANCommandFactory.removeDocument(viewerManager);
            Preferences.removeDocument(transcriptionForThisFrame);
            // remove this elan frame as locale listener
            ElanLocale.removeElanLocaleListener(transcriptionForThisFrame);
            if (viewerManager != null) {
                viewerManager.cleanUpOnClose();
            }
            if (layoutManager != null) {
                	layoutManager.cleanUpOnClose();	
            }
            
            transcriptionForThisFrame = null;
            viewerManager = null;
            layoutManager = null;
        }
        setJMenuBar(null);
        menuBar = null;
        dispose();
        System.runFinalization();
        System.gc();
    }

    /**
     * Adds an action to a menu. The menu is specified by an id, the position in 
     * the menu can be specified by an index. The action is supposed to handle its
     * own events.
     * 
     * @param action the action to add to a Menu (inside a JMenuItem)
     * @param menuId the identifier of the menu, a constant from #FrameConstants
     * @param index the index to insert the action, -1 means to add at the end
     */
    public void addActionToMenu(Action action, int menuId, int index) {
    	if (action == null) {
    		return;
    	}
    	Object mm = getMenuById(menuId);
    	if (mm instanceof JMenu) {
    		JMenu menu = (JMenu) mm;
    		if (menu == menuWindow) {
    			JRadioButtonMenuItem item = new JRadioButtonMenuItem(action);
    			menu.add(item, index);
    			windowsGroup.add(item);
    		} else if (menu == menuMediaPlayer) {
    			JCheckBoxMenuItem item = new JCheckBoxMenuItem(action);
    			menu.add(item, index);
    		} else {
        		JMenuItem item = new JMenuItem(action);
        		menu.add(item, index);
    		}
    	}
    }
    
    /**
     * Looks for an Action with the specified id (== the LONG_DESCRIPTION) under 
     * the menu identified by the menu id. If it is found it is removed from the menu 
     * and returned.
     * @param actionId the id of the Action, currently stored in the LONG_DESCRIPTION value
     * @param menuId the identifier of the menu, a constant from #FrameConstants
     * @return the action that has been removed from the menu
     */
    public Action removeActionFromMenu(String actionId, int menuId) {
    	if (actionId == null) {
    		return null;
    	}
    	Object mm = getMenuById(menuId);
    	if (mm instanceof JMenu) {
    		JMenu menu = (JMenu) mm;
    		JMenuItem item;
    		Action ac;
    		Object mi;
    		for (int i = 0; i < menu.getMenuComponentCount(); i++) {
    			mi = menu.getMenuComponent(i);
    			if (mi instanceof JMenuItem) {
    				item = (JMenuItem) mi;
    				ac = item.getAction();
    				if (ac != null) {
    					if (actionId.equals(ac.getValue(Action.LONG_DESCRIPTION))) {
    						menu.remove(item);
    						if (menu == menuWindow) {
    							windowsGroup.remove(item);
    						}
    						return ac;
    					}
    				}
    			}
    		}
    	}
    	return null;
    }
    
    /**
     * Sets the action / menuitem identified by actionId selected. 
     * Note: this method might need to be extended with a boolean
     * argument for the selected state.
     * 
     * @param actionId the id of the action
     * @param menuId the id of the menu, a constant from #FrameConstants
     */
    public void setMenuSelected(String actionId, int menuId) {
    	if (actionId == null) {
    		return;
    	}
    	
    	Object mm = getMenuById(menuId);
    	if (mm instanceof JMenu) {
    		JMenu menu = (JMenu) mm;
    		JRadioButtonMenuItem item;
    		Action ac;
    		Object mi;
    		for (int i = 0; i < menu.getMenuComponentCount(); i++) {
    			mi = menu.getMenuComponent(i);
    			if (mi instanceof JRadioButtonMenuItem) {
    				item = (JRadioButtonMenuItem) mi;
    				ac = item.getAction();
    				if (ac != null) {
    					if (actionId.equals(ac.getValue(Action.LONG_DESCRIPTION))) {
    						item.setSelected(true);
    					}
    				}
    			} else if (mi instanceof JCheckBoxMenuItem) {
    				JCheckBoxMenuItem chItem = (JCheckBoxMenuItem) mi;
    				ac = chItem.getAction();
    				if (ac != null) {
    					if (actionId.equals(ac.getValue(Action.LONG_DESCRIPTION))) {
    						chItem.setSelected(true);
    					}
    				}
    			}
    		}
    	}
    }

    /**
     * Enables or disables a menu or menuitem, identified by meniId.
     * Most menu items can also be enabled/disabled through their 
     * ActionCommands.
     * 
     * @param menuId the identifier of the menu, a constant defined 
     * in FrameConstants
     * @param enabled the enabled state
     */
    public void setMenuEnabled(int menuId, boolean enabled) {
	    	JMenuItem menu = getMenuById(menuId);
	    	if (menu != null) {
	    		menu.setEnabled(enabled);
	    	}
    }

    /**
     * Update or re-initialize the menu with the specified id.
     * 
     * @param menuId the menu Id
     */
    public void updateMenu(int menuId) {
    	if (menuId == FrameConstants.MEDIA_PLAYER) {
    		// delagate to the responsible manager
    		pvMenuManager.reinitializePlayerMenu();
    	}
    }
    
    /**
     * Looks for an Action with the specified id (== the LONG_DESCRIPTION) under 
     * the menu identified by the menu id. If it is found it is removed from the menu 
     * and returned.
     * @param actionId the id of the Action, currently stored in the LONG_DESCRIPTION value
     * @param menuId the identifier of the menu, a constant from #FrameConstants
     * @return the action that has been removed from the menu
     */
    public void enableOrDisableMenus(List<String> actionIdList, int menuId, boolean enabled) {    	    	
    	Object mm = getMenuById(menuId);
    	JMenu menu;
    	if (mm instanceof JMenu) {
			menu = (JMenu) mm;			
			if(actionIdList == null){
				menu.setEnabled(enabled);
				return;
			}			
			JMenuItem item;
    		Object mi;
    		Action ac;
    		String ca;
    		for (int i = 0; i < menu.getMenuComponentCount(); i++) {
    			mi = menu.getMenuComponent(i);
    			if (mi instanceof JMenuItem) {
    				item = (JMenuItem) mi;
    				ac = item.getAction();
    				ca = item.getActionCommand();
    				if (ac != null & ca != null) {
    					if (actionIdList.contains(ca)) {
    						item.setEnabled(enabled);
    						ac.setEnabled(enabled);
    					}
    				}
    			}
    		}
    	}    			
    }
    
    /**
     * Enables or disables all commands that are not in a menu but are instead 
     * added to the menubar's action map (and are accessible through an accelerator
     * key). 
     *  
     * @param enable
     */
    public void enableCommands(boolean enable) {
    	ActionMap menuMap = menuBar.getActionMap();
    	if (menuMap != null) {
			Iterator keyIt = registeredActions.keySet().iterator();
			Object key;
			while (keyIt.hasNext()) {
				key = keyIt.next();
				((Action) registeredActions.get(key)).setEnabled(enable);
			}
    		/*
    		if (!enable) {
    			menuMap.clear();
    		} else {
    			Iterator keyIt = registeredActions.keySet().iterator();
    			Object key;
    			while (keyIt.hasNext()) {
    				key = keyIt.next();
    				menuMap.put(key, (Action) registeredActions.get(key));
    			}
    		}
    		*/
    	}
    }
    
    /**
     * Returns the menu item identified by one of the menu constants in 
     * FrameConstants. This could be extended eventually to give access to 
     * any menu (item).<br>
     * Note: the mapping from frame constant to menu or menuitem might need 
     * to be reconsidered...
     * 
     * @param id the id of the menu, as defined in FrameConstants
     * @return the corresponding menu or menuitem
     */
    private JMenuItem getMenuById(int id) {
    	switch(id) {
    	case FrameConstants.FILE:
    		return menuFile;
    	case FrameConstants.EDIT:
    		return menuEdit;
    	case FrameConstants.ANNOTATION:
    		return menuAnnotation;
    	case FrameConstants.TIER:
    		return menuTier;
    	case FrameConstants.TYPE:
    		return menuType;
    	case FrameConstants.SEARCH:
    		return menuSearch;
    	case FrameConstants.VIEW:
    		return menuView;
    	case FrameConstants.OPTION:
    		return menuOptions;
    	case FrameConstants.WINDOW:
    		return menuWindow;
    	case FrameConstants.HELP:
    		return menuHelp;
    	case FrameConstants.RECENT:
    		return menuRecentFiles;
    	case FrameConstants.EXPORT:
    		return menuExport;
    	case FrameConstants.IMPORT:
    		return menuImport;
    	case FrameConstants.LANG:
    		return menuAppLanguage;
    	case FrameConstants.PROPAGATION:
    		return menuChangeTimePropMode;
    	case FrameConstants.FRAME_LENGTH:
    	    return menuFrameLength;
    	case FrameConstants.ANNOTATION_MODE:
    		return menuItemAnnoMode;
    	case FrameConstants.SYNC_MODE:
    		return menuItemSyncMode;
    	case FrameConstants.KIOSK_MODE:
    		return menuItemKioskMode;
    	case FrameConstants.PLAY_AROUND_SEL:
    		return menuItemPlayAround;
    	case FrameConstants.RATE_VOL_TOGGLE:
    		return menuItemRateVol;
    	case FrameConstants.MEDIA_PLAYER:
    		return menuMediaPlayer;
    	}
    	return null;
    }
    
    /**
     * This flag is set to true after loading of preferences, which runs on a separate
     * thread. Other classes can wait for this flag to be true before performing their
     * own actions, e.g. setting the media time.
     * 
	 * @return Returns the fullyInitialized flag.
	 */
	public boolean isFullyInitialized() {
		return fullyInitialized;
	}

	/**
     * DOCUMENT ME!
     *
     * @param arg DOCUMENT ME!
     */
    public static void main(String[] arg) {
        // supress floppy message from web start
        System.out.println("Java version: " + System.getProperty("java.version"));
        System.out.println("Runtime version: " + System.getProperty("java.runtime.version"));
        mpi.eudico.client.annotator.util.SystemInstallationSecurity.Instance()
                                                                   .go();

        //use java look and feel, so everything (multiple file chooser) looks the same
        //on windows and mac 
        /*
           try {
               UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
           } catch (Exception ex) {
               ex.printStackTrace();
           }
         */
        FrameManager.getInstance().setExitAllowed(true);

        //		String p2p = System.getProperty("P2P");
        //		if ((p2p != null) && (System.getProperty("P2P").equals("true"))) {
        //			mpi.eudico.p2p.ElanP2P.initP2P();
        //		}
        // make sure the directory for Elan data exists, for the time being the dir only contains global preferences
        File dataDir = new File(Constants.ELAN_DATA_DIR);

        if (!dataDir.exists()) {
            dataDir.mkdir();
        }

        // temporary, clean up old crap
        File oldCrap = new File(Constants.STRPROPERTIESFILE);
        oldCrap.delete();
        oldCrap = new File(Constants.USERHOME + Constants.FILESEPARATOR +
                ".elan.pfs");
        oldCrap.delete();

        if ((arg != null) && (arg.length > 0) && (arg[0].length() != 0)) {
            //new ElanFrame2(arg[0]);
            FrameManager.getInstance().createFrame(arg[0]);
        } else {
            //new ElanFrame2();
        	FrameManager.getInstance().createEmptyFrame();
        }

        // preliminary external launcher 
        mpi.eudico.client.annotator.integration.ExternalLauncher.start();
    }

    /**
     * Switches between "Metal" and the Mac native Look and Feel. "Metal" on
     * MacOS isn't always behaving well; menus and popup menus  often are not
     * updated correctly etc.
     */
    private void switchMacLF() {
        if (menuMacNativeLF != null) {
            if (menuMacNativeLF.getState()) {
                try {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    SwingUtilities.updateComponentTreeUI(this);
                    Preferences.set("UseMacLF", new Boolean(true), null);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            } else {
                try {
                    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                    SwingUtilities.updateComponentTreeUI(this);
                    Preferences.set("UseMacLF", new Boolean(false), null);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Mac OS X specific handling of the main (screen) menu Quit application
     * event. Implementation of MacApplicationListener.
     */
    public void macHandleQuit() {
    	FrameManager.getInstance().exit();
    }

    /**
     * Mac OS X specific handling of the main (screen) menu About application
     * event. Implementation of MacApplicationListener.
     */
    public void macHandleAbout() {
    	if (menuActions != null) {
    		MenuAction ma = null;
    		for (int i = 0; i < menuActions.size(); i++) {
    			ma = (MenuAction) menuActions.get(i);
    			if (ma instanceof AboutMA) {
    				ma.actionPerformed(null);
    				break;
    			}
    		}
    	}
    }
    
    /**
     * Mac specific handling of Preferences from the main menu bar.
     */
    public void macHandlePreferences() {
    	MenuAction ma = null;
	    	if (menuActions != null) {
	    		
	    		for (int i = 0; i < menuActions.size(); i++) {
	    			ma = (MenuAction) menuActions.get(i);
	    			if (ma instanceof EditPreferencesMA) {
	    				ma.actionPerformed(null);
	    				break;
	    			}
	    		}
	    	} 
	    	if (!(ma instanceof EditPreferencesMA)) {
	    		MenuAction ma2 = new EditPreferencesMA("Menu.Edit.Preferences.Edit", this);
	    		ma2.actionPerformed(null);
	    	}
    }

    /**
     * Listener for ElanFrame WindowEvents
     */
    private class ElanFrameWindowListener extends WindowAdapter {
        // triggered when the window is closed to quit the application
        // handle warnings and save operations in doExit();
        // EXIT WIL BE REPLACED BY CLOSE
        public void windowClosing(WindowEvent e) {
            //doExit();
        	if (transcriptionForThisFrame != null) {
        		// do nothing if this is an empty frame because a new
        		// empty frame would be created
        		checkSaveAndClose();	
        	} else {
        		// 07-2007 changed behaviour: with one empty window, exit the 
        		// application when the close button is pressed
        		doClose(true);
        	}
        	
        }
        
        /**
         * Notifies the FrameManager that this frame has been activated.
         * Menus will be updated. 
         */
		public void windowActivated(WindowEvent e) {
			FrameManager.getInstance().frameActivated(ElanFrame2.this);
		}
        
    }

    public void finalize() throws Throwable {
    	    System.out.println("Finalize ELAN window...");
    	    super.finalize();
    }
    
    public boolean isIntialized(){
    	return initialized;
    }
    
    /*
       private class InitThread extends Thread {
           final String path;
           InitThread(String eafPath) {
               path = eafPath;
           }
           public void run() {
               final IndeterminateProgressMonitor monitor = new IndeterminateProgressMonitor(
                   ElanFrame2.this,
                   true,
                   ElanLocale.getString("Frame.ElanFrame.Progress.Open") + path,
                   false,
                   null);
               new Thread(new Runnable(){
                   public void run() {
                       monitor.show();
                   }
               }).start();
               //monitor.show();
               ElanFrame2.this.initElan();
               monitor.close();
           }
       }
     */
}