package mpi.eudico.client.annotator.commands;

import java.awt.Cursor;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

public class AddParticipantCommand implements UndoableCommand{
	private String commandName;
	private TranscriptionImpl transcription;
	private List names;	
	private Tier parentTier = null;
	private String participantName;
	private ArrayList newTiersCommand ;
	private String oldValue;
	private String newValue;
	private boolean changePrefix;
	private boolean tierStrucSelected;
	
	
	 /** A command to delete a tier (and depending tiers) from a transcription.
	  *
	  * @param name the name of the command
	  */
	 public AddParticipantCommand(String name) {
	        commandName = name;
	        newTiersCommand = new ArrayList();
	 }
	 
	 /**
	   * <b>Note: </b>it is assumed the types and order of the arguments are correct.<br>
	   * If arg[2] is null it is assumed that there is no number part.
	   *
	   * @param receiver the TranscriptionImpl
	   * @param arguments the arguments: <ul>
	   * 		  	<li>arg[0] = the selected tiers/participants (ArrayList)</li> 
	   * 			<li>arg[1] = participantName (String)</li> 
	   * 			<li>arg[2] = oldValue (String)</li> 
	   * 			<li>arg[3] = newValue (String)</li> 
	   * 			<li>arg[4] = changePrefix (Boolean)</li> 
	   *            <li>arg[5] = tierStrucSelected (Boolean)</li> 
	   *      </ul>
	   */
	 public void execute(Object receiver, Object[] arguments) {
		 transcription = (TranscriptionImpl) receiver;
	     names = (List) arguments[0];  
	     participantName = (String) arguments[1];
	     oldValue = (String) arguments[2];
	     newValue = (String) arguments[3];
	     changePrefix = (Boolean) arguments[4];
	     tierStrucSelected = (Boolean) arguments[5];
	        
	     newValue = newValue.trim();
	 	              
	     if ((names == null) || (names.size() == 0)) {
	    	 ClientLogger.LOG.warning("No tier/participant selected.");
	         transcription.setNotifying(true);
	        return;
	     } 
	     
	     setWaitCursor(true);
	     
	     if(tierStrucSelected){	     
	    	 for(int i= 0; i< names.size();i++){
	    		 TierImpl tier = (TierImpl) transcription.getTierWithId((String)names.get(i));
	    		 if(tier != null){
	    			 parentTier = null;
	    			 addTier(tier, true);  	    			 
	    			 Vector childTiers = tier.getDependentTiers();
	    			 if(childTiers !=null){
	    				 for(int c=0; c < childTiers.size(); c++){
	    					 addTier((TierImpl)childTiers.get(c), false);   
	    				 }
	    			 }
	    		 }
	    	}
	     }else {
	    	 List tiers = transcription.getTiers();
	    	 List parentTiers = new ArrayList();
	    	 for(int i= 0; i< tiers.size();i++){
	    		 TierImpl tier = (TierImpl) tiers.get(i);
	    		 if(tier != null){
	    			 if(names.contains(tier.getParticipant())){
	    				 if(!tier.hasParentTier()){	    					
	    					 if(parentTiers.contains(tier)){
	    						 continue;
	    					 } 
	    				 } else {
	    					 if(parentTiers.contains(tier.getParentTier())){
	    						 continue;
	    					 }else {
	    						 tier = (TierImpl) tier.getParentTier();
	    					 }
	    				 }
	    				 parentTier = null;	
	    				 parentTiers.add(tier);
						 addTier(tier, true);  						
	    				 Vector childTiers = tier.getDependentTiers();
	    				 if(childTiers !=null){
	    					 for(int c=0; c < childTiers.size(); c++){
	    						 addTier((TierImpl)childTiers.get(c), false);   
	    					 }
	    				 }
	    			 }
	    		 }
	    	 }
	     }
	     
	     setWaitCursor(false);
	 }
	 
	 private void addTier(TierImpl tier, boolean parent){	  
		 String tierName =  tier.getName();			
         String lingType = tier.getLinguisticType().getLinguisticTypeName();            
         String annotator = tier.getAnnotator();
         Locale locale = tier.getDefaultLocale();           
         if(oldValue != null && oldValue.trim().length() > 0){        	
    	    if(changePrefix){
    	    	if(tierName.startsWith(oldValue)){
    	    		tierName= tierName.replaceFirst(oldValue, newValue);
    	    	} else {
    	    		tierName = newValue +"-" +tierName;
    	    	}
    	    } else {
    	    	if(tierName.endsWith(oldValue)){
    	    		tierName= tierName.substring(0, tierName.lastIndexOf(oldValue)) + newValue;
    	    	} else {
    	    		tierName = tierName +"-" +newValue;
    	    	}
    	    }        	 
  	        
         } else {
           	if(changePrefix){
           		tierName = newValue +"-" +tierName;
           	} else {
           		tierName = tierName +"-" +newValue;
           	}
    	 }
            
         Object[] args = new Object[]{tierName, parentTier, lingType, participantName, annotator, locale};
         Command c = new AddTierCommand(ELANCommandFactory.ADD_TIER);
    	 c.execute(transcription, args);
    	   
    	 newTiersCommand.add(c);
    	 
    	 if(parent){
     	   	parentTier = transcription.getTierWithId(tierName);
     	 }
	 }	
	 
	 /**
	  * Changes the cursor to either a 'busy' cursor or the default cursor.
	  *
	  * @param showWaitCursor when <code>true</code> show the 'busy' cursor
	  */
	 private void setWaitCursor(boolean showWaitCursor) {
		 if (showWaitCursor) {
			 ELANCommandFactory.getRootFrame(transcription).getRootPane()
	         	.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	        } else {
	            ELANCommandFactory.getRootFrame(transcription).getRootPane()
	                              .setCursor(Cursor.getDefaultCursor());
	        }
	    }
	 
	 /**
	   * Returns the name of the command.
	   *
	   * @return the name of the command
	   */
	 public String getName() {
		 return commandName;
	 }

	 public void undo() {
		 if (transcription != null && (newTiersCommand != null)) {	
			 setWaitCursor(true);
			 for (int i = 0; i < newTiersCommand.size(); i++) {
				 ((UndoableCommand)newTiersCommand.get(i)).undo();
			 }
			 setWaitCursor(false);
		 }
	 }	        		 

	 public void redo() {
		 if ((transcription != null) && (newTiersCommand != null)) {	
			 setWaitCursor(true);
			 for (int i = 0; i < newTiersCommand.size(); i++) {
				 ((UndoableCommand)newTiersCommand.get(i)).redo();
			 }
			 setWaitCursor(false);
		 }
	 }
}
