package mpi.eudico.client.annotator.commands;


import java.util.Vector;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

/**
 * An undoable command that builds on the new annotation command.
 * It takes the new annotation as input, and creates new annotation recursively
 * on all the dependent tiers
 * This is then one undoable command in the undo/redo list.
 */
public class NewAnnotationRecursiveCommand extends NewAnnotationCommand {    
    private TierImpl tier;
    
    
    /**
     * Creates a new NewDependentAnnotationCommand instance
     * 
     * @param name the name of the command
     */
    public NewAnnotationRecursiveCommand(String name) {
    	super(name);    	
    }

    
    /**
     * <b>Note: </b>it is assumed the types and order of the arguments are
     * correct.<br>
     * July 2006: removed the ViewerManager as one of the objects in the arguments array
     *
     * @param receiver the TierImpl
     * @param arguments the arguments:  <ul> <li>arg[0] = the begin time of the
     *        annotation (Long)</li> <li>arg[1] = the end time of the
     *        annotation (Long)</li> </ul>
     */
    public void execute(Object receiver, Object[] arguments) { 
    	super.execute(receiver, arguments);
    	tier = (TierImpl) receiver;         
        
        createNewAnnotaionsRecursively();
    }
    
    /**
     * The creation of the new annotation recursively
     * on all dependent tiers
     */
    
    private void createNewAnnotaionsRecursively(){
    	    	
    	Vector dependentTiers = tier.getDependentTiers();
	    if(dependentTiers != null){
	    	for(int i=0; i< dependentTiers.size(); i++){	    		
	        		TierImpl currentChildTier = (TierImpl) dependentTiers.elementAt(i);
	        		
	        		if(currentChildTier.isTimeAlignable()){
	            		currentChildTier.createAnnotation(newAnnBegin, newAnnEnd);
	            	}else{
	            		long time = (newAnnBegin +  newAnnEnd) / 2;
	            		currentChildTier.createAnnotation(time, time);	            		
	            	} 
	        	}
	    	((TranscriptionImpl) tier.getParent()).notifyListeners(tier, 3 , tier.getAnnotationAtTime(newAnnBegin));
	        }   
    }
    

    /**
     * The redo action.
     */
    public void redo() {  
    	super.redo();   
    	createNewAnnotaionsRecursively();
    }
    
    /**
     * The undo action.
     */
    public void undo() {
        super.undo();     
    }
    
    /**
     * Returns the name of the command.
     *
     * @return the name of the command
     */
    public String getName() {
        return super.getName();
    }
}


