package mpi.eudico.client.annotator.export;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.Selection;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.util.FileExtension;
import mpi.eudico.client.util.CheckBoxTableCellRenderer;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.MediaDescriptor;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.praat.PraatTGEncoderInfo;
import mpi.eudico.server.corpora.clomimpl.praat.PraatTextGridEncoder;
import mpi.eudico.client.annotator.gui.FileAndEncodingChooser;


public class ExportPraatDialog extends AbstractExtTierExportDialog 
    implements ActionListener, ItemListener, ListSelectionListener{
    private JCheckBox rootTiersCB;
    private JCheckBox selectionCB;
    private JCheckBox correctTimesCB;
    
    private List tierOrderList = null;
    
    
    /**
     * @param parent the parent frame
     * @param modal the modal flag
     * @param transcription the transcription
     */
    public ExportPraatDialog(Frame parent, boolean modal,
            Transcription transcription, Selection selection, List tierOrderList) {
        super(parent, modal, transcription, selection);
        if (selection != null) {
            System.out.println("sb: " + selection.getBeginTime() + " se: " + selection.getEndTime());
        }
        if(tierOrderList != null){
        	this.tierOrderList =  tierOrderList;
        }
        makeLayout();
        extractTiers();
        postInit();
    }

    /**
     * The action performed event handling.
     *
     * @param ae the action event
     */
    public void actionPerformed(ActionEvent ae) {
        super.actionPerformed(ae);
    }

    /**
     * The item state changed handling.
     *
     * @param ie the ItemEvent
     */
    public void itemStateChanged(ItemEvent ie) {
        extractTiers();
    }

    /**
     * Updates the checked state of the export checkboxes.
     *
     * @param lse the list selection event
     */
    public void valueChanged(ListSelectionEvent lse) {
        if ((model != null) && lse.getValueIsAdjusting()) {
            int b = lse.getFirstIndex();
            int e = lse.getLastIndex();
            int col = model.findColumn(EXPORT_COLUMN);

            for (int i = b; i <= e; i++) {
                if (tierTable.isRowSelected(i)) {
                    model.setValueAt(Boolean.TRUE, i, col);
                }
            }
        }
    }

    /**
     * Extract candidate tiers for export.
     */
    protected void extractTiers() {
        if (model != null) {
            for (int i = model.getRowCount() - 1; i >= 0; i--) {
                model.removeRow(i);
            }

            if (transcription != null) {      
            	            	
            	 boolean rootsOnly = rootTiersCB.isSelected();
            	
            	if (rootsOnly) {
            		 //read Preferences
                    Object useTyp = Preferences.get("ExportPraatDialog.ParentTierOrder", transcription);
                    if (useTyp instanceof ArrayList) {
                    	tierOrderList = (List)useTyp;	
                  	} else {
                  		tierOrderList = null;
                  	}
            	}else {
            		//read Preferences
                    Object useTyp = Preferences.get("ExportPraatDialog.TierOrder", transcription);
                    if (useTyp instanceof ArrayList) {
                    	tierOrderList = (List)useTyp;	
                  	}  else {
                  		tierOrderList = null;
                  	}
            	}
            	
            	if(tierOrderList == null){
            		tierOrderList = transcription.getTiers();
            	}
                
                TierImpl t = null;              

                for (int i = 0; i < tierOrderList.size(); i++) {
                	if(tierOrderList.get(i) instanceof String){
                		String tierName = (String)tierOrderList.get(i) ;
                		t = (TierImpl) transcription.getTierWithId(tierName);
                	} else if(tierOrderList.get(i) instanceof TierImpl){
                		t = (TierImpl) tierOrderList.get(i);
                	}               
                    
                	if(t != null){
                		if (rootsOnly) {
                			// add only root tiers
                			if (t.getParentTier() == null) {
                				model.addRow(new Object[] { Boolean.TRUE, t.getName() });
                			}
                		} else {
                			// add all
                			if (t.getParentTier() == null) {
                				model.addRow(new Object[] { Boolean.TRUE, t.getName() });
                			} else {
                				model.addRow(new Object[] { Boolean.FALSE, t.getName() });
                			}
                		}
                	}
                }
            }
            
            //read Preferences
            Object useTyp = Preferences.get("ExportPraatDialog.selectedTiers", transcription);
            if (useTyp instanceof ArrayList) {
            	loadTierPreferences((ArrayList)useTyp);        	
          	 } 

            if (model.getRowCount() > 1) {
                upButton.setEnabled(true);
                downButton.setEnabled(true);
            } else {
                upButton.setEnabled(false);
                downButton.setEnabled(false);
            }
        } else {
            upButton.setEnabled(false);
            downButton.setEnabled(false);
        }
    }

    
    /**
     * Initializes UI elements.
     */
    protected void makeLayout() {
        super.makeLayout();
        rootTiersCB = new JCheckBox();
        selectionCB = new JCheckBox();
        correctTimesCB = new JCheckBox();
        
        GridBagConstraints gridBagConstraints;

        rootTiersCB.setSelected(true);
        rootTiersCB.addItemListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        tierSelectionPanel.add(rootTiersCB, gridBagConstraints);

        model.setColumnIdentifiers(new String[] { EXPORT_COLUMN, TIER_NAME_COLUMN });

        tierTable.getColumn(EXPORT_COLUMN).setCellEditor(new DefaultCellEditor(
                new JCheckBox()));
        tierTable.getColumn(EXPORT_COLUMN).setCellRenderer(new CheckBoxTableCellRenderer());
        tierTable.getColumn(EXPORT_COLUMN).setMaxWidth(30);
        tierTable.setShowVerticalLines(false);
        tierTable.setTableHeader(null);
        tierTable.getSelectionModel().addListSelectionListener(this);
    
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = insets;
        optionsPanel.add(selectionCB, gridBagConstraints);
        
        gridBagConstraints.gridy = 1;
        optionsPanel.add(correctTimesCB, gridBagConstraints);

        setPreferredSetting();
        updateLocale();
    }
    
    /**
     * Starts the actual export after performing some checks.
     *
     * @return true if export succeeded, false oherwise
     * @see mpi.eudico.client.annotator.export.AbstractBasicExportDialog#startExport()
     */
    protected boolean startExport() throws IOException {
        List selectedTiers = getSelectedTiers();
        savePreferences();

        if (selectedTiers.size() == 0) {
            JOptionPane.showMessageDialog(this,
                ElanLocale.getString("ExportTradTranscript.Message.NoTiers"),
                ElanLocale.getString("Message.Warning"),
                JOptionPane.WARNING_MESSAGE);

            return false;
        }

        String[] encodings = new String[]{ElanLocale.getString("Button.Default"), 
        		FileAndEncodingChooser.UTF_8, FileAndEncodingChooser.UTF_16};
        
        // prompt for file name and location
        File exportFile = promptForFile(ElanLocale.getString(
                    "ExportPraatDialog.Title"), FileExtension.PRAAT_TEXTGRID_EXT,
                    ElanFileFilter.createFileFilter(ElanFileFilter.PRAAT_TEXTGRID_TYPE), 
                    true, encodings);

        if (exportFile == null) {
            return false;
        }
        long begin = 0l;
        long end = ELANCommandFactory.getViewerManager(transcription).getMasterMediaPlayer().getMediaDuration();
        boolean exportSelection = false;
        if (selectionCB.isSelected()) {
            if (selection != null && selection.getBeginTime() < selection.getEndTime()) {
                begin = selection.getBeginTime();
                end = selection.getEndTime();
                exportSelection = true;
            }
        }
        long mediaOffset = 0L;

        if (correctTimesCB.isSelected()) {
            Vector mds = transcription.getMediaDescriptors();

            if ((mds != null) && (mds.size() > 0)) {
                mediaOffset = ((MediaDescriptor) mds.get(0)).timeOrigin;
            }
        }
        
        PraatTGEncoderInfo encInfo = new PraatTGEncoderInfo(begin, end);
        encInfo.setEncoding(encoding);
        encInfo.setOffset(mediaOffset);
        encInfo.setExportSelection(exportSelection);
        
        PraatTextGridEncoder encoder = new PraatTextGridEncoder();
        encoder.encodeAndSave(transcription, encInfo, selectedTiers, 
                exportFile.getAbsolutePath());
        
        return true;
    }
    
    /**
     * Applies localized strings to the ui elements. For historic reasons the
     * string identifiers start with "TokenizeDialog"
     */
    protected void updateLocale() {
        super.updateLocale();
        setTitle(ElanLocale.getString("ExportPraatDialog.Title"));
        titleLabel.setText(ElanLocale.getString("ExportPraatDialog.Title"));
        rootTiersCB.setText(ElanLocale.getString(
                "ExportTradTranscript.Label.RootTiers"));
        selectionCB.setText(ElanLocale.getString("ExportDialog.Restrict"));
        correctTimesCB.setText(ElanLocale.getString("ExportDialog.CorrectTimes"));
    }
    
    /**
     * Intializes the dialogBox with the last preferred/ used settings 
     *
     */
    private void setPreferredSetting()
    {
    	Object useTyp = Preferences.get("ExportPraatDialog.rootTiersCB", null);
    
    	if(useTyp != null){
    		rootTiersCB.setSelected((Boolean)useTyp); 
    	}	
     
    	useTyp = Preferences.get("ExportPraatDialog.selectionCB", null);
    	if(useTyp != null){
    		selectionCB.setSelected((Boolean)useTyp); 
    	}  
    	
    	Object correctObj = Preferences.get("ExportPraatDialog.correctTimesCB", null);
    	if (correctObj instanceof Boolean) {
    		correctTimesCB.setSelected((Boolean) correctObj);
    	}
    }
    
    /**
     * Saves the preferred settings Used. 
     *
     */
    private void savePreferences(){
    	Preferences.set("ExportPraatDialog.rootTiersCB", rootTiersCB.isSelected(), null);    
    	Preferences.set("ExportPraatDialog.selectionCB", selectionCB.isSelected(), null);
    	Preferences.set("ExportPraatDialog.correctTimesCB", correctTimesCB.isSelected(), null);
    	Preferences.set("ExportPraatDialog.selectedTiers", getSelectedTiers(), transcription);
    	
    	boolean orderChanged = false;
    	List newOrderList = new ArrayList();      
    	for (int i = 0; i < model.getRowCount(); i++) {
            String tierName =   (String) model.getValueAt(i,1); 
            if(tierOrderList.get(i) instanceof String){        		
        		if(!tierName.equals((String)tierOrderList.get(i)) ){
                	orderChanged = true;
                }
        	} else if(tierOrderList.get(i) instanceof TierImpl){
        		if(!tierName.equals( ((TierImpl)tierOrderList.get(i)).getName())){
                	orderChanged = true;
                }
        	}            
            newOrderList.add(tierName);
    	}    	
    	
    	if(orderChanged){
    		if (rootTiersCB.isSelected()) {
    			Preferences.set("ExportPraatDialog.ParentTierOrder", newOrderList, transcription);
    		}
    		else {
    			Preferences.set("ExportPraatDialog.TierOrder", newOrderList, transcription);
    		}
    	} 
    }
}
