package mpi.eudico.client.annotator.export;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.Selection;

import mpi.eudico.client.annotator.commands.ELANCommandFactory;

import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.util.FileExtension;

import mpi.eudico.client.util.CheckBoxTableCellRenderer;
import mpi.eudico.client.util.EAF2SMIL;
import mpi.eudico.client.util.Transcription2QtSubtitle;

import mpi.eudico.server.corpora.clom.Transcription;

import mpi.eudico.server.corpora.clomimpl.abstr.MediaDescriptor;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import java.io.File;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import java.util.HashMap;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.xml.transform.TransformerException;

import org.xml.sax.SAXParseException;


/**
 * DOCUMENT ME! $Id: ExportQtSubtitleDialog.java 20536 2010-10-27 09:15:02Z aarsom $
 *
 * @author $Author: ericauer $
 * @version $Revision: 1.3 $
 */
public class ExportQtSubtitleDialog extends AbstractExtTierExportDialog
    implements ListSelectionListener, ChangeListener {
    private JCheckBox minimalDurCB;
    private JTextField minimalDurTF;
    private JCheckBox correctTimesCB;
    private JCheckBox mergeTiersCB;
    private JCheckBox recalculateTimesCB;
    
    private JButton fontSettingsButton;
    private HashMap fontSettingHashMap;
    
    private boolean smilExport;
    
    

    /**
     * DOCUMENT ME!
     *
     * @param parent
     * @param modal
     * @param transcription
     * @param selection
     */
    public ExportQtSubtitleDialog(Frame parent, boolean modal,
        Transcription transcription, Selection selection) {
    	this(parent,modal,transcription, selection, false);
    }
    
    /**
     * DOCUMENT ME!
     *
     * @param parent
     * @param modal
     * @param transcription
     * @param selection
     * @param smilExport
     */
    public ExportQtSubtitleDialog(Frame parent, boolean modal,
        Transcription transcription, Selection selection, boolean smilExport) {    	
    	super(parent, modal, transcription, selection);
    	this.smilExport = smilExport;
        makeLayout();
        extractTiers();
        postInit();
        restrictCheckBox.requestFocus();        
    }

    /**
     * Extract candidate tiers for export.
     */
    protected void extractTiers() {
        if (model != null) {
            for (int i = model.getRowCount() - 1; i >= 0; i--) {
                model.removeRow(i);
            }

            if (transcription != null) {
                Vector v = transcription.getTiers();
                TierImpl t;

                for (int i = 0; i < v.size(); i++) {
                    t = (TierImpl) v.get(i);

                    // add all
                    if (i == 0) {
                        model.addRow(new Object[] { Boolean.TRUE, t.getName() });
                    } else {
                        model.addRow(new Object[] { Boolean.FALSE, t.getName() });
                    }
                }
                
              //read Preferences
                Object useTyp = Preferences.get("ExportQtSubtitleDialog.selectedTiers", transcription);
                if (useTyp != null) {
                	loadTierPreferences((ArrayList)useTyp);        	
              	 }

                if (model.getRowCount() > 0) {
                    tierTable.setRowSelectionInterval(0, 0);
                }
            }
        }
    }

    /**
     * Initializes UI elements. Note: (for the time being) the checkbox column
     * indicating the selected state has been removed. It isn't that useful in
     * a single selection mode. The table could be replaced by a JList. When
     * the checkbox column would  be added again one of two things should
     * happen: either a custom tablecelleditor should be written for the
     * checkbox column or the valueChanged method should iterate over all rows
     * and uncheck all boxes, except the one for the selected roe.
     */
    protected void makeLayout() {
        super.makeLayout();
        model.setColumnIdentifiers(new String[] { EXPORT_COLUMN, TIER_NAME_COLUMN });

        tierTable.getColumn(EXPORT_COLUMN)
                 .setCellEditor(new DefaultCellEditor(new JCheckBox()));
        tierTable.getColumn(EXPORT_COLUMN)
                 .setCellRenderer(new CheckBoxTableCellRenderer());
        tierTable.getColumn(EXPORT_COLUMN).setMaxWidth(30);
        tierTable.setShowVerticalLines(false);
        tierTable.setTableHeader(null);
        //tierTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tierTable.getSelectionModel().addListSelectionListener(this);

        // options
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;

        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        optionsPanel.add(restrictCheckBox, gridBagConstraints);
        
        recalculateTimesCB = new JCheckBox();  
        recalculateTimesCB.setEnabled(false);
        gridBagConstraints.gridy = 1;     
        gridBagConstraints.insets = new Insets(4,22,4,6);
        optionsPanel.add(recalculateTimesCB, gridBagConstraints);

        correctTimesCB = new JCheckBox();
        correctTimesCB.setSelected(true);
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = insets;
        optionsPanel.add(correctTimesCB, gridBagConstraints);

        minimalDurCB = new JCheckBox();
        minimalDurCB.setSelected(false);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        optionsPanel.add(minimalDurCB, gridBagConstraints);

        minimalDurTF = new JTextField(6);
        minimalDurTF.setEnabled(false);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        optionsPanel.add(minimalDurTF, gridBagConstraints);

        mergeTiersCB = new JCheckBox();
        mergeTiersCB.setSelected(true);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        optionsPanel.add(mergeTiersCB, gridBagConstraints);        
        
        fontSettingsButton = new JButton();
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        optionsPanel.add(fontSettingsButton, gridBagConstraints);
        fontSettingsButton.addActionListener(this); 
        
        minimalDurCB.addChangeListener(this);
        restrictCheckBox.addChangeListener(this);
        setPreferredSetting();

        updateLocale();
    }

    /**
     * @see mpi.eudico.client.annotator.export.AbstractTierExportDialog#startExport()
     */
    protected boolean startExport() throws IOException {
        List selectedTiers = getSelectedTiers();
        savePreferences();
        
        if (selectedTiers.size() == 0) {
            JOptionPane.showMessageDialog(this,
                ElanLocale.getString("ExportTradTranscript.Message.NoTiers"),
                ElanLocale.getString("Message.Warning"),
                JOptionPane.WARNING_MESSAGE);

            return false;
        }

        int minimalDur = 0;

        if (minimalDurCB.isSelected()) {
            String dur = minimalDurTF.getText();

            if ((dur == null) || (dur.length() == 0)) {
                JOptionPane.showMessageDialog(this,
                    ElanLocale.getString("ExportDialog.Message.InvalidNumber"),
                    ElanLocale.getString("Message.Warning"),
                    JOptionPane.WARNING_MESSAGE);

                minimalDurTF.requestFocus();

                return false;
            }

            try {
                minimalDur = Integer.parseInt(dur);
            } catch (NumberFormatException nfe) {
                JOptionPane.showMessageDialog(this,
                    ElanLocale.getString("ExportDialog.Message.InvalidNumber"),
                    ElanLocale.getString("Message.Warning"),
                    JOptionPane.WARNING_MESSAGE);

                minimalDurTF.requestFocus();

                return false;
            }
        }
        
     // prompt for file name and location
        File exportFile;
        if(smilExport){        	
            exportFile = promptForFile(ElanLocale.getString(
                    "ExportQtSmilDialog.Title "), FileExtension.SMIL_EXT, 
                    ElanFileFilter.createFileFilter(ElanFileFilter.SMIL_TYPE), false);
            
            if (exportFile == null) {
                return false;
            }
        
        }else{        
          exportFile = promptForFile(ElanLocale.getString(
                    "ExportQtSubtitleDialog.Title"), FileExtension.TEXT_EXT,
                ElanFileFilter.createFileFilter(ElanFileFilter.TEXT_TYPE), false);
        }

        if (exportFile == null) {
            return false;
        }

        long offset = 0L;

        if (correctTimesCB.isSelected()) {
            Vector mediaDescriptors = transcription.getMediaDescriptors();

            if (mediaDescriptors.size() > 0) {
                offset = ((MediaDescriptor) mediaDescriptors.get(0)).timeOrigin;
            }
        }

        boolean merge = mergeTiersCB.isSelected();
        long mediaDur = ELANCommandFactory.getViewerManager(transcription)
                                          .getMasterMediaPlayer()
                                          .getMediaDuration();

        String[] tierNames = (String[]) selectedTiers.toArray(new String[0]);
        long b = 0L;
        long e = Long.MAX_VALUE;

        if (restrictCheckBox.isSelected()) {
            b = selection.getBeginTime();
            e = selection.getEndTime();
        } 
        
        String mediaURL = "";        
        if ( ((TranscriptionImpl) transcription).getMediaDescriptors().size() > 0) {
            mediaURL = ((MediaDescriptor) ((TranscriptionImpl) transcription).getMediaDescriptors().
            get(0)).mediaURL;
            
        }    

        if (!merge) {
             Transcription2QtSubtitle.exportTiers(transcription, tierNames,
                exportFile, b, e, offset, minimalDur, mediaDur, recalculateTimesCB.isSelected(), fontSettingHashMap);            
        } else {
            Transcription2QtSubtitle.exportTiersMerged(transcription,
                tierNames, exportFile, b, e, offset, minimalDur, mediaDur, recalculateTimesCB.isSelected(), fontSettingHashMap);
        }  
        
        // .sml output for Smil Quicktime Export
        if(smilExport){
        	
        	 String smilFile = exportFile.getAbsolutePath();
             int index = smilFile.lastIndexOf('.');
             if (index > 0) {
             	smilFile = smilFile.substring(0, index);
             }             
             smilFile += "." + FileExtension.SMIL_EXT[1];
             
             String mediaPath ="";
             if(mediaURL.length() > 0){ 
             	index = mediaURL.lastIndexOf("/");
             	mediaPath = mediaURL.substring(index+1);       	
             }
        
             try {	        
            	 if (selection != null && restrictCheckBox.isSelected()) {            		 
            		 ExportQtSmilDialog.export2SMILQt(	new File(((TranscriptionImpl) transcription).getPathName()),
            					 new File(smilFile), tierNames,	mediaPath, b+offset, e+offset, recalculateTimesCB.isSelected(), merge, fontSettingHashMap);
            	 } else {
            		 ExportQtSmilDialog.export2SMILQt(	new File(((TranscriptionImpl) transcription).getPathName()), 
						new File(smilFile), tierNames, mediaPath, mediaDur, merge, fontSettingHashMap);
            	 }
            	 return true;
             }
             catch (TransformerException te) {
            	 // this is ugly
            	 throw new IOException("TransformerException: " + te.getMessage());        	
             }
        	}
        
        	return true;
    }

    /**
     * @see mpi.eudico.client.annotator.export.AbstractTierExportDialog#updateLocale()
     */
    protected void updateLocale() {
        super.updateLocale();
        setTitle(ElanLocale.getString("ExportQtSubtitleDialog.Title"));
        titleLabel.setText(ElanLocale.getString(
                "ExportQtSubtitleDialog.TitleLabel"));
        correctTimesCB.setText(ElanLocale.getString("ExportDialog.CorrectTimes"));
        minimalDurCB.setText(ElanLocale.getString(
                "ExportDialog.Label.MinimalDur"));
        mergeTiersCB.setText(ElanLocale.getString(
                "ExportQtSubtitleDialog.Label.Merge"));
       fontSettingsButton.setText(ElanLocale.getString(
        			"ExportQtSubtitleDialog.Button.FontSetting"));  
       recalculateTimesCB.setText(ElanLocale.getString("ExportDialog.RecalculateTimes"));
       if(smilExport){
    	   setTitle(ElanLocale.getString("ExportQtSmilDialog.Title"));
           titleLabel.setText(ElanLocale.getString(
                   "ExportQtSmilDialog.TitleLabel"));   	   
    	   
       }
    }

    /**
     * Updates the checked state of the export checkboxes.
     *
     * @param lse the list selection event
     */
    public void valueChanged(ListSelectionEvent lse) {
        if ((model != null) && lse.getValueIsAdjusting()) {
            int b = lse.getFirstIndex();
            int e = lse.getLastIndex();
            int col = model.findColumn(EXPORT_COLUMN);

            for (int i = b; i <= e; i++) {
                if (tierTable.isRowSelected(i)) {
                    model.setValueAt(Boolean.TRUE, i, col);
                }
            }
        }
    }

    /**
     * Enables/disables the minimal duration textfield and the recalculate time check box
     *
     * @param e the change event
     */
    public void stateChanged(ChangeEvent e) {
        if (e.getSource() == minimalDurCB) {
            minimalDurTF.setEnabled(minimalDurCB.isSelected());
        }else if (e.getSource() == restrictCheckBox){
        	recalculateTimesCB.setEnabled(restrictCheckBox.isSelected());
        }
    }
    
    /**
     * The action performed event handling.
     *
     * @param ae the action event
     */
    public void actionPerformed(ActionEvent ae) {
    	super.actionPerformed(ae);
    	if(ae.getSource() == fontSettingsButton){
    		setNewFontSetting(DisplaySettingsPane.getNewFontSetting(this, ElanLocale.getString("DisplaySettingsPane.Title")));    		
    		}
    	}
    
    
	protected void setNewFontSetting(HashMap newSetting){
		if (newSetting != null){
			fontSettingHashMap = new HashMap(); 
			fontSettingHashMap = newSetting;
		}
	}
    /**
     * 
     * Intializes the dialogBox with the last preferred/ used settings 
     *
     */
    protected void setPreferredSetting()
    {
    	Object useTyp = Preferences.get("ExportQtSubtitleDialog.restrictCheckBox", null);
    
    	if(useTyp != null){
    		restrictCheckBox.setSelected((Boolean)useTyp); 
    	}	
     
    	useTyp = Preferences.get("ExportQtSubtitleDialog.minimalDurCB", null);
    	if(useTyp != null){
    		minimalDurCB.setSelected((Boolean)useTyp); 
    	}
     
    	useTyp = Preferences.get("ExportQtSubtitleDialog.correctTimesCB", null);
    	if(useTyp != null){
    		correctTimesCB.setSelected((Boolean)useTyp); 
    	}
     
    	useTyp = Preferences.get("ExportQtSubtitleDialog.mergeTiersCB", null);
    	if(useTyp != null){
    		mergeTiersCB.setSelected((Boolean)useTyp); 
    	}
    	
    	useTyp = Preferences.get("ExportQtSubtitleDialog.minimalDurTF", null);
    	if(useTyp != null){
    		minimalDurTF.setText(useTyp.toString()); 
    	}
    	
    	useTyp = Preferences.get("ExportQtSubtitleDialog.recalculateTimesCB", null);
    	if(useTyp != null){
    		recalculateTimesCB.setSelected((Boolean)useTyp);    		
    	}
    }
    
    /**
     * Saves the preferred settings Used. 
     *
     */
    protected void savePreferences(){
    	Preferences.set("ExportQtSubtitleDialog.restrictCheckBox", restrictCheckBox.isSelected(), null);    
    	Preferences.set("ExportQtSubtitleDialog.minimalDurCB", minimalDurCB.isSelected(), null);
    	Preferences.set("ExportQtSubtitleDialog.correctTimesCB", correctTimesCB.isSelected(), null);
    	Preferences.set("ExportQtSubtitleDialog.mergeTiersCB", mergeTiersCB.isSelected(), null);   
    	Preferences.set("ExportQtSubtitleDialog.recalculateTimesCB", recalculateTimesCB.isSelected(), null);
    	if (minimalDurTF.getText() != null)
    		Preferences.set("ExportQtSubtitleDialog.minimalDurTF", minimalDurTF.getText(), null);    	
    	Preferences.set("ExportQtSubtitleDialog.selectedTiers", getSelectedTiers(), transcription);
    }
}
