package mpi.eudico.client.annotator.gui;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.tier.TierExportTableModel;
import mpi.eudico.client.util.CheckBoxTableCellRenderer;

/**
 * A dialog for sorting and selecting tiers.
 * Does not work on a Transcription object but rather on lists of tiers
 * @author Han Sloetjes
 */
public class TierSortAndSelectDialog extends ClosableDialog implements ActionListener,
ListSelectionListener {
	private List<String> tierOrder;
	private List<String> selectedTiers;
	
    /** panel for start and close buttons (bottom component) */
    protected JPanel buttonPanel;
    /** close button */
    private JButton cancelButton;

    /** start export button */
    private JButton okButton;
	protected JPanel tierButtonPanel;
	protected JButton downButton;
	protected JButton upButton;
	protected JButton allButton;
	protected JButton noneButton;
    /** table for tiers */
    protected DefaultTableModel model;
    /** panel for a tier table */
    protected JPanel tierSelectionPanel;

    /** table ui */
    protected JTable tierTable;
    /** column id for the include in export checkbox column, invisible */
    protected final String SELECT_COLUMN = "select";

    /** column id for the tier name column, invisible */
    protected final String TIER_NAME_COLUMN = "tier";
    
    private List<String> returnedTiers = null;
    
    /**
     * Constructor.
     * @param owner the parent window
     * @param tierOrder current, initial tier order
     * @param selectedTiers current, initial selected tiers
     * @throws HeadlessException
     */
	public TierSortAndSelectDialog(Frame owner, List<String> tierOrder,
			List<String> selectedTiers) throws HeadlessException {
		super(owner);
		this.tierOrder = tierOrder;
		this.selectedTiers = selectedTiers;
		initComponents();
	}

	/**
     * Constructor.
     * @param owner the parent window
     * @param modal the modal flag
     * @param tierOrder current, initial tier order
     * @param selectedTiers current, initial selected tiers
	 * @throws HeadlessException
	 */
	public TierSortAndSelectDialog(Frame owner, boolean modal, List<String> tierOrder,
			List<String> selectedTiers)
			throws HeadlessException {
		super(owner, modal);
		this.tierOrder = tierOrder;
		this.selectedTiers = selectedTiers;
		initComponents();
	}

	/**
     * Constructor.
     * @param owner the parent window
     * @param tierOrder current, initial tier order
     * @param selectedTiers current, initial selected tiers
	 * @throws HeadlessException
	 */
	public TierSortAndSelectDialog(Dialog owner, List<String> tierOrder,
			List<String> selectedTiers) throws HeadlessException {
		super(owner);
		this.tierOrder = tierOrder;
		this.selectedTiers = selectedTiers;
		initComponents();
	}

	/**
     * Constructor.
     * @param owner the parent window
     * @param modal the modal flag
     * @param tierOrder current, initial tier order
     * @param selectedTiers current, initial selected tiers
	 * @throws HeadlessException
	 */
	public TierSortAndSelectDialog(Dialog owner, boolean modal, List<String> tierOrder,
			List<String> selectedTiers)
			throws HeadlessException {
		super(owner, modal);
		this.tierOrder = tierOrder;
		this.selectedTiers = selectedTiers;
		initComponents();
	}

	/**
	 * Initializes panes, buttons and tier table 
	 */
	private void initComponents() {
		getContentPane().setLayout(new GridBagLayout());
		Insets insets = new Insets(4, 6, 4, 6);
		
		model = new TierExportTableModel();
		tierTable = new JTable(model);
		
        model.setColumnIdentifiers(new String[] { SELECT_COLUMN, TIER_NAME_COLUMN });
        tierTable.getColumn(SELECT_COLUMN)
                 .setCellEditor(new DefaultCellEditor(new JCheckBox()));
        tierTable.getColumn(SELECT_COLUMN)
                 .setCellRenderer(new CheckBoxTableCellRenderer());
        tierTable.getColumn(SELECT_COLUMN).setMaxWidth(30);
        tierTable.setShowVerticalLines(false);
        tierTable.setTableHeader(null);
        tierTable.getSelectionModel().addListSelectionListener(this);

        if (tierOrder != null) {
            for (String name : tierOrder) {
                if (selectedTiers != null) {
                    if (selectedTiers.contains(name)) {
                        model.addRow(new Object[] { Boolean.TRUE, name });
                    } else {
                        model.addRow(new Object[] { Boolean.FALSE, name });
                    }
                } else {
                    model.addRow(new Object[] { Boolean.TRUE, name });
                }
            }
        }
        
		tierSelectionPanel = new JPanel(new GridBagLayout());
		 GridBagConstraints gridBagConstraints;
		Dimension tableDim = new Dimension(50, 100);

        JScrollPane tierScrollPane = new JScrollPane(tierTable);
        tierScrollPane.setPreferredSize(tableDim);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = insets;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        tierSelectionPanel.add(tierScrollPane, gridBagConstraints);

        // add more elements to this panel
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = insets;
        getContentPane().add(tierSelectionPanel, gridBagConstraints);
		
        // add more 
        upButton = new JButton();
        downButton = new JButton();
        allButton = new JButton(ElanLocale.getString("Button.SelectAll"));
        noneButton = new JButton(ElanLocale.getString("Button.SelectNone"));
        tierButtonPanel = new JPanel(new GridBagLayout());
        
        try {
            ImageIcon upIcon = new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/navigation/Up16.gif"));
            ImageIcon downIcon = new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/navigation/Down16.gif"));
            upButton.setIcon(upIcon);
            downButton.setIcon(downIcon);
        } catch (Exception ex) {
            upButton.setText("Up");
            downButton.setText("Down");
        }
        upButton.addActionListener(this);
        downButton.addActionListener(this);
        allButton.addActionListener(this);
        noneButton.addActionListener(this);
		
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        tierButtonPanel.add(upButton, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = insets;
        tierButtonPanel.add(downButton, gridBagConstraints);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.insets = new Insets(4, 20, 4, 2);
        tierButtonPanel.add(allButton, gridBagConstraints);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.insets = insets;
        tierButtonPanel.add(noneButton, gridBagConstraints);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        tierSelectionPanel.add(tierButtonPanel, gridBagConstraints);
        
        cancelButton = new JButton(ElanLocale.getString("Button.Cancel"));
        cancelButton.addActionListener(this);
        okButton = new JButton(ElanLocale.getString("Button.OK"));
        okButton.addActionListener(this);
        buttonPanel = new JPanel(new GridLayout(1, 2, 6, 0));
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = insets;
        getContentPane().add(buttonPanel, gridBagConstraints);
        
        pack();
        int minW = 300;
        int minH = 400;
        setSize(Math.max(minW, getWidth()), Math.max(minH, getHeight()));
	}
	
	/**
	 * Returns the selected tiers.
	 * 
	 * @return a list of selected tiers
	 */
    private List<String> getSelectedTiers() {
        int includeCol = model.findColumn(SELECT_COLUMN);
        int nameCol = model.findColumn(TIER_NAME_COLUMN);

        ArrayList<String> selectedTiers = new ArrayList<String>();

        // add selected tiers in the right order
        for (int i = 0; i < model.getRowCount(); i++) {
            Boolean include = (Boolean) model.getValueAt(i, includeCol);

            if (include.booleanValue()) {
                selectedTiers.add((String) model.getValueAt(i, nameCol));
            }
        }

        return selectedTiers;
    }
    
    /**
     * Returns the selected tiers.
     *
     * @return the selected tiers
     */
    public List<String> getValue() {
        return returnedTiers;
    }
    
    /**
     * Returns all tiers in the current order.
     * 
     * @return the list of all tiers
     */
    public List<String> getTierOrder() {
    	int nameCol = model.findColumn(TIER_NAME_COLUMN);
    	ArrayList<String> orderedTiers = new ArrayList<String>();
    	
        for (int i = 0; i < model.getRowCount(); i++) {
        	orderedTiers.add((String) model.getValueAt(i, nameCol));
        }
        
    	return orderedTiers;
    }
    
    /**
     * Moves selected tiers up in the list of tiers.
     */
    protected void moveDown() {
        if ((tierTable == null) || (model == null) ||
                (model.getRowCount() < 2)) {
            return;
        }

        int[] selected = tierTable.getSelectedRows();

        for (int i = selected.length - 1; i >= 0; i--) {
            int row = selected[i];

            if ((row < (model.getRowCount() - 1)) &&
                    !tierTable.isRowSelected(row + 1)) {
                model.moveRow(row, row, row + 1);
                tierTable.changeSelection(row, 0, true, false);
                tierTable.changeSelection(row + 1, 0, true, false);
            }
        }
    }

    /**
     * Moves selected tiers up in the list of tiers.
     */
    protected void moveUp() {
        if ((tierTable == null) || (model == null) ||
                (model.getRowCount() < 2)) {
            return;
        }

        int[] selected = tierTable.getSelectedRows();

        for (int i = 0; i < selected.length; i++) {
            int row = selected[i];

            if ((row > 0) && !tierTable.isRowSelected(row - 1)) {
                model.moveRow(row, row, row - 1);
                tierTable.changeSelection(row, 0, true, false);
                tierTable.changeSelection(row - 1, 0, true, false);
            }
        }
    }

    /**
     * The action event handling.
     */
	public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == allButton) {
            if (model != null) {
                for (int i = 0; i < tierTable.getRowCount(); i++) {
                    model.setValueAt(Boolean.TRUE, i, 0);
                }
            }
        } else if (ae.getSource() == noneButton) {
            if (model != null) {
                for (int i = 0; i < tierTable.getRowCount(); i++) {
                    model.setValueAt(Boolean.FALSE, i, 0);
                }
            }
        } else if (ae.getSource() == upButton) {
            moveUp();
        } else if (ae.getSource() == downButton) {
            moveDown();
        } else if (ae.getSource() == cancelButton) {
            setVisible(false);
            dispose();
        } else if (ae.getSource() == okButton) {
            returnedTiers = getSelectedTiers();
            setVisible(false);
            dispose();
        }
		
	}

	/**
	 * Selection of rows in the table.
	 */
	public void valueChanged(ListSelectionEvent lse) {
        if ((model != null) && lse.getValueIsAdjusting()) {
            int b = lse.getFirstIndex();
            int e = lse.getLastIndex();
            int col = model.findColumn(SELECT_COLUMN);

            for (int i = b; i <= e; i++) {
                if (tierTable.isRowSelected(i)) {
                    model.setValueAt(Boolean.TRUE, i, col);
                }
            }
        }
	}

}
