package mpi.eudico.client.annotator.tier;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.commands.global.MultiFileAnnotationsFromOverlapsCommand;
import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.gui.ClosableDialog;
import mpi.eudico.client.annotator.gui.MFDomainDialog;
import mpi.eudico.client.annotator.gui.MultiFileChooser;
import mpi.eudico.client.annotator.gui.multistep.MultiStepPane;
import mpi.eudico.client.annotator.gui.multistep.StepPane;

import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clomimpl.type.LinguisticType;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;

/**
 * A dialog that contains the options to create annotations based on the selected tiers
 * @author Jeffrey Lemein
 * @version March, 2011
 */
public class AnnotationsFromOverlapsDialog implements ClientLogger {
	private TranscriptionImpl transcription;
	private MultiStepPane multiPane;
	private Insets globalInset, singleTabInset, firstSingleTabInset, doubleTabInset;
	private JFrame parentFrame;
	private JDialog stepDialog;
	private boolean atLastStep;
	
	//global
	private List<TranscriptionImpl> openedFileList;
	private FileTierSelectionPanel step1Panel;
	private OverlapsComputationCriteriaPanel step2Panel;
	private DestinationTierPanel step3Panel;
	private DestinationValuePanel step4Panel;
	
	//step 1
	private JTable step1TierTable;
	private Set<String> tierSet;
	//private Set<String> selectedTiers;
	private JRadioButton currentlyOpenedFileRB, selectedFilesFromDiskRB, filesFromDomainRB;
	
	//step 2
	private JRadioButton overlapRB, valuesTheSameRB, valuesDifferentRB, valuesMatchConstraintsRB;
	private JTable tierValueTable;
	
	//step 3
	private JTextField newTierNameField;
	private JRadioButton rootTierRB, childTierRB;
	private JComboBox parentTierCB;
	private JTable linguisticTypeTable;
	
	//step 4
	private JRadioButton durationRadioButton, specificValueRB, tierValueRadioButton, concatValuesRB, smpteRB, palRB, ntscRB;
	private JRadioButton msecRB, secRB, hrRB;
	private JTextField specificValueTF;
	private JComboBox tierSelectBox;
	
	/**
	 * Constructor
	 * @param parentFrame 	the frame where this dialog belongs to
	 * @param transcription the transcription of where the overlaps are computed for
	 */
	public AnnotationsFromOverlapsDialog( JFrame parentFrame, TranscriptionImpl transcription ){		
		this.parentFrame = parentFrame;
		this.transcription = transcription;
		
		atLastStep = false;
		
		globalInset = new Insets(5, 10, 5, 10);
		singleTabInset = new Insets(0, 30, 0, 10);
		doubleTabInset = new Insets(0, 50, 0, 10);
		firstSingleTabInset = new Insets(5, 10, 0, 10);
		
		//create set of tier names
		if(transcription != null){
			tierSet = new TreeSet<String>();
			Vector<TierImpl> tiers = transcription.getTiers();
			for( TierImpl tier : tiers )
				tierSet.add(tier.getName());
		}
		
		//multistep pane
		multiPane = new MultiStepPane();
		
		step1Panel = new FileTierSelectionPanel(multiPane);
		step2Panel = new OverlapsComputationCriteriaPanel(multiPane);
		step3Panel = new DestinationTierPanel(multiPane);
		step4Panel = new DestinationValuePanel(multiPane);
		
		multiPane.addStep(step1Panel);
		multiPane.addStep(step2Panel);
		multiPane.addStep(step3Panel);
		multiPane.addStep(step4Panel);
		
		stepDialog = multiPane.createDialog(this.parentFrame, ElanLocale.getString("OverlapsDialog.Title"), true);
		stepDialog.setPreferredSize(new Dimension(600, 600));
		//stepDialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
		
		stepDialog.pack();
		//stepDialog.setLocationRelativeTo(parentFrame);
		stepDialog.setVisible(true);
	}

	/**
	 * Shows a multiple file chooser dialog, checks if every selected file exists
	 * and stores the selected files in private variable eafFiles
	 * @return boolean to indicate if file selection went successful
	 */
	private List<String> showMultiFileChooser(){
		//retrieve last used directory or use default one
        String dirPath = (String) Preferences.get("LastUsedEAFDir", null);

        if (dirPath == null)
            dirPath = System.getProperty("user.dir");

        //create a multifile chooser and set relevant attributes
        MultiFileChooser multiFileChooser = new MultiFileChooser();   
        multiFileChooser.setCurrentDirectory( new File(dirPath));
        multiFileChooser.setDialogTitle( ElanLocale.getString("Frame.ElanFrame.OpenDialog.Title"));
        
        //only accept eaf file types
        multiFileChooser.setFileFilter( ElanFileFilter.createFileFilter( ElanFileFilter.EAF_TYPE ));
		
		//show multiple file chooser and store result
        int returnValue = multiFileChooser.showDialog(parentFrame, ElanLocale.getString("Button.OK"));

		if (returnValue == JFileChooser.APPROVE_OPTION) {
			//get the selected files that the user has chosen in the multiple file dialog
            Object[] objects = multiFileChooser.getFiles();
            
            //convert all objects to the file array and checks if all files exist
            //this extra step is done because mutliFileChooser.getFiles()
            //can't be casted to a File array directly (ClassCastException)
            List<String> filenames = new ArrayList<String>();
            
            //assume that all selected files exist
            boolean allFilesExist = true;
            
            for( int i=0; i<objects.length; i++ ){
            	File file = (File) objects[i];
            	
            	//check if file exists
            	if( file.exists() )
            		filenames.add(file.getAbsolutePath());
            	else
            		//flag that one of the files does not exist
            		allFilesExist = false;
            }
            
            
            if( !allFilesExist ){
            	//ask if user wants to continue with exporting, if some of the files doesn't exist.
            	int answer = JOptionPane.showConfirmDialog(parentFrame, ElanLocale.getString("OverlapsDialog.Message.NotAllFilesExist"), ElanLocale.getString("OverlapsDialog.Message.NotAllFilesExistTitle"), JOptionPane.YES_NO_OPTION );
            	
            	//stop with exporting if user doesn't want to continue
            	if( answer != JOptionPane.YES_OPTION )
            		return null;
            }

            if( filenames.isEmpty() )
            	return null;
            else{
            	//store last used directory
                Preferences.set("LastUsedEAFDir", (new File( filenames.get(filenames.size()-1) )).getParent(), null);
            }
            
            return filenames;
        }
		
		return null;
	}

	
	/**
	 * Retrieves all necessary information to be used for calculating the overlaps.
	 * After retrieving all information it calls the overlaps command to begin calculating overlaps.
	 */
	private void executeOverlapsCommand(){
		SelectableContentTableModel model = (SelectableContentTableModel) step1TierTable.getModel();
		
		//retrieve selected tier names
		Vector selectedValues = model.getSelectedValues();
		String sourceTiers[] = new String[selectedValues.size()];
		for( int i=0; i<selectedValues.size(); i++ )
			sourceTiers[i] = (String) selectedValues.get(i);
		
		//retrieve overlaps criteria
		int overlapsCriteria = 0;
		if( overlapRB.isSelected() )
			overlapsCriteria = 0;
		else if( valuesTheSameRB.isSelected() )
			overlapsCriteria = 1;
		else if( valuesDifferentRB.isSelected() )
			overlapsCriteria = 2;
		else if( valuesMatchConstraintsRB.isSelected() )
			overlapsCriteria = 3;
		
		//retrieve specific settings
		List<String[]> tierValuePairs = new ArrayList<String[]>();
		DisplayableContentTableModel m = (DisplayableContentTableModel) tierValueTable.getModel();
		
		for( int i=0; i<m.getRowCount(); i++ )
			tierValuePairs.add(new String[]{(String)m.getValueAt(i, 1), (String)m.getValueAt(i, 3)});
		
		//retrieve name for destination tier
		String destTierName = newTierNameField.getText().trim();
		
		//retrieve linguistic type
		int selectedRow = linguisticTypeTable.getSelectedRow();
		String linguisticType;
		
		linguisticType = (String)linguisticTypeTable.getModel().getValueAt(selectedRow, 0);
		LinguisticType destLingType;
		
		if( transcription != null )
			destLingType = transcription.getLinguisticTypeByName(linguisticType);
		else
			destLingType = openedFileList.get(0).getLinguisticTypeByName(linguisticType);
		
		if(destLingType == null){
			LOG.severe("Selected linguistic type can not be identified");
		}
		
		//retrieve annotation value type
		int annotationValueType = -1;
		if( durationRadioButton.isSelected() )
			annotationValueType = 0;
		else if( specificValueRB.isSelected() )
			annotationValueType = 1;
		else if( tierValueRadioButton.isSelected() )
			annotationValueType = 2;
		else if(concatValuesRB.isSelected())
			annotationValueType = 3;
		//parent tier
		Tier parentTier = null;
		String parentTierName = "";
		if( childTierRB.isSelected() ){
			parentTierName = (String) parentTierCB.getSelectedItem();
			//parentTier = transcription.getTierWithId(tierName);
		}
		
		//store time format
		String timeFormat = null;
		boolean usePalFormat = true;
		
		if( msecRB.isSelected() ) 
			timeFormat = Constants.MS_STRING; //msec
		else if( secRB.isSelected() ) 
			timeFormat = Constants.SSMS_STRING; //ss:msec
		else if( hrRB.isSelected() ) 
			timeFormat = Constants.HHMMSSMS_STRING; //hh:mm:ss.ms
		else if( smpteRB.isSelected() && palRB.isSelected() )
		{
			timeFormat = Constants.HHMMSSFF_STRING; //SMPTE (PAL)
			usePalFormat = true;
		}
		else if( smpteRB.isSelected() && ntscRB.isSelected() )
		{
			timeFormat = Constants.HHMMSSFF_STRING; //SMPTE (NTSC)
			usePalFormat = false;
		}
		//retrieve annotation value
		String annWithValue = specificValueTF.getText().trim();
		
		//annotation from specific tier name
		String annFromTier = tierSelectBox.getSelectedItem().toString();
		
		int transcriptionMode;
		if( currentlyOpenedFileRB.isSelected() )
			transcriptionMode = 0;
		else
			transcriptionMode = 1;
		
		Object[] filenames;
		if( openedFileList != null )
			filenames = openedFileList.toArray();
		else
			filenames = null;
		
		Object args[] = new Object[]{ sourceTiers, destTierName, destLingType,
									annotationValueType, timeFormat, annWithValue, annFromTier, 
									overlapsCriteria, tierValuePairs, transcriptionMode, filenames,
									usePalFormat, parentTierName };
		
		if( transcription != null ){
			Command com = ELANCommandFactory.createCommand(transcription, ELANCommandFactory.ANN_FROM_OVERLAP);
			com.execute(transcription, args);
		}
		else{
			MultiFileAnnotationsFromOverlapsCommand command = new MultiFileAnnotationsFromOverlapsCommand();
			command.execute(transcription, args);
		}
		
        
	}
	
	/**
	 * Panel for step 1: File and tier selection
	 * @author Jeffrey Lemein
	 * @version July, 2010
	 */
	private class FileTierSelectionPanel extends StepPane{		
		//file selection components
		JPanel fileSelectionPanel;
		ButtonGroup buttonGroup;
		JButton selectFilesBtn, selectDomainBtn;
		
		//tier selection components
		JPanel tierSelectionPanel, buttonPanel;
		JButton selectAllButton, selectNoneButton;
		JScrollPane tierTableScrollPane;
		
		/**
		 * Constructor
		 */
		public FileTierSelectionPanel(MultiStepPane mp){
			super(mp);
			initComponents();
		}
		
		@Override
		protected void initComponents(){			
			//initialize 
			initFileSelectionPanel();
			initTierSelectionPanel();
			
			setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.insets = globalInset;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.fill = GridBagConstraints.BOTH;
			gbc.anchor = GridBagConstraints.CENTER;
			add(fileSelectionPanel, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 1;
			gbc.weightx = 1.0;
			gbc.weighty = 1.0;
			gbc.fill = GridBagConstraints.BOTH;
			gbc.anchor = GridBagConstraints.CENTER;
			add(tierSelectionPanel, gbc);
		}
		
		@Override
		public String getStepTitle(){
			return ElanLocale.getString("OverlapsDialog.Title.Step1Title");
		}
		
		@Override
		public void enterStepForward(){
			updateButtonStates();
		}
		
		@Override
		public void enterStepBackward(){
			updateButtonStates();
		}
		
		@Override
		public boolean leaveStepForward(){
			step2Panel.updateTierValues();
			step3Panel.updateTierValues();
			step4Panel.updateValues();
			return true;
		}
		/**
		 * Initializes the upper part containing file selection
		 */
		private void initFileSelectionPanel(){			
			//panel
			fileSelectionPanel = new JPanel(new GridBagLayout());
			fileSelectionPanel.setBorder(new TitledBorder( ElanLocale.getString("OverlapsDialog.Panel.Title.FileSelection")));
			
			//create all radio buttons
			RadioButtonHandler radioButtonListener = new RadioButtonHandler();
			currentlyOpenedFileRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.CurrentlyOpenedFile"));
			currentlyOpenedFileRB.addActionListener(radioButtonListener);
			selectedFilesFromDiskRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.FilesFromFileBrowser"));
			selectedFilesFromDiskRB.addActionListener(radioButtonListener);
			filesFromDomainRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.FilesFromDomain"));;
			filesFromDomainRB.addActionListener(radioButtonListener);
			
			//add radio buttons to button group
			buttonGroup = new ButtonGroup();
			buttonGroup.add(currentlyOpenedFileRB);
			buttonGroup.add(selectedFilesFromDiskRB);
			buttonGroup.add(filesFromDomainRB);
			
			//create all buttons
			ButtonHandler buttonHandler = new ButtonHandler();
			
			selectFilesBtn = new JButton(ElanLocale.getString("Button.Browse"));
			selectFilesBtn.addActionListener(buttonHandler);
			
			selectDomainBtn = new JButton(ElanLocale.getString("OverlapsDialog.Button.Domain"));
			selectDomainBtn.addActionListener(buttonHandler);
			selectDomainBtn.setEnabled(false);
			
			//handle multiple file case vs. single file case
			if( transcription == null ){
				//MULTIPLE_FILES: disable all radio buttons dealing with single file
				currentlyOpenedFileRB.setEnabled(false);
				selectedFilesFromDiskRB.setEnabled(true);
				selectedFilesFromDiskRB.setSelected(true);
				selectFilesBtn.setEnabled(true);
			}else{
				//SINGLE FILES: disable all multiple file functionality
				selectedFilesFromDiskRB.setEnabled(false);
				filesFromDomainRB.setEnabled(false);
				selectFilesBtn.setEnabled(false);
				selectDomainBtn.setEnabled(false);
				
				currentlyOpenedFileRB.setSelected(true);
			}
			
			
			//add buttons to panel
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 2;
			gbc.gridheight = 1;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.insets = singleTabInset;			
			fileSelectionPanel.add(currentlyOpenedFileRB, gbc);
			
			//files from disk
			gbc.gridy = 1;
			gbc.gridwidth = 1;
			fileSelectionPanel.add(selectedFilesFromDiskRB, gbc);
			gbc.gridx = 1;
			gbc.weightx = 0.0;
			fileSelectionPanel.add(selectFilesBtn, gbc);
			
			//files from domain
			gbc.gridx = 0;
			gbc.gridy = 2;
			gbc.weightx = 1.0;
			fileSelectionPanel.add(filesFromDomainRB, gbc);
			gbc.gridx = 1;
			gbc.weightx = 0.0;
			fileSelectionPanel.add(selectDomainBtn, gbc);
		}
		
		/**
		 * Intializes tier table pane
		 */
		private void initTierSelectionPanel(){
			//panel
			buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			tierSelectionPanel = new JPanel(new GridBagLayout());
			tierSelectionPanel.setBorder(new TitledBorder(ElanLocale.getString("OverlapsDialog.Panel.Title.TierSelection")));
			
			//buttons
			selectAllButton = new JButton(ElanLocale.getString("Button.SelectAll"));
			selectNoneButton = new JButton(ElanLocale.getString("Button.SelectNone"));
			
			ButtonHandler buttonHandler = new ButtonHandler();
			selectAllButton.addActionListener(buttonHandler);
			selectNoneButton.addActionListener(buttonHandler);
			
			//create table to show tiers in
			step1TierTable = new JTable();
			step1TierTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			step1TierTable.setShowVerticalLines(true);
			
			//first menu depends on multiple files or single files
			if( transcription != null ){
				Object[] tierArray = tierSet.toArray();
				String[] stereotypeArray = new String[tierArray.length];
				String[] linguisticArray = new String[tierArray.length];
				
				for( int i=0; i<tierArray.length; i++ ){
					TierImpl tier = (TierImpl)transcription.getTierWithId( (String)tierArray[i] );
					linguisticArray[i] = tier.getLinguisticType().getLinguisticTypeName();
					stereotypeArray[i] = "";
					
					if( tier.getLinguisticType().hasConstraints() )
						stereotypeArray[i] = Constraint.stereoTypes[tier.getLinguisticType().getConstraints().getStereoType()];
				}
				
				SelectableContentTableModel model = new SelectableContentTableModel(step1TierTable, tierArray, linguisticArray, stereotypeArray );
				model.addActionListener(new ModelClickedHandler());
				
				step1TierTable.setModel(model);
				step1TierTable.getColumnModel().getColumn(0).setHeaderValue(null);
				step1TierTable.getColumnModel().getColumn(1).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.TierName") );
				step1TierTable.getColumnModel().getColumn(2).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.LinguisticType") );
				step1TierTable.getColumnModel().getColumn(3).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.Stereotype") );
				step1TierTable.getColumnModel().getColumn(0).setMaxWidth(30);
			}else{
				DisplayableContentTableModel model = new DisplayableContentTableModel(ElanLocale.getString("OverlapsDialog.Message3"));
				step1TierTable.setModel(model);
				step1TierTable.getColumnModel().getColumn(0).setHeaderValue(ElanLocale.getString("OverlapsDialog.Column.Header.Message"));
				
				selectFilesBtn.setEnabled(true);
				selectDomainBtn.setEnabled(false);
			}
			
			//--- common layout code ----
			//add table to scroll pane
			tierTableScrollPane = new JScrollPane(step1TierTable);
			tierTableScrollPane.setColumnHeaderView(null);
			
			//add buttons
			buttonPanel.add(selectAllButton);
			buttonPanel.add(selectNoneButton);
			
			//add table
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.BOTH;
			gbc.anchor = GridBagConstraints.CENTER;
			gbc.weightx = 1.0;
			gbc.weighty = 1.0;
			gbc.insets = globalInset;
			tierSelectionPanel.add(tierTableScrollPane, gbc);
			
			//add button panel
			gbc.gridy = 1;
			gbc.weightx = 0.0;
			gbc.weighty = 0.0;
			gbc.anchor = GridBagConstraints.WEST;
			Insets insets = new Insets(0, globalInset.left, 0, globalInset.right);
			gbc.insets = insets;
			tierSelectionPanel.add(buttonPanel, gbc);
		}

		/**
		 * Updates the button states according to some constraints (like everything has to be filled in, consistently)
		 */
		public void updateButtonStates(){
			try{
				SelectableContentTableModel model = (SelectableContentTableModel) step1TierTable.getModel();
				multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, model.getSelectedValues().size() > 0);
				multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, false);
			}catch(ClassCastException e){
				//if there is no selection model, then no selection is made
				multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, false);
			}
		}
	
		private class ButtonHandler implements ActionListener{
			public void actionPerformed(ActionEvent e) {
				updateButtonStates();
				if( e != null ){
					JButton button = (JButton) e.getSource();
					
					if( button == selectFilesBtn ){
						List<String> filenames = showMultiFileChooser();
						if( filenames != null && !filenames.isEmpty() ){
							Thread t = new OpenFilesThread( filenames, step1TierTable, new ModelClickedHandler() );
							t.start();
						}
					}else if( button == selectDomainBtn ){
						//create domain dialog
						MFDomainDialog domainDialog = new MFDomainDialog(parentFrame, true);
						domainDialog.setVisible(true);
						
						//when domain is selected, get the search paths
						List<String> searchPaths = domainDialog.getSearchPaths();
						
						//check if domain contains files
						if( !searchPaths.isEmpty() ){
							//load the files in the selected domain
							Thread t = new OpenFilesThread(searchPaths, step1TierTable, new ModelClickedHandler());
							t.start();
						}
					}else if( button == selectAllButton ){
						try{
							SelectableContentTableModel model = (SelectableContentTableModel)step1TierTable.getModel();
							model.selectAll();
							updateButtonStates();
						}catch(ClassCastException exception){
							//do nothing
						}
					}else if( button == selectNoneButton ){
						try{
							SelectableContentTableModel model = (SelectableContentTableModel)step1TierTable.getModel();
							model.selectNone();
							updateButtonStates();
						}catch(ClassCastException exception){
							//do nothing
						}
					}
				}				
			}
			
		}
		
		private class ModelClickedHandler implements ActionListener{
			public void actionPerformed(ActionEvent e){
				updateButtonStates();
			}
		}
		
		private class RadioButtonHandler implements ActionListener{
			private JRadioButton previouslySelectedRadioButton;
			
			public void actionPerformed(ActionEvent e) {
				JRadioButton rb = (JRadioButton) e.getSource();
				
				if( rb == currentlyOpenedFileRB ){
					if( previouslySelectedRadioButton != rb ){
						openedFileList = null;
						previouslySelectedRadioButton = rb;
					}
					
					selectFilesBtn.setEnabled(false);
					selectDomainBtn.setEnabled(false);
					
					Object[] tierArray = tierSet.toArray();
					String[] stereotypeArray = new String[tierArray.length];
					String[] linguisticArray = new String[tierArray.length];
					
					for( int i=0; i<tierArray.length; i++ ){
						TierImpl tier = (TierImpl)transcription.getTierWithId( (String)tierArray[i] );
						linguisticArray[i] = tier.getLinguisticType().getLinguisticTypeName();
						stereotypeArray[i] = "";
						
						if( tier.getLinguisticType().hasConstraints() )
							stereotypeArray[i] = Constraint.stereoTypes[tier.getLinguisticType().getConstraints().getStereoType()];
					}
					
					SelectableContentTableModel model = new SelectableContentTableModel(step1TierTable, tierArray, linguisticArray, stereotypeArray );;
					model.addActionListener(new ModelClickedHandler());
					step1TierTable.setModel(model);
					step1TierTable.getColumnModel().getColumn(0).setHeaderValue(null);
					step1TierTable.getColumnModel().getColumn(1).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.TierName") );
					step1TierTable.getColumnModel().getColumn(2).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.LinguisticType") );
					step1TierTable.getColumnModel().getColumn(3).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.Stereotype") );
					step1TierTable.getColumnModel().getColumn(0).setMaxWidth(30);
					
				}else if( rb == selectedFilesFromDiskRB ){
					if( previouslySelectedRadioButton != rb ){
						openedFileList = null;
						previouslySelectedRadioButton = rb;
					}
					
					DisplayableContentTableModel model = new DisplayableContentTableModel(ElanLocale.getString("OverlapsDialog.Message3"));
					step1TierTable.setModel(model);
					step1TierTable.getColumnModel().getColumn(0).setHeaderValue(ElanLocale.getString("OverlapsDialog.Column.Header.Message"));
					
					selectFilesBtn.setEnabled(true);
					selectDomainBtn.setEnabled(false);
				}else if( rb == filesFromDomainRB ){
					if( previouslySelectedRadioButton != rb ){
						openedFileList = null;
						previouslySelectedRadioButton = rb;
					}
					
					DisplayableContentTableModel model = new DisplayableContentTableModel(ElanLocale.getString("OverlapsDialog.Message3"));
					step1TierTable.setModel(model);
					step1TierTable.getColumnModel().getColumn(0).setHeaderValue(ElanLocale.getString("OverlapsDialog.Column.Header.Message"));
					
					selectFilesBtn.setEnabled(false);
					selectDomainBtn.setEnabled(true);
				}
				
				updateButtonStates();
			}
		}
	}
	
	/**
	 * Panel for step 2: Overlaps Computation Panel for specifying criteria
	 * @author Jeffrey Lemein
	 *
	 */
	private class OverlapsComputationCriteriaPanel extends StepPane{
		ButtonGroup criteriaButtonGroup;
		JLabel createAnnotationLabel;
		JButton valuesMatchConstraintsBtn;
		JPanel criteriaPanel;
		
		JComboBox tierNameCB;
		JTextField annotationValueTF;
		JScrollPane scrollPane;
		DisplayableContentTableModel tableModel;
		
		public OverlapsComputationCriteriaPanel(MultiStepPane mp){
			super(mp);
			initComponents();
		}
		
		protected void initComponents(){			
			//create label
			createAnnotationLabel = new JLabel(ElanLocale.getString("OverlapsDialog.Label.CreateAnnotation"));
			
			//create elements for constraint panel
			tierNameCB = new JComboBox();
			tableModel = new DisplayableContentTableModel(4);
			tierValueTable = new JTable(tableModel);
			tierValueTable.getColumnModel().getColumn(0).setMaxWidth(35);
			tierValueTable.getColumnModel().getColumn(2).setMaxWidth(60);
			
			//create radio buttons
			RadioButtonListener radioButtonListener = new RadioButtonListener();
			overlapRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.Overlap"), true);
			overlapRB.addActionListener(radioButtonListener);
			valuesTheSameRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.ValuesTheSame"));
			valuesTheSameRB.addActionListener(radioButtonListener);
			valuesDifferentRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.ValuesDifferent"));
			valuesDifferentRB.addActionListener(radioButtonListener);
			valuesMatchConstraintsRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.ValuesMatchConstraints"));
			valuesMatchConstraintsRB.addActionListener(radioButtonListener);
			
			//create button for annotations based on different values for different tiers
			valuesMatchConstraintsBtn = new JButton(ElanLocale.getString("OverlapsDialog.Button.Constraints"));
			valuesMatchConstraintsBtn.addActionListener(new ButtonListener());
			valuesMatchConstraintsBtn.setEnabled(false);
			
			//add buttons to button group
			criteriaButtonGroup = new ButtonGroup();
			criteriaButtonGroup.add(overlapRB);
			criteriaButtonGroup.add(valuesTheSameRB);
			criteriaButtonGroup.add(valuesDifferentRB);
			criteriaButtonGroup.add(valuesMatchConstraintsRB);
			
			//create panel
			criteriaPanel = new JPanel(new GridBagLayout());
			criteriaPanel.setBorder(new TitledBorder(ElanLocale.getString("OverlapsDialog.Panel.Title.Criteria")));
			
			//add radio buttons to criteria panel
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 2;
			gbc.gridheight = 1;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.anchor = GridBagConstraints.NORTHWEST;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.insets = globalInset;
			criteriaPanel.add(createAnnotationLabel, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 1;
			gbc.insets = singleTabInset;
			criteriaPanel.add(overlapRB, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 2;
			gbc.insets = singleTabInset;
			criteriaPanel.add(valuesTheSameRB, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 3;
			criteriaPanel.add(valuesDifferentRB, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 4;
			gbc.gridwidth = 1;
			gbc.weightx = 1.0;
			criteriaPanel.add(valuesMatchConstraintsRB, gbc);
			
			gbc.gridx = 1;
			gbc.gridy = 4;
			gbc.gridwidth = 1;
			gbc.weightx = 0.0;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.fill = GridBagConstraints.NONE;
			criteriaPanel.add(valuesMatchConstraintsBtn, gbc);
			
			//add components to panel
			setLayout(new GridBagLayout());
			gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.weighty = 1.0;
			gbc.weightx = 1.0;
			gbc.anchor = GridBagConstraints.NORTHWEST;
			gbc.insets = globalInset;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			add(criteriaPanel, gbc);
		}
		
		public String getStepTitle(){
			return ElanLocale.getString("OverlapsDialog.Title.Step2Title");
		}
		
		@Override
		public void enterStepForward(){
			updateButtonStates();
		}
		
		@Override
		public void enterStepBackward(){
			updateButtonStates();
		}
		
		public void updateTierValues(){
			//get selected values in tier table from step 1
			SelectableContentTableModel model = (SelectableContentTableModel) step1TierTable.getModel();
			Vector tierList = model.getSelectedValues();
			
			tierNameCB.removeAllItems();
			
			//add tier names that are selected in tier table in step 1
			for( int i=0; i<tierList.size(); i++ )
				tierNameCB.addItem( tierList.get(i).toString() );
			
			//remove all rows in tableModel that contain elements referring to nonselected tiers
			for( int r=tableModel.getRowCount()-1; r>=0; r-- )
				if( !tierList.contains(tableModel.getValueAt(r, 1)) )
					tableModel.removeRow(r);
		}
		
		public void updateButtonStates(){
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, true);
			multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, true);
		}
	
		private class RadioButtonListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				boolean b = valuesMatchConstraintsRB.isSelected();
				
				valuesMatchConstraintsBtn.setEnabled(b);
			}
		}
		
		private class ButtonListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				JButton btn = (JButton)e.getSource();
				
				if( btn == valuesMatchConstraintsBtn ){
					ClosableDialog dialog = new ConstraintDialog();
					dialog.setSize(400, 500);
					dialog.setVisible(true);
				}
			}
		}
		
		private class ConstraintDialog extends ClosableDialog implements ActionListener, ItemListener{
			private JPanel constraintPanel, buttonPanel;
			private JButton removeButton, removeAllButton, addButton, okButton;
			private JLabel containsLabel;
			private JPanel specifyConstraintPanel, constraintOverviewPanel;
			
			public ConstraintDialog(){
				super(parentFrame, true);
				
				setTitle(ElanLocale.getString("OverlapsDialog.Dialog.ConstraintDialog.Title"));
				setLayout(new GridBagLayout());
				GridBagConstraints gbc = new GridBagConstraints();
				
				//initialize constraint panel
				initConstraintPanel();
				
				gbc.gridx = 0;
				gbc.gridy = 0;
				gbc.gridwidth = 1;
				gbc.gridheight = 1;
				gbc.weightx = 1.0;
				gbc.weighty = 1.0;
				gbc.fill = GridBagConstraints.BOTH;
				gbc.anchor = GridBagConstraints.NORTHWEST;
				gbc.insets = new Insets(4, 5, 4, 4);
				
				add(constraintPanel, gbc);
			}
			
			private void initConstraintPanel(){
				tierNameCB.addItemListener(this);
				
				//create panel
				constraintPanel = new JPanel(new GridBagLayout());
				
				specifyConstraintPanel = new JPanel(new GridBagLayout());
				specifyConstraintPanel.setBorder(new TitledBorder(ElanLocale.getString("OverlapsDialog.Panel.Title.SpecifyConstraint")));
				
				constraintOverviewPanel = new JPanel(new GridBagLayout());
				constraintOverviewPanel.setBorder(new TitledBorder(ElanLocale.getString("OverlapsDialog.Panel.Title.ConstraintsOverview")));
				
				//create components
				annotationValueTF = new JTextField();
				
				containsLabel = new JLabel(ElanLocale.getString("OverlapsDialog.Combo.Contains"));
				
				addButton = new JButton(ElanLocale.getString("Button.Add"));
				addButton.addActionListener(this);
				
				tierValueTable.getColumnModel().getColumn(0).setHeaderValue("");
				tierValueTable.getColumnModel().getColumn(0).setWidth(10);
				tierValueTable.getColumnModel().getColumn(1).setHeaderValue(ElanLocale.getString("OverlapsDialog.Column.Header.Tier"));
				tierValueTable.getColumnModel().getColumn(2).setHeaderValue("");
				tierValueTable.getColumnModel().getColumn(2).setWidth(30);
				tierValueTable.getColumnModel().getColumn(3).setHeaderValue(ElanLocale.getString("OverlapsDialog.Column.Header.Value"));
				tableModel.connectTable(tierValueTable);
				scrollPane = new JScrollPane(tierValueTable);
				
				removeButton = new JButton(ElanLocale.getString("OverlapsDialog.Button.Remove"));
				removeButton.addActionListener(this);
				removeAllButton = new JButton(ElanLocale.getString("OverlapsDialog.Button.RemoveAll"));
				removeAllButton.addActionListener(this);
				okButton = new JButton(ElanLocale.getString("Button.OK"));
				okButton.addActionListener(this);
				
				//layout components
				GridBagConstraints gbc = new GridBagConstraints();
				
				//== specify constraint panel ====
				gbc.gridx = 0;
				gbc.gridy = 0;
				gbc.gridwidth = 1;
				gbc.weightx = 0.5;
				gbc.insets = new Insets(5, 10, 5, 0);
				gbc.anchor = GridBagConstraints.CENTER;
				gbc.fill = GridBagConstraints.HORIZONTAL;
				specifyConstraintPanel.add(tierNameCB, gbc);
				
				gbc.gridy = 0;
				gbc.gridx = 1;
				gbc.fill = GridBagConstraints.BOTH;
				gbc.weightx = 0.0;
				specifyConstraintPanel.add(containsLabel, gbc);
				
				gbc.gridx = 2;
				gbc.weightx = 0.5;
				gbc.fill = GridBagConstraints.BOTH;
				specifyConstraintPanel.add(annotationValueTF, gbc);
				
				gbc.gridx = 3;
				gbc.weightx = 0.0;
				gbc.insets = globalInset;
				gbc.anchor = GridBagConstraints.WEST;
				gbc.fill = GridBagConstraints.NONE;
				specifyConstraintPanel.add(addButton, gbc);				
				
				//== Constraint overview panel ========
				gbc.gridx = 0;
				gbc.gridy = 0;
				gbc.gridwidth = 1;
				gbc.fill = GridBagConstraints.BOTH;
				gbc.anchor = GridBagConstraints.CENTER;
				gbc.weightx = 1.0;
				gbc.weighty = 1.0;
				gbc.insets = globalInset;
				constraintOverviewPanel.add(scrollPane, gbc);
				
				buttonPanel = new JPanel(new FlowLayout());
				buttonPanel.add(removeButton);
				buttonPanel.add(removeAllButton);
				gbc.gridx = 0;
				gbc.gridy = 1;
				gbc.weightx = 0.0;
				gbc.weighty = 0.0;
				gbc.gridwidth = 1;
				Insets insets = new Insets(0, globalInset.left, globalInset.bottom, globalInset.right);
				gbc.insets = insets;
				gbc.anchor = GridBagConstraints.WEST;
				gbc.fill = GridBagConstraints.NONE;
				constraintOverviewPanel.add(buttonPanel, gbc);
				
				//== Add panels to view ====
				gbc.gridx = 0;
				gbc.gridy = 0;
				gbc.gridwidth = 1;
				gbc.anchor = GridBagConstraints.CENTER;
				gbc.fill = GridBagConstraints.HORIZONTAL;
				gbc.weightx = 1.0;
				gbc.weighty = 0.0;
				gbc.insets = new Insets(0, 0, 0, 0);
				constraintPanel.add(specifyConstraintPanel, gbc);
				
				gbc.gridx = 0;
				gbc.gridy = 1;
				gbc.fill = GridBagConstraints.BOTH;
				gbc.weighty = 1.0;
				constraintPanel.add(constraintOverviewPanel, gbc);
				
				gbc.gridx = 0;
				gbc.gridy = 2;
				gbc.weightx = 0.0;
				gbc.weighty = 0.0;
				gbc.anchor = GridBagConstraints.NORTHWEST;
				gbc.fill = GridBagConstraints.NONE;
				gbc.insets = globalInset;
				constraintPanel.add(okButton, gbc);
			}

			public void actionPerformed(ActionEvent e) {
				JButton btn = (JButton) e.getSource();
				
				if( btn == addButton ){
					addButton.setEnabled(false);
					int selectedIndex = tierNameCB.getSelectedIndex();
					
					//add tier in constraint panel
					String[] value = new String[]{"", tierNameCB.getItemAt(selectedIndex).toString(), ElanLocale.getString("OverlapsDialog.Combo.Contains"), annotationValueTF.getText()};
					int rowIndex = findIndexToAdd(value[1], value[3]);
					
					//don't add duplicate tier constraints
					if( tableModel.contains(value[1], 1))
						return;
					
					//check if value in front of the to be added value corresponds to the same tier
					if( rowIndex > 0 ){
						value[0] = ElanLocale.getString("OverlapsDialog.Message.And");
						tableModel.addRowAt(value, rowIndex);
					}else{
						tableModel.addRowAt(value, rowIndex);
						if( tableModel.getRowCount() >= 2 )
							tableModel.setValueAt(ElanLocale.getString("OverlapsDialog.Message.And"), 1, 0);
					}
					
					tableModel.updateTables();
				}else if( btn == removeButton ){
					int[] selectedRows = tierValueTable.getSelectedRows();
					for( int i=selectedRows.length-1; i>=0; i-- )
						tableModel.removeRow(selectedRows[i]);
					
					//update first column values
					for( int i=0; i<tableModel.getRowCount(); i++ ){
						if( i == 0 )
							tableModel.setValueAt("", 0, 0);
						else
							tableModel.setValueAt(ElanLocale.getString("OverlapsDialog.Message.And"), i, 0);
					}
					tierValueTable.getSelectionModel().clearSelection();
					
					String tierName = (String) tierNameCB.getSelectedItem();
					addButton.setEnabled(!tableModel.contains(tierName, 1));
					updateAddButtonState();
				}else if(btn == removeAllButton ){
					for( int i=tableModel.getRowCount()-1; i>=0; i-- )
						tableModel.removeRow(i);
					
					tierValueTable.getSelectionModel().clearSelection();
					updateAddButtonState();
				}else if( btn == okButton ){
					setVisible(false);
					dispose();
				}
			}
			
			private int findIndexToAdd(String tiername, String value){
				Collator collator = Collator.getInstance();
				
				for( int r=0; r<tableModel.getRowCount(); r++ ){
					int res = collator.compare(tiername, tableModel.getValueAt(r,1));
					if( res == -1 ){
						return r;
					}else if( res == 0 ){
						if( collator.compare(value, tableModel.getValueAt(r,3)) < 0 )
							return r;
					}					
				}
				
				return tableModel.getRowCount();
			}

			public void itemStateChanged(ItemEvent e) {
				updateAddButtonState();			
			}
			
			private void updateAddButtonState(){
				String tierName = (String) tierNameCB.getSelectedItem();
				addButton.setEnabled(!tableModel.contains(tierName, 1));	
			}
		}
	}
	
	/**
	 * Panel for step 3
	 * @author Jeffrey Lemein
	 * @version July, 2010
	 */
	private class DestinationTierPanel extends StepPane implements ListSelectionListener, ActionListener, KeyListener{
		private JLabel newTierNameLabel, decideRootOrChildLabel, lingTypeLabel;
		private JScrollPane lingTypeTableScrollPane;
		private JPanel destinationTierConfigurationPanel;
		private ButtonGroup rootChildButtonGroup;
		
		/**
		 * Constructor
		 */
		public DestinationTierPanel(MultiStepPane mp){
			super(mp);
			initComponents();
		}
		
		@Override
		protected void initComponents(){			
			newTierNameLabel = new JLabel( ElanLocale.getString("OverlapsDialog.Label.newTierNameLabel") );
			lingTypeLabel = new JLabel( ElanLocale.getString("OverlapsDialog.Label.lingTypeLabel") );
			
			//create text field
			newTierNameField = new JTextField();
			newTierNameField.addKeyListener(this);
			
			//create parent tier combobox + checkbox
			decideRootOrChildLabel = new JLabel(ElanLocale.getString("OverlapsDialog.Label.RootChildLabel"));
			rootTierRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.RootTier"), true);
			rootTierRB.addActionListener(this);
			childTierRB = new JRadioButton(ElanLocale.getString("OverlapsDialog.Radio.ChildTier"));
			childTierRB.addActionListener(this);
			
			rootChildButtonGroup = new ButtonGroup();
			rootChildButtonGroup.add(rootTierRB);
			rootChildButtonGroup.add(childTierRB);
			
			parentTierCB = new JComboBox();
			parentTierCB.addActionListener(this);
			parentTierCB.setEnabled(false);
			
			//create table
			linguisticTypeTable = new JTable();
			linguisticTypeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			linguisticTypeTable.setModel(new LinguisticTypeTableModel());
			linguisticTypeTable.getColumnModel().getColumn(0).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.LinguisticType") );
			linguisticTypeTable.getColumnModel().getColumn(1).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.Stereotype") );
			linguisticTypeTable.getSelectionModel().addListSelectionListener(this);
			lingTypeTableScrollPane = new JScrollPane(linguisticTypeTable);
			
			//layout components
			destinationTierConfigurationPanel = new JPanel(new GridBagLayout());
			destinationTierConfigurationPanel.setBorder(new TitledBorder(ElanLocale.getString("OverlapsDialog.Panel.Title.DestTierConfiguration")));
			
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 2;
			gbc.gridheight = 1;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.insets = globalInset;
			destinationTierConfigurationPanel.add(newTierNameLabel, gbc);
			
			gbc.gridy = 1;
			destinationTierConfigurationPanel.add(newTierNameField, gbc);
			
			gbc.gridy = 2;
			destinationTierConfigurationPanel.add(decideRootOrChildLabel, gbc);
			
			gbc.gridy = 3;
			gbc.insets = singleTabInset;
			destinationTierConfigurationPanel.add(rootTierRB, gbc);
			
			gbc.gridy = 4;
			gbc.gridwidth = 1;
			gbc.weightx = 0.0;
			destinationTierConfigurationPanel.add(childTierRB, gbc);
			gbc.gridx = 1;
			gbc.weightx = 1.0;
			gbc.insets = globalInset;
			destinationTierConfigurationPanel.add(parentTierCB, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 5;
			gbc.gridwidth = 2;
			destinationTierConfigurationPanel.add(lingTypeLabel, gbc);
			
			gbc.gridy = 6;
			gbc.weighty = 1.0;
			gbc.fill = GridBagConstraints.BOTH;
			destinationTierConfigurationPanel.add(lingTypeTableScrollPane, gbc);
			
			setLayout(new GridBagLayout());
			gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.weighty = 1.0;
			gbc.weightx = 1.0;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.BOTH;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.insets = globalInset;			
			add(destinationTierConfigurationPanel, gbc);
		}
		
		public void updateTierValues(){
			//get selected values in tier table from step 1
			SelectableContentTableModel model = (SelectableContentTableModel) step1TierTable.getModel();
			Vector tierList = model.getSelectedValues();
			
			parentTierCB.removeAllItems();
			
			//add tier names that are selected in tier table in step 1
			for( int i=0; i<tierList.size(); i++ ){
				//TierImpl tier = (TierImpl) transcription.getTierWithId( (String)tierList.get(i) );
				parentTierCB.addItem((String)tierList.get(i));
				//parentTierCB.addItem( tierList.get(i).toString() );
			}
		}
		
		private boolean isValidParentTier(List<TranscriptionImpl> transImpls, String tierName){			
			for( TranscriptionImpl t : transImpls ){
				TierImpl tier = (TierImpl) t.getTierWithId(tierName);
				if( tier == null )
					return false;
			}
			
			return true;
		}
		
		@Override
		public String getStepTitle(){
			return ElanLocale.getString("OverlapsDialog.Title.Step3Title");
		}
		
		@Override
		public void enterStepForward(){
			updateButtonStates();
		}
		
		@Override
		public void enterStepBackward(){
			updateButtonStates();
		}
		
		@Override
		public boolean leaveStepForward(){
			String newTierName = newTierNameField.getText().trim();
			
			if( tierSet.contains(newTierName) )
				JOptionPane.showMessageDialog(this, ElanLocale.getString("OverlapsDialog.Message1.Part1") + newTierName + ElanLocale.getString("OverlapsDialog.Message1.Part2"), ElanLocale.getString("OverlapsDialog.Message.Title"), JOptionPane.ERROR_MESSAGE);
			else if( newTierName.length() <= 0 )
				JOptionPane.showMessageDialog(this, ElanLocale.getString("OverlapsDialog.Message2"), ElanLocale.getString("OverlapsDialog.Message.Title"), JOptionPane.ERROR_MESSAGE);
			
			return true;
		}
		
		/**
		 * Table Model for  linguistic types
		 * @author Jeffrey Lemein
		 * @version July, 2010
		 */
		private class LinguisticTypeTableModel extends AbstractTableModel{
			private String[] columnNames;
			private String[][] rowData;
			private boolean multipleFileMode;
			
			/**
			 * Constructor
			 */
			public LinguisticTypeTableModel(){
				columnNames = new String[2];
				columnNames[0] = new String( ElanLocale.getString("OverlapsDialog.Column.LinguisticType") );
				columnNames[1] = new String(ElanLocale.getString("OverlapsDialog.Column.Stereotype") );
				
				multipleFileMode = transcription == null;
				updateLinguisticTypes(null);
			}
			
			/**
			 * Retrieves the list of possible linguistic types that a new destination tier can have.
			 * This list is dependent on the number of files.
			 * @return
			 */
			private Vector getLinguisticTypeList(){
				Vector<LinguisticType> lingTypes = new Vector<LinguisticType>();
				
				if( !multipleFileMode ){
					lingTypes.addAll(transcription.getLinguisticTypes());
				}else{
					//multiple file mode
					//in case no files have been opened yet
					if( openedFileList == null )
						return lingTypes;
					
					//else...
					Vector<LinguisticType> removeList = new Vector<LinguisticType>();
					
					for( int i=0; i<openedFileList.size(); i++ ){
						TranscriptionImpl t = openedFileList.get(i);
						
						if( i <= 0 )
							lingTypes.addAll(t.getLinguisticTypes());
						else{
							Vector v = openedFileList.get(i).getLinguisticTypes();
							
							for( int j=0; j<v.size(); j++ ){
								LinguisticType newLt = (LinguisticType) v.get(j);
								
								if( !lingTypes.contains(newLt) && !removeList.contains(newLt) ){
									lingTypes.add(newLt);
								}else{
									LinguisticType oldLt = lingTypes.get(lingTypes.indexOf(newLt));
									
									//check if these two linguistic types have the same constraints.
									//if not, then the linguistic type should be removed from the list
									if( oldLt.isUsingControlledVocabulary() != newLt.isUsingControlledVocabulary() 
										|| oldLt.hasConstraints() != newLt.hasConstraints()
										|| (oldLt.hasConstraints() && newLt.hasConstraints() && oldLt.getConstraints().getStereoType() != newLt.getConstraints().getStereoType()) ){
										
										//tegenstrijdig
										lingTypes.remove(oldLt);
										removeList.add(oldLt);
									}
								}
							}
						}
					}
				}
				
				return lingTypes;
			}
			
			/**
			 * Adds the linguistic types to the linguistic type table (by looking at the parent tier name and ...)
			 * @param parentTierName
			 */
			public void updateLinguisticTypes(String parentTierName){
				List<String> linguisticTypeNames = new ArrayList<String>();
				List<String> stereotypeNames = new ArrayList<String>();
				List<LinguisticType> linguisticTypeList = getLinguisticTypeList();
				
				//SINGLE FILE MODE
				if( !multipleFileMode ){
					//only one file is used
					if( parentTierName == null ){
						//to be created tier will be root tier (so only add top-level linguistic types)
						for( LinguisticType lt : linguisticTypeList )
							if( !lt.hasConstraints() && !lt.isUsingControlledVocabulary()  ){
								linguisticTypeNames.add(lt.getLinguisticTypeName());
								stereotypeNames.add("");
							}
					}else{
						//to be created tier has a parent tier (so included_in, top-level linguistic type (if parent has same linguistic type))
						TierImpl parentTier = (TierImpl) transcription.getTierWithId(parentTierName);
						
						for( LinguisticType lt : linguisticTypeList ){
							//don't add linguistic types that are using controlled vocabulary
							if( lt.isUsingControlledVocabulary() )
								continue;
							
							//add linguistic type if it equals included_in
							if( lt.hasConstraints() && lt.getConstraints().getStereoType() == Constraint.INCLUDED_IN ){
								linguisticTypeNames.add(lt.getLinguisticTypeName());
								stereotypeNames.add( Constraint.stereoTypes[ lt.getConstraints().getStereoType() ] );
								//JOptionPane.showMessageDialog(null, lt.getLinguisticTypeName() + " added");
								continue;
							}
							
							//and if parent tier has top level linguistic type, add this type too
							/**if( !lt.hasConstraints() ){
								linguisticTypeNames.add(lt.getLinguisticTypeName());
								stereotypeNames.add("");
								//JOptionPane.showMessageDialog(null, lt.getLinguisticTypeName() + " added");
								continue;
							}	**/						
						}
					}
				//MULTIPLE FILE MODE
				}else{
					//we work with multiple transcription/files
					//so construct linguisticTypeList with linguistic types that are valid
					
					
					//NEW ROOT TIER
					if( parentTierName == null ){
						//no parent tier selected, so add top-level linguistic types
						for( LinguisticType lt : linguisticTypeList )
							if( !lt.hasConstraints() && !lt.isUsingControlledVocabulary() ){
								linguisticTypeNames.add(lt.getLinguisticTypeName());
								stereotypeNames.add("");
							}
					//NEW CHILD TIER
					}else{
						boolean allTiersAreRoot = true;
						
						for( TranscriptionImpl t : openedFileList ){
							TierImpl parentTier = (TierImpl) t.getTierWithId(parentTierName);
							if( parentTier != null && parentTier.getLinguisticType().hasConstraints() ){
								allTiersAreRoot = false;
								break;
							}
						}
						
						//a parent tier is selected, so add included_in
						for( LinguisticType lt : linguisticTypeList ){
							if( lt.isUsingControlledVocabulary() )
								continue;
							
							//included_in can always be added
							if( lt.hasConstraints() && lt.getConstraints().getStereoType() == Constraint.INCLUDED_IN ){
								linguisticTypeNames.add(lt.getLinguisticTypeName());
								stereotypeNames.add( Constraint.stereoTypes[ lt.getConstraints().getStereoType() ] );
								//JOptionPane.showMessageDialog(null, lt.getLinguisticTypeName() + " added (it should only be included_in\nValue of stereotype " + lt.getConstraints().getStereoType() + " == " + Constraint.INCLUDED_IN);
								continue;
							}
							
							//if( allTiersAreRoot && !lt.hasConstraints() ){
							//	linguisticTypeNames.add(lt.getLinguisticTypeName());
							//	stereotypeNames.add("");
								//JOptionPane.showMessageDialog(null, lt.getLinguisticTypeName() + " added. hasConstraints = " + lt.hasConstraints());
							//}
						}
					}
				}
				
				//add the linguistic type with its stereotype
				final int SIZE = linguisticTypeNames.size();
				rowData = new String[SIZE][];
				for( int i=0; i<SIZE; i++ )
					rowData[i] = new String[]{ linguisticTypeNames.get(i), stereotypeNames.get(i) };
				
				fireTableDataChanged();
			}
			
			public int getColumnCount() {
				return columnNames.length;
			}

			public int getRowCount() {
				if( rowData != null )
					return rowData.length;
				else
					return 0;
			}

			public Object getValueAt(int row, int col) {
				return rowData[row][col];
			}
			
		}
		
		/**
		 * Updates the button states according to some constraints (like everything has to be filled in, consistently)
		 */
		public void updateButtonStates(){
			multiPane.setButtonText(MultiStepPane.NEXT_BUTTON, ElanLocale.getString("Button.Next"));
			
			String newTierName = newTierNameField.getText().trim();
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, newTierName.length() > 0 && !tierSet.contains(newTierName) && linguisticTypeTable.getSelectedRowCount() > 0);
			multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, true);
		}

		/**
		 * Value changed event handler that updates button states when new selection is made
		 */
		public void valueChanged(ListSelectionEvent arg0) {
			updateButtonStates();	
		}

		/**
		 * Not used
		 */
		public void keyPressed(KeyEvent e) {}

		/**
		 * Key release handler that updates the button states
		 */
		public void keyReleased(KeyEvent e) {
			updateButtonStates();
		}

		/**
		 * Key typed handler that updates the button states
		 */
		public void keyTyped(KeyEvent e) {
			updateButtonStates();
		}

		//action listener for parent tier checkbox and parent tier combobox
		public void actionPerformed(ActionEvent e) {
			parentTierCB.setEnabled(childTierRB.isSelected());
			
			//because a checkbox is selected, update the possible linguistic types
			String parentTierName = null;
			LinguisticTypeTableModel model = (LinguisticTypeTableModel) linguisticTypeTable.getModel();
			
			if( childTierRB.isSelected() )
				parentTierName = (String)parentTierCB.getSelectedItem();
			
			model.updateLinguisticTypes(parentTierName);
			
			updateButtonStates();
		}
	}
	
	/**
	 * Panel for step 4
	 * @author Jeffrey Lemein
	 * @version July, 2010
	 */
	private class DestinationValuePanel extends StepPane implements ActionListener, KeyListener{
		private JLabel newValueLabel;
		private ButtonGroup valueKindRadioGroup, timeRadioGroup, smpteRadioGroup;
		private JPanel tierValuePanel;
		
		/**
		 * Constructor
		 */
		public DestinationValuePanel(MultiStepPane mp){
			super(mp);
			initComponents();
		}
		
		@Override
		protected void initComponents(){			
			//create radio buttons for annotation value
			durationRadioButton = new JRadioButton( ElanLocale.getString("OverlapsDialog.Radio.DurationValue"), true);
			specificValueRB = new JRadioButton( ElanLocale.getString("OverlapsDialog.Radio.SpecificValue") );
			tierValueRadioButton = new JRadioButton( ElanLocale.getString("OverlapsDialog.Radio.SpecificTier") );
			concatValuesRB = new JRadioButton( ElanLocale.getString("OverlapsDialog.Radio.ConcatValues") );
			
			valueKindRadioGroup = new ButtonGroup();
			valueKindRadioGroup.add(durationRadioButton);
			valueKindRadioGroup.add(tierValueRadioButton);
			valueKindRadioGroup.add(specificValueRB);
			valueKindRadioGroup.add(concatValuesRB);
			
			durationRadioButton.addActionListener(this);
			specificValueRB.addActionListener(this);
			tierValueRadioButton.addActionListener(this);
			concatValuesRB.addActionListener(this);
			
			//radio buttons for the time formats
			msecRB = new JRadioButton(ElanLocale.getString("TimeCodeFormat.MilliSec"), true);
			secRB = new JRadioButton(ElanLocale.getString("TimeCodeFormat.Seconds"));
			hrRB = new JRadioButton( ElanLocale.getString("TimeCodeFormat.TimeCode") );
			smpteRB = new JRadioButton( ElanLocale.getString("TimeCodeFormat.TimeCode.SMPTE") );
			palRB = new JRadioButton( ElanLocale.getString("TimeCodeFormat.TimeCode.SMPTE.PAL"), true);
			ntscRB = new JRadioButton( ElanLocale.getString("TimeCodeFormat.TimeCode.SMPTE.NTSC"));
			
			msecRB.addActionListener(this);
			secRB.addActionListener(this);
			hrRB.addActionListener(this);
			smpteRB.addActionListener(this);
			
			palRB.setEnabled(false);
			ntscRB.setEnabled(false);
			
			timeRadioGroup = new ButtonGroup();
			timeRadioGroup.add(msecRB);
			timeRadioGroup.add(secRB);
			timeRadioGroup.add(hrRB);
			timeRadioGroup.add(smpteRB);
			
			smpteRadioGroup = new ButtonGroup();
			smpteRadioGroup.add(palRB);
			smpteRadioGroup.add(ntscRB);

			//create text field
			specificValueTF = new JTextField();
			specificValueTF.setEnabled(false);
			specificValueTF.addKeyListener(this);
			
			//create combobox
			tierSelectBox = new JComboBox();
			tierSelectBox.setEnabled(false);
			
			
			//layout components
			tierValuePanel = new JPanel(new GridBagLayout());
			tierValuePanel.setBorder(new TitledBorder(ElanLocale.getString("OverlapsDialog.Panel.Title.TierValue")));
			
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 2;
			gbc.gridheight = 1;
			gbc.anchor = GridBagConstraints.NORTHWEST;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.insets = firstSingleTabInset;
			tierValuePanel.add(durationRadioButton, gbc);
			
			gbc.insets = singleTabInset;
			gbc.gridy = 1;
			tierValuePanel.add(msecRB, gbc);
			
			gbc.gridy = 2;
			tierValuePanel.add(secRB, gbc);
			
			gbc.gridy = 3;
			tierValuePanel.add(hrRB, gbc);
			
			gbc.gridy = 4;
			tierValuePanel.add(smpteRB, gbc);
			
			gbc.gridy = 5;
			gbc.insets = doubleTabInset;
			tierValuePanel.add(palRB, gbc);
			
			gbc.gridy = 6;
			tierValuePanel.add(ntscRB, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 7;
			gbc.insets = globalInset;
			gbc.gridwidth = 1;
			gbc.weightx = 0.0;
			tierValuePanel.add(concatValuesRB, gbc);			
			
			gbc.gridx = 0;
			gbc.gridy = 8;
			gbc.insets = globalInset;
			gbc.gridwidth = 1;
			gbc.weightx = 0.0;
			tierValuePanel.add(specificValueRB, gbc);
			
			gbc.gridx = 1;
			gbc.weightx = 1.0;
			tierValuePanel.add(specificValueTF, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 9;
			gbc.weightx = 0.0;
			tierValuePanel.add(tierValueRadioButton, gbc);
			
			gbc.gridx = 1;
			gbc.weightx = 1.0;
			tierValuePanel.add(tierSelectBox, gbc);
			
			//add panels to screen
			setLayout(new GridBagLayout());
			gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.weightx = 1.0;
			gbc.weighty = 1.0;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.NORTH;
			gbc.insets = globalInset;			
			add(tierValuePanel, gbc);
		}

		@Override
		public String getStepTitle(){
			return ElanLocale.getString("OverlapsDialog.Title.Step4Title");
		}
		
		@Override
		public void enterStepForward(){
			updateButtonStates();
			atLastStep = true;
		}
		
		@Override
		public boolean doFinish(){
			if( atLastStep )
				executeOverlapsCommand();
			
			return true;
		}
		
		public boolean leaveStepBackward(){
			atLastStep = false;
			multiPane.setButtonEnabled(MultiStepPane.FINISH_BUTTON, false);
			return true;
		}
		
		/**
		 * Updates the combobox with tier names that are selected in the table in step 1
		 */
		public void updateValues(){
			//get selected values in tier table from step 1
			SelectableContentTableModel model = (SelectableContentTableModel) step1TierTable.getModel();
			Vector tierList = model.getSelectedValues();
			
			tierSelectBox.removeAllItems();
			
			//add tier names that are selected in tier table in step 1
			for( int i=0; i<tierList.size(); i++ )
				tierSelectBox.addItem( tierList.get(i).toString() );
		}
		
		/**
		 * Set the button states appropriately, according to constraints
		 */
		public void updateButtonStates(){
			boolean b = durationRadioButton.isSelected();
			msecRB.setEnabled(b);
			secRB.setEnabled(b);
			hrRB.setEnabled(b);
			smpteRB.setEnabled(b);
			
			palRB.setEnabled(b && smpteRB.isSelected());
			ntscRB.setEnabled(b && smpteRB.isSelected());
			
			tierSelectBox.setEnabled(tierValueRadioButton.isSelected());
			
			specificValueTF.setEnabled(specificValueRB.isSelected());
			
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, false);
			
			if( specificValueRB.isSelected() )
				multiPane.setButtonEnabled(MultiStepPane.FINISH_BUTTON, specificValueTF.getText().trim().length() > 0);
			else
				multiPane.setButtonEnabled(MultiStepPane.FINISH_BUTTON, true);
			
			multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, true);
		}
		
		/**
		 * Action handler for buttons and radio buttons in this panel
		 */
		public void actionPerformed(ActionEvent e) {			
			updateButtonStates();
		}

		/**
		 * Not used
		 */
		public void keyPressed(KeyEvent e) {}

		/**
		 * Key release handler that updates the button states
		 */
		public void keyReleased(KeyEvent e) {
			updateButtonStates();
		}

		/**
		 * Key typed handler that updates the button states
		 */
		public void keyTyped(KeyEvent e) {
			updateButtonStates();
		}
	}

	private class OpenFilesThread extends Thread{
		private List<String> filenames;
		private JTable tierTable;
		private DisplayableContentTableModel model;
		private ActionListener listener;
		
		public OpenFilesThread( List<String> filenames, JTable tierTable, ActionListener listener ){
			this.filenames = filenames;
			this.tierTable = tierTable;
			this.listener = listener;
			model = new DisplayableContentTableModel( new String[]{ElanLocale.getString("OverlapsDialog.Message4.Part1"), "0 " + ElanLocale.getString("OverlapsDialog.Message4.Part2") + " " + filenames.size() + " " + ElanLocale.getString("OverlapsDialog.Message4.Part3") + " (0%)"} );
			model.connectTable(this.tierTable);
			this.tierTable.setModel(model);
		}
		
		@Override
		public void run(){
			//Set<String> uniqueTierNames = new TreeSet<String>();
			tierSet = new TreeSet<String>();
			openedFileList = new ArrayList<TranscriptionImpl>();

			multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, false);
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, false);
			
			for( int i=0; i<filenames.size(); i++ ){
				//open file and store in list with transcription implementations
				TranscriptionImpl transImpl = new TranscriptionImpl( filenames.get(i) );
				openedFileList.add(transImpl);
				
				//get tiers
				Vector<TierImpl> tiersInFile = transImpl.getTiers();
				
				//store tiers
				for( TierImpl tier : tiersInFile ){
					tierSet.add(tier.getName());
				}
				model.updateMessage(1, (i+1) + " " + ElanLocale.getString("OverlapsDialog.Message4.Part2") + " " + filenames.size() + " " + ElanLocale.getString("OverlapsDialog.Message4.Part3") + " (" + Math.round((i+1)/((float)filenames.size())*100.0f) + "%)");
			}
			
			//update table
			if( tierSet.isEmpty() ){
				//if there are no tiers to be loaded (all files do not contain tiers)
				openedFileList.clear();
				DisplayableContentTableModel model = (DisplayableContentTableModel) tierTable.getModel();
				//tierTable.setModel(dataModel)
				
				model.setValueAt( ElanLocale.getString("OverlapsDialog.Message3"), 0, 0);
			}else{
				SelectableContentTableModel model = new SelectableContentTableModel(tierSet, tierTable);
				model.addActionListener(listener);
				tierTable.setModel(model);
				tierTable.getColumnModel().getColumn(0).setHeaderValue(null);
				tierTable.getColumnModel().getColumn(1).setHeaderValue( ElanLocale.getString("OverlapsDialog.Column.TierName") );
				tierTable.getColumnModel().getColumn(0).setMaxWidth(30);
				tierTable.repaint();
			}
		}
	}
}
