package mpi.eudico.client.annotator.timeseries;

import java.awt.Component;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.recognizer.data.FileParam;
import mpi.eudico.client.annotator.timeseries.csv.CSVWriter;
import mpi.eudico.client.annotator.timeseries.xml.XMLWriter;
import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.util.FileExtension;

/**
 * A class for starting the export track process.
 * 
 * @author Han Sloetjes
 */
public class ExportTrack {

	/**
	 * Constructor.
	 */
	public ExportTrack() {
		super();
	}
	
	/**
	 * Shows a save as dialog and starts the export
	 * 
	 * @param track timeseries track
	 */
	public void exportTrack(AbstractTSTrack track) {
		exportTrack(null, track);
	}
	
	/**
	 * Shows a save as dialog and starts the export
	 * 
	 * @param track timeseries track
	 */
	public void exportTrack(Component parent, AbstractTSTrack track) {
		if (track == null) {
			return;
		}
		
		String savePath = promptForTierFile(parent, track.getSource());
		if (savePath != null) {
			File tf = new File(savePath);
			try { 
				if (tf.exists()) {
	                int answer = JOptionPane.showConfirmDialog(parent,
	                        ElanLocale.getString("Message.Overwrite"),
	                        ElanLocale.getString("SaveDialog.Message.Title"),
	                        JOptionPane.YES_NO_OPTION,
	                        JOptionPane.WARNING_MESSAGE);

	                if (answer == JOptionPane.NO_OPTION) {
	                    return;
	                }
				}
			} catch (Exception ex) {// any exception
				return;
			}
			
			String lower = savePath.toLowerCase();
			if (lower.endsWith("csv")) {
				try {
					new CSVWriter().writeTrackToCSV(tf, track);
				} catch (IOException ioe) {
					JOptionPane.showMessageDialog(parent, ElanLocale.getString(
						"Recognizer.RecognizerPanel.Warning.SaveFailed")  + ioe.getMessage(), 
						ElanLocale.getString("Message.Warning"), JOptionPane.ERROR_MESSAGE);
				}
			} else {
				try {
					new XMLWriter().writeTrackToXML(tf, track);
				} catch (IOException ioe) {
					JOptionPane.showMessageDialog(parent, ElanLocale.getString(
						"Recognizer.RecognizerPanel.Warning.SaveFailed")  + ioe.getMessage(), 
						ElanLocale.getString("Message.Warning"), JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
	
	/**
	 * Prompts the user to specify a location where to store the timeseries.
	 * 
	 * @param a parent component for the save dialog, can be null
	 * @param track the track
	 * @return the path or null if canceled
	 */
	private String promptForTierFile(Component parent, String source) {
		String prefPath = null;
		if (source != null) {
			source = source.replaceAll("\\", "/");
			int index = source.lastIndexOf('/');
			if (index > -1 ) {
				prefPath = source.substring(index);
			}
		}
		if (prefPath == null) {
			Object val = Preferences.get("LinkedFileDir", null);
			
			if (val instanceof String) {
				prefPath = (String) val;
			}
		}
		JFileChooser chooser = null;
		if (prefPath != null) {
			chooser = new JFileChooser(prefPath);
		} else {
			chooser = new JFileChooser();
		}
		FileFilter ff =  ElanFileFilter.createFileFilter(ElanFileFilter.CSV_TYPE);
		FileFilter ff2 = ElanFileFilter.createFileFilter(ElanFileFilter.XML_TYPE);
			
		chooser.addChoosableFileFilter(ff);
		chooser.addChoosableFileFilter(ff2);
		chooser.setFileFilter(ff2);
		
		if (chooser.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
			File f = chooser.getSelectedFile();
			File par = f.getParentFile();
			if (par!= null && par.isDirectory()) {
				Preferences.set("LinkedFileDir", par.getAbsolutePath(), null);
			}
			
			if (chooser.getFileFilter() == ff) {
				String lower = f.getAbsolutePath().toLowerCase();
				boolean valid = false;
				for (int i = 0; i < FileExtension.CSV_EXT.length; i++) {
					if (lower.endsWith(FileExtension.CSV_EXT[i])) {
						valid = true;
						break;
					}
				}
				if (!valid) {
					f = new File(f.getAbsolutePath() + "." + FileExtension.CSV_EXT[0]);
				}
			} else {
				if (!f.getAbsolutePath().toLowerCase().endsWith("xml")) {
					f = new File(f.getAbsolutePath() + ".xml");
				}
			}
			return f.getAbsolutePath();
		} else {
			return null;
		}		
	}

}
