package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLayoutManager;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.gui.ClosableDialog;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clomimpl.type.LinguisticType;

/**
 * Class to select the tiers for each column
 * 
 * @author Aarthy Somasundaram
 *
 */
public class SelectChildTiersDlg extends ClosableDialog implements ActionListener{
	
	/** default String */
	private static final String defaultValue = "select a tier";
	
	/** default String */
	private static final String NO_TIER = "No Tier";
	
	/** column id for the tier name column */
    private final String TIER_NAME_COLUMN = "Top Level Tiers / Reference Tiers";
    
    /** column id for the 1st column */
    private String TIER_COL1 = "Type1 : ";
    
    /** column id for the 2nd column */
    private String TIER_COL2 = "Type2 : ";
    
    /** column id for the 3rd column */
    private String TIER_COL3 = "Type3 : ";
    
    private JTable selectionTable;
	DefaultTableModel model;
    private Transcription transcription ;
   
    private String type1;
    private String type2;
    private String type3;
    
    private JButton okButton;
    private JButton cancelButton;
    
 	private HashMap<TierImpl, TierImpl> tierMapColumn1;
 	private HashMap<TierImpl, TierImpl> tierMapColumn2;
 	private HashMap<TierImpl, TierImpl> tierMapColumn3;
 	
 	private boolean valueChanged = false;
 	
 	/**
 	 * Creates an instance of SelectChildTiersDlg
 	 * 
 	 * @param layoutManager, ElanLayoutManager
 	 * @param type1, type for first column
 	 * @param type2, type for second column
 	 * @param type3, type for third column
 	 */
    public SelectChildTiersDlg(ElanLayoutManager layoutManager, String type1, String type2, String type3){
        super(layoutManager.getElanFrame(), ElanLocale.getString("TranscriptionManager.SelectTierDlg.Title"), true);    
        transcription = layoutManager.getViewerManager().getTranscription();
		this.type1 = type1;
		this.type2 = type2;
		this.type3 = type3;
		
		if(type3 !=null){
       	 	TIER_COL3 = TIER_COL3 + type3;  
        }
		if(type2 !=null ){
			TIER_COL2 = TIER_COL2 + type2;  
        }
		if(type1 !=null ){
        	TIER_COL1 = TIER_COL1 + type1;
        }
		
		tierMapColumn1 = new HashMap<TierImpl, TierImpl>();
		tierMapColumn2 = new HashMap<TierImpl, TierImpl>();
		tierMapColumn3 = new HashMap<TierImpl, TierImpl>();
		
		initComponents();
		fillTable();
		postInit();
    }
    
    /**
     * Pack, size and set location.
     */
    private void postInit() {
    	pack();       
        setResizable(true);
        setLocationRelativeTo(getParent());
    }
	
	/**
     * Initializes the components for ui.
     */
    public void initComponents() {      	
//    	 setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
         getContentPane().setLayout(new GridBagLayout());
        
         model = new DefaultTableModel(){     	   
        	 public boolean isCellEditable(int row, int column) {        	
        		 if(getColumnName(column).equals(TIER_NAME_COLUMN)){
        			 return false;
        		 } 
        		 String value = (String)this.getValueAt(row, column);
        		 if(value != null && value.equals(NO_TIER)){
        			 return false;
        		 }        		
        		 
        		 return true;
        	 }
         };
         
         DefaultTableCellRenderer render = new DefaultTableCellRenderer(){
        	 public Component getTableCellRendererComponent(JTable table,
        			 Object value, boolean isSelected, boolean hasFocus, int row,
        			 	int column){
        		 
        		 if(table.getColumnName(column).equals(TIER_NAME_COLUMN)){
        			 JLabel label = new JLabel((String) value);
        			 label.setFont(new Font(table.getFont().getFontName(), Font.ITALIC, table.getFont().getSize()));
        			 label.setBackground(Color.LIGHT_GRAY);
        			 label.setOpaque(true);
        			 return label; 
        		 }
        		 
        		 Component cell = super.getTableCellRendererComponent(table, value, 
        				 isSelected, hasFocus, row, column);
        		 
        		 if(value.equals(defaultValue)){    			   
        			 cell.setForeground(Color.GRAY);
        		 } else {
        			 cell.setForeground(Color.BLACK);
        		 }        		       		
        		 return cell;    		   
        	 }
         };
         
         if(type3 !=null){
        	 model.setColumnCount(4);    
        	 model.setColumnIdentifiers(new String[] {TIER_NAME_COLUMN, TIER_COL1, TIER_COL2, TIER_COL3});  
         }else if(type2 !=null ){
        	 model.setColumnCount(3);    
        	 model.setColumnIdentifiers(new String[] {TIER_NAME_COLUMN, TIER_COL1, TIER_COL2});  
         } else if(type1 !=null ){
        	 model.setColumnCount(2);    
        	 model.setColumnIdentifiers(new String[] {TIER_NAME_COLUMN, TIER_COL1});  
         }
       
         selectionTable = new JTable(model);
         selectionTable.setFont(Constants.DEFAULTFONT);
         selectionTable.setCellSelectionEnabled(true);		
         selectionTable.setShowGrid(true);
         selectionTable.setGridColor(Color.BLACK);
         selectionTable.setDefaultEditor(Object.class, new TableCellEditor());   
         selectionTable.setRowHeight(25);         
         selectionTable.setDefaultRenderer(Object.class, render);               
         
         JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 6, 2));
         okButton = new JButton();
         okButton.setText(ElanLocale.getString("Button.Apply"));
         okButton.setEnabled(false);
         okButton.addActionListener(this);
         cancelButton = new JButton();
         cancelButton.setText(ElanLocale.getString("Button.Cancel"));
         cancelButton.addActionListener(this);
         buttonPanel.add(okButton);
         buttonPanel.add(cancelButton);
                
         Insets insets = new Insets(2, 6, 2, 6);
         GridBagConstraints gridBagConstraints = new GridBagConstraints(); 
         
         JScrollPane tierScrollPane = new JScrollPane(selectionTable);
         gridBagConstraints = new GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 0;
         gridBagConstraints.gridwidth = 1;
         gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
         gridBagConstraints.insets = insets;
         gridBagConstraints.fill = GridBagConstraints.BOTH;
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.weighty = 1.0;
         getContentPane().add(tierScrollPane, gridBagConstraints);  
         
         gridBagConstraints = new GridBagConstraints();
         gridBagConstraints.gridy = 2;
         gridBagConstraints.anchor = GridBagConstraints.SOUTH;
         gridBagConstraints.insets = insets;
         getContentPane().add(buttonPanel, gridBagConstraints); 
         
         addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent we) {
                 doClose();
             }
         });
    }
    
    /**
     * Closes this dialog
     */
    private void doClose() {
		setVisible(false);
		dispose();
	}
    
    private List<TierImpl> getLinkedTiersOfType(TierImpl refTier, String type){
    	List childTiers = refTier.getChildTiers();
    	List<TierImpl> linkedTiers = new ArrayList<TierImpl>();
    	for(int i=0; i < childTiers.size(); i++){
			TierImpl childTier = (TierImpl) childTiers.get(i);
			if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
				if(childTier.getLinguisticType().getLinguisticTypeName().equals(type)){	
					linkedTiers.add(childTier);
				}
				// get all the types of the tiers depending on this child tier
				List dependentTiers = childTier.getDependentTiers();
				for(int y=0; y < dependentTiers.size(); y++) {
					TierImpl dependantTier = (TierImpl) dependentTiers.get(y);	
					if(dependantTier.getLinguisticType().getLinguisticTypeName().equals(type)){								
						linkedTiers.add(dependantTier);	
					}
				}
			}
		}
    	return linkedTiers;	   
    }
    /**
     * Fills the table with values
     */
    private void fillTable(){   
    	
    	boolean column1ParentType = false;
    	List parentTierList = new ArrayList();
    	List type1TierList = transcription.getTiersWithLinguisticType(type1);
    	
    	if(type1TierList != null && type1TierList.size() > 0){
			TierImpl t = (TierImpl)type1TierList.get(0);	
			//check whether type1 is symbolic Associated
	 		if(t.getLinguisticType().getConstraints() != null && t.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION ){ 		
	 			
	 			for(int x = 0; x < type1TierList.size(); x++){
	 				TierImpl tier = (TierImpl)type1TierList.get(x);
					
	 				TierImpl parentTier = (TierImpl) tier.getParentTier();
					while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
						parentTier = (TierImpl) parentTier.getParentTier();
					}
					
					if(parentTierList.contains(parentTier)){
						continue;
					} else {						
						parentTierList.add(parentTier);	
					}
	 			}
	 		} else {
	 			parentTierList = type1TierList;
	 			column1ParentType = true;
	 		}
    	}
    	
       	for(int x = 0; x < parentTierList.size(); x++){
        	TierImpl refTier = (TierImpl)parentTierList.get(x);	 	
        	List<TierImpl> column1Tiers = new ArrayList<TierImpl>();
        	List<TierImpl> column2Tiers = new ArrayList<TierImpl>();
        	List<TierImpl> column3Tiers = new ArrayList<TierImpl>();
        	
        	List childTiers = refTier.getChildTiers();

        	String refTierName = refTier.getName();
        	String tierName1 = null;
        	String tierName2 = null;
        	String tierName3 = null;
        	if(column1ParentType){
        		column1Tiers.add(refTier);
        	} else {   
        		column1Tiers.addAll(getLinkedTiersOfType(refTier, type1));        		
        	}    
        	
        	if(column1Tiers.size() == 1){
                TierImpl t = (TierImpl) column1Tiers.get(0);
                tierName1= t.getName();
            } else if(column1Tiers.size() == 0) {
             	tierName1 = NO_TIER;
            }else {
            	tierName1 = defaultValue;
            }        	
        	
        	if(type2 != null){
        		column2Tiers.addAll(getLinkedTiersOfType(refTier, type2)); 
        	}
        	
        	if(type3 != null){
        		column3Tiers.addAll(getLinkedTiersOfType(refTier, type3));    
        	}
        	        	
        	if(column2Tiers.size()==1){
        		if(type2.equals(type1)){
        			tierName2 = NO_TIER;
        		} else {
        			TierImpl t1 = (TierImpl) column2Tiers.get(0);
        			tierName2= t1.getName();
        		}
            } else if(column2Tiers.size()==0){
            	tierName2 = NO_TIER;
            } else {
            	tierName2 = defaultValue;
            }
            	
            if(column3Tiers.size() == 1){
            	if(type2.equals(type3) || type1.equals(type3)){
            		tierName3= NO_TIER;
            	} else {
            		TierImpl t1 = (TierImpl) column3Tiers.get(0);
            		tierName3= t1.getName();
            	}
            }else if(column3Tiers.size()==0){
            	tierName3 = NO_TIER;
            }else {
            	tierName3 = defaultValue;
            	
            	if(type1.equals(type3) && type2.equals(type3)){
            		if(column3Tiers.size() <= 2){
            			tierName3 = NO_TIER;
            		}
                } else {
                	if(type2.equals(type3) || type1.equals(type3)){
                		if(column3Tiers.size() <= 1 ){ 
                		tierName3= NO_TIER;
                		}
                	}
                } 
           	} 	
            
        	
        	if(type3 !=null){
        		 model.addRow(new Object[] { refTierName, tierName1, tierName2, tierName3});  
        	} else if(type2 !=null){        		
            		model.addRow(new Object[] { refTierName, tierName1, tierName2}); 
        	} else if(type1 !=null){        		
            		model.addRow(new Object[] { refTierName, tierName1}); 
        	}
       	} 				
    }
    
    public boolean isValueChanged(){
    	return valueChanged;
    }
    
    public void actionPerformed(ActionEvent e) {
		if(e.getSource() == okButton){
			for(int i= 0; i < selectionTable.getRowCount(); i++){				
				String tierName1 = (String)selectionTable.getValueAt(i, selectionTable.getColumnModel().getColumnIndex(TIER_COL1));;
				String tierName2 = (String)selectionTable.getValueAt(i, selectionTable.getColumnModel().getColumnIndex(TIER_COL2));;
				String tierName3 = (String)selectionTable.getValueAt(i, selectionTable.getColumnModel().getColumnIndex(TIER_COL3));
				TierImpl tier = null;
				
				String message ="Tiers for each column shoud be unique. Select different tiers for all the columns.";
				
				if(!tierName1.equals(defaultValue) && !tierName1.equals(NO_TIER)){
					if(tierName1.equals(tierName2) || tierName1.equals(tierName3)){	
						JOptionPane.showMessageDialog(this, message,
						            ElanLocale.getString("Message.Warning"), JOptionPane.WARNING_MESSAGE);		
						return;
					}
				}
				
				if(!tierName2.equals(defaultValue) && !tierName2.equals(NO_TIER)){
					if(tierName2.equals(tierName3)){					
						
						JOptionPane.showMessageDialog(this, message,
						            ElanLocale.getString("Message.Warning"), JOptionPane.WARNING_MESSAGE);	
						return;
					}
				}
				
				TierImpl refTier = (TierImpl) transcription.getTierWithId((String)selectionTable.getValueAt(i, 
						selectionTable.getColumnModel().getColumnIndex(TIER_NAME_COLUMN)));					
				if(type1 !=null ){		        	
		        	 if(tierName1 != null && !tierName1.equals(defaultValue) && !tierName1.equals(NO_TIER)){
		        		 tier = (TierImpl) transcription.getTierWithId(tierName1);
		        	 }
		        	 tierMapColumn1.put(refTier, tier);
				}		         
		        
				if(type2 !=null ){		  
		        	tier = null;
		        	if(tierName2 != null && !tierName2.equals(defaultValue) && !tierName2.equals(NO_TIER)){
		        		tier = (TierImpl) transcription.getTierWithId(tierName2);
		        	}
		        	tierMapColumn2.put(refTier, tier);
		        } 		         
		       
		        if(type3 !=null){	
		        	tier = null;
		        	if(tierName3 != null && !tierName3.equals(defaultValue) && !tierName3.equals(NO_TIER)){
		        		tier = (TierImpl) transcription.getTierWithId(tierName3);
		        	}
		        	tierMapColumn3.put(refTier, tier);
		        }
			}	
			valueChanged = true;
			doClose();			
		}else if (e.getSource() == cancelButton){
			tierMapColumn1.clear();
			tierMapColumn2.clear();
			tierMapColumn3.clear();			
			doClose();
		}		
	}
    
    /**
     * Returns the tierMap which is mapped with the parent tier 
     * and the tier for first column
     * 
     * @return tierMap, map with parentTier and first column linked tier
     */
    public HashMap<TierImpl, TierImpl> getTierMapColumn1(){
    	return tierMapColumn1;
    }
    
    /**
     * Returns the tierMap which is mapped with the parent tier 
     * and the tier for first column
     * 
     * @return tierMap, map with parentTier and second column linked tier
     */
    public HashMap<TierImpl, TierImpl> getTierMapColumn2(){
    	return tierMapColumn2;
    }
    
    /**
     * Returns the tierMap which is mapped with the parent tier 
     * and the tier for first column
     * 
     * @return tierMap, map with parentTier and second column linked tier
     */
    public HashMap<TierImpl, TierImpl> getTierMapColumn3() {
    	return tierMapColumn3;		
	}
    
    /**
     * Cell Editor for the selection JTable
     * 
     * @author aarsom
     *
     */
    private class TableCellEditor extends DefaultCellEditor implements ActionListener {    	
    	private int startEditInOneClick = 1;
    	private JComboBox comboBox;

    	public TableCellEditor() {
    		super(new JComboBox());
    		setClickCountToStart(startEditInOneClick);      		
    	}
    	
    	public Component getTableCellEditorComponent(
    			JTable table,
    			Object value,
    			boolean isSelected,
    			int row,
    			int column) {
    		
    		List<TierImpl> linkedTiers = new ArrayList<TierImpl>();
    		String refTier = (String) table.getValueAt(row,table.getColumnModel().getColumnIndex(TIER_NAME_COLUMN));
    		TierImpl tier = (TierImpl) transcription.getTierWithId(refTier);
    		
//    		if(tier.getLinguisticType().getLinguisticTypeName().equals(TIER_COL1){
//    			linkedTiers.add(tier);
//    		} else {
    			if(table.getColumnName(column).equals(TIER_COL1)){    			
    				linkedTiers = getLinkedTiersOfType(tier,type1);        		
    			} else if(table.getColumnName(column).equals(TIER_COL2)){
    				linkedTiers = getLinkedTiersOfType(tier,type2);        		
    			} else if(table.getColumnName(column).equals(TIER_COL3)){
    				linkedTiers = getLinkedTiersOfType(tier,type3);
    			}
//    		}
    		
    		if(linkedTiers.size() <= 1){
    			return null;
    		}
    		
    		
    		comboBox = new JComboBox();    		
    		
    		for(int i=0; i< linkedTiers.size(); i++){
				TierImpl dependentTier = linkedTiers.get(i); 				
				comboBox.addItem(dependentTier.getName());
    		}
    		comboBox.addActionListener(this);
    		
    		
//    		if(dependingTiers != null){
//				for(int i=0; i< dependingTiers.size();i++){
//					TierImpl dependentTier = (TierImpl)dependingTiers.get(i); 				
//					if(dependentTier.getLinguisticType().getLinguisticTypeName().equals(type)){
//						if(dependentTier.hasParentTier()){
//							if(dependentTier.getParentTier() == tier){
//								comboBox.addItem(dependentTier.getName());
//							}
//						}
//					}
//				}
//    		}
    		
//    		if(comboBox.getItemCount() <= 0){
//    			table.setValueAt("No Tiers", row, column);
//    			return null;
//    		}
    		return comboBox;
    	}
    	
    	public Object getCellEditorValue() {    		
    		return comboBox.getSelectedItem();
    	}

		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == comboBox ){
				if (comboBox .getSelectedItem() != null){
					okButton.setEnabled(true);	
					selectionTable.editingStopped(new ChangeEvent(this));
				}
			}
		}
    }	
}
