package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import mpi.eudico.client.annotator.ElanLayoutManager;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.layout.TranscriptionManager;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clomimpl.type.LinguisticType;


import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import mpi.eudico.client.annotator.gui.ClosableDialog;
import mpi.eudico.client.annotator.gui.HTMLViewer;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.util.gui.SimpleHtmlViewer;

/**
 * Dialog to select the linguistic types for different columns 
 * in the transcription table
 * 
 * @author Aarthy Somasundaram
 *
 */
public class TranscriptionModeSettingsDlg extends ClosableDialog implements ActionListener{
    
    private Transcription transcription ;
    private ElanLayoutManager layoutManager;
    
    private JButton applyButton;
    private JButton cancelButton;

    private JComboBox columnType1ComboBox;
    private JComboBox columnType2ComboBox;	
    private JComboBox columnType3ComboBox;
    private JComboBox fontSizeComboBox;
	private JButton butSelectTiers;
	private JButton helpButton;
    
    private List<String> linkedTypes;    
    private String columnType1;
    private String columnType2;
    private String columnType3;	
 	private HashMap<TierImpl, TierImpl> tierMapColumn1;
 	private HashMap<TierImpl, TierImpl> tierMapColumn2;
 	private HashMap<TierImpl, TierImpl> tierMapColumn3;
 	
 	private boolean valueChanged = false;
 	
 	/**
 	 * Creates a TranscriptionModeSettingsDlg instance
 	 * 
 	 * @param layoutManager ElanLayoutManager
 	 * @param columnType1,  type for the first column
 	 * @param columnType2,  type for the second column
 	 * @param columnType3,  type for the third column
 	 * @param size, font size for the transcription table
 	 */
    public TranscriptionModeSettingsDlg(ElanLayoutManager layoutManager, String columnType1, String columnType2, String columnType3, Integer size){
        super(layoutManager.getElanFrame(), ElanLocale.getString("TranscriptionManager.ChangeSettingsDlg.Title"), true);        
        transcription = layoutManager.getViewerManager().getTranscription();
        linkedTypes = new ArrayList<String>();
        this.columnType1 = columnType1;
        this.columnType2 = columnType2;
        this.columnType3 = columnType3;
        this.layoutManager = layoutManager;
        
        tierMapColumn1 = new HashMap<TierImpl, TierImpl>();
		tierMapColumn2 = new HashMap<TierImpl, TierImpl>();
		tierMapColumn3 = new HashMap<TierImpl, TierImpl>();        
        
		initComponents();		
		if(size != null){
			fontSizeComboBox.setSelectedItem(size);
		}
		postInit();
    }
    
    /**
     * Pack, size and set location.
     */
    private void postInit() {
    	pack();       
        setResizable(true);
        setLocationRelativeTo(getParent());
    }
	
	/**
     * Initializes the components for ui.
     */
    private void initComponents() {  
    	
	    columnType1ComboBox = new JComboBox();	 
	    columnType1ComboBox.addItem(TranscriptionManager.COMBOBOX_DEFAULT_STRING);
	    
	    columnType2ComboBox = new JComboBox();	
		columnType2ComboBox.setEnabled(false);
		
		columnType3ComboBox = new JComboBox();	
		columnType3ComboBox.setEnabled(false);
		
		fontSizeComboBox = new JComboBox();
		for(int i = 10 ; i <= 20; i++){
			fontSizeComboBox.addItem(new Integer(i));
		}
		
		butSelectTiers = new JButton(ElanLocale.getString("TranscriptionModeSettingsDlg.SelectTier"));		
		butSelectTiers.setEnabled(false);
		
		helpButton = new JButton(new ImageIcon(this.getClass().getResource("/toolbarButtonGraphics/general/Help24.gif")));
		helpButton.setPreferredSize(new Dimension(24,24));
		//helpButton.setBorder(null);
		
		applyButton = new JButton();
        applyButton.setText(ElanLocale.getString("Button.Apply"));
        applyButton.setEnabled(false);  
        
        cancelButton = new JButton();
        cancelButton.setText(ElanLocale.getString("Button.Cancel"));
		
			    
	    List types = transcription.getLinguisticTypes();		    
		for(int i = 0; i < types.size(); i++){
			LinguisticType type = (LinguisticType)types.get(i);	
			Vector tierList = transcription.getTiersWithLinguisticType(type.getLinguisticTypeName());
			if( tierList!= null && tierList.size() > 0){
				columnType1ComboBox.addItem(type.getLinguisticTypeName());
			}
		}
		
		getContentPane().setLayout(new GridBagLayout());
		
//		JLabel infoLabel = new JLabel();
//		infoLabel.setText("For more information on how to select the types");
		JPanel settingsPanel = getSettingsPanel();
		//infoLabel.setPreferredSize(new Dimension((int)settingsPanel.getPreferredSize().getWidth(), infoLabel.getPreferredSize().height));
				
		GridBagConstraints gbc = new GridBagConstraints();
	    gbc.gridx = 0;
	    gbc.gridy = 0;
	    gbc.anchor =gbc.WEST;
	    gbc.fill = gbc.NONE;	    
	    gbc.weighty = 0.5;	
	    gbc.insets = new Insets(4,6,4,6);
	   // getContentPane().add(infoLabel,gbc);
	    
	    gbc.gridx = 0;
	    gbc.gridy = 1;
	    gbc.gridwidth = 2;
	    getContentPane().add(settingsPanel,gbc);
	    
	    gbc.gridx = 1;
	    gbc.gridy = 0;
	    gbc.anchor =gbc.EAST;
	    getContentPane().add(helpButton,gbc);
		
		columnType1ComboBox.addActionListener(this);
	    columnType2ComboBox.addActionListener(this);
	    columnType3ComboBox.addActionListener(this);
 	    cancelButton.addActionListener(this);
	    applyButton.addActionListener(this);
	    butSelectTiers.addActionListener(this);
	    helpButton.addActionListener(this);
	    
 	   if(columnType1 != null){
			columnType1ComboBox.setSelectedItem(columnType1);
			if(columnType1ComboBox.getSelectedIndex() < 0 ){
				columnType1ComboBox.setSelectedIndex(0);
			}			
		}
 	   
 	  if(columnType2 != null){
			columnType2ComboBox.setSelectedItem(columnType2);
			if(columnType2ComboBox.getSelectedIndex() < 0  && columnType2ComboBox.getItemCount() > 0 ){
				columnType2ComboBox.setSelectedIndex(0);
			}
		}
 	   
 	 if(columnType3 != null){
			columnType3ComboBox.setSelectedItem(columnType3);
			if(columnType3ComboBox.getSelectedIndex() < 0  && columnType3ComboBox.getItemCount() > 0){
				columnType3ComboBox.setSelectedIndex(0);
			}
		}	
 	  
 	  columnType1 = null;
      columnType2 = null;
      columnType3 = null;
 	    addWindowListener(new WindowAdapter() {
 	    	public void windowClosing(WindowEvent we) {
 	    		doClose();
            }
        });
    }
    
    
    private JPanel getSettingsPanel(){
    	JPanel panel = new JPanel();
    	panel.setLayout(new GridBagLayout());
    	panel.setBorder(new TitledBorder(ElanLocale.getString("TranscriptionModeSettingsDlg.Title"))); 
		
		JLabel columnType1Label = new JLabel(ElanLocale.getString("TranscriptionModeSettingsDlg.Column1Type"));
		JLabel columnType2Label = new JLabel(ElanLocale.getString("TranscriptionModeSettingsDlg.Column2Type"));
		JLabel columnType3Label = new JLabel(ElanLocale.getString("TranscriptionModeSettingsDlg.Column3Type"));
		JLabel fontSizeLabel   = new JLabel(ElanLocale.getString("TranscriptionModeSettingsDlg.FontSize"));
    	
    	// layout for the components in the settings panel
		GridBagConstraints gbc = new GridBagConstraints();
	    gbc.gridx = 0;
	    gbc.gridy = 0;
	    gbc.anchor =gbc.WEST;
	    gbc.fill = gbc.NONE;	    
	    gbc.weighty = 0.5;	
	    gbc.insets = new Insets(4,6,4,6);
	    panel.add(columnType1Label,gbc);
		
	    gbc.gridy = 1;
	    panel.add(columnType2Label,gbc);
	    
	    gbc.gridy = 2;
	    panel.add(columnType3Label,gbc);
	    
		gbc.gridx = 1;
	    gbc.gridy = 0;
	    gbc.weightx = 0.5;
	    panel.add(columnType1ComboBox,gbc);	
	    
	    gbc.gridx = 1;	
	    gbc.gridy = 1;
	    panel.add(columnType2ComboBox,gbc);	
	    
	    gbc.gridx = 1;	
	    gbc.gridy = 2;
	    panel.add(columnType3ComboBox,gbc);		    
    
        JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 6, 2));               
        buttonPanel.add(applyButton);
        buttonPanel.add(cancelButton);  
 	    
 	    gbc.gridx = 0;	
	    gbc.gridy = 4;
	    panel.add(butSelectTiers,gbc);
	    
 	    gbc.gridx = 0;	
	    gbc.gridy = 3;	   
	    panel.add(fontSizeLabel,gbc);
	    
	    gbc.gridx = 1;	
	    gbc.gridy = 3;
	    panel.add(fontSizeComboBox,gbc);
	    
	    gbc.gridx = 1;	
 	    gbc.gridy = 4;
 	    gbc.anchor = gbc.EAST;
 	    panel.add(buttonPanel,gbc);	
 	    
 	    return panel;   	
    }
    
    /**
     * Get all the types linked with the selected type
     * value in columnType1ComboBox
     */
    private void getLinkedTypes(){   
    	linkedTypes.clear();
    	
    	List tierList = transcription.getTiersWithLinguisticType(columnType1ComboBox.getSelectedItem().toString());	
    	List<TierImpl> parentTiers = new ArrayList<TierImpl>();
    	
		for(int i= 0 ; i < tierList.size(); i++){
			TierImpl tier = (TierImpl) tierList.get(i);		
			
			List childTiers = new ArrayList();
			
			//if type selected for the first column is not symbolic associated type
			if(tier.getLinguisticType().getConstraints() == null || tier.getLinguisticType().getConstraints().getStereoType() != Constraint.SYMBOLIC_ASSOCIATION ){
				childTiers = tier.getChildTiers();
			} else {
				// if not,
				TierImpl parentTier = (TierImpl) tier.getParentTier();
				while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
					parentTier = (TierImpl) parentTier.getParentTier();
				}
				
				childTiers.addAll(parentTier.getChildTiers());
				childTiers.remove(tier);
			}
			
			for(int x=0; x < childTiers.size(); x++){
				TierImpl childTier = (TierImpl) childTiers.get(x);
				if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
					// get all the types of the tiers depending on this child tier
					List dependentTiers = childTier.getDependentTiers();
					for(int y=0; y < dependentTiers.size(); y++) {
						TierImpl dependantTier = (TierImpl) dependentTiers.get(y);	
						if(dependantTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
							if(!linkedTypes.contains(dependantTier.getLinguisticType().getLinguisticTypeName())){
								linkedTypes.add(dependantTier.getLinguisticType().getLinguisticTypeName());
							}
						}
					}
					if(!linkedTypes.contains(childTier.getLinguisticType().getLinguisticTypeName())){
						linkedTypes.add(childTier.getLinguisticType().getLinguisticTypeName());
					}
				}
			}
		}
			
  	
//    	List tierList = transcription.getTiersWithLinguisticType(columnType1ComboBox.getSelectedItem().toString());	
//    	List<TierImpl> parentTiers = new ArrayList<TierImpl>();
//    	
//		for(int i= 0 ; i < tierList.size(); i++){
//			TierImpl tier = (TierImpl) tierList.get(i);		
//			
//			// if column1Type is  symbolic associated, get all dependent tiers of the same type of a same parent
//			if(tier.getLinguisticType().getConstraints() != null && tier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION ){
//				List linkedTiers = new ArrayList();
//				
//				TierImpl parentTier = (TierImpl) tier.getParentTier();
//				while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null || parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
//					parentTier = (TierImpl) parentTier.getParentTier();
//				}
//				//linkedTiers.addAll(tier.getChildTiers());
//				//linkedTiers.addAll(((TierImpl)tier.getParentTier()).getChildTiers());
//				if(parentTier.getDependentTiers() !=null){
//					linkedTiers.addAll(parentTier.getDependentTiers());
//				}
//				
//				linkedTiers.remove(tier);
//				for(int x=0; x< linkedTiers.size(); x++){
//					TierImpl linkedTier = (TierImpl) linkedTiers.get(x);
//					if(linkedTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
//						if(!linkedTypes.contains(linkedTier.getLinguisticType().getLinguisticTypeName())){
//							linkedTypes.add(linkedTier.getLinguisticType().getLinguisticTypeName());
//						}
//					}
//				}
//			} else{
//				// if column1Type is something else,  get all dependent tiers with the same type under this tier 				
//				List linkedTiers = tier.getChildTiers();
//				
//				for(int x=0; x < linkedTiers.size(); x++){
//					TierImpl childTier = (TierImpl) linkedTiers.get(x);
//					if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
//						if(!linkedTypes.contains(childTier.getLinguisticType().getLinguisticTypeName())){
//							linkedTypes.add(childTier.getLinguisticType().getLinguisticTypeName());
//						}
//					}
//				} 
//			}
//		}
    }
    
    /**
	 * Loads the columnType2ComboBox values according to 
	 * the selected type for the first column
	 */
	private void initializecolumnType2ComboBox(){
		columnType2ComboBox.removeActionListener(this);
		columnType2ComboBox.removeAllItems();
		
		//boolean removeType2 = false;
				
//		if(linkedTypes.contains(columnType1ComboBox.getSelectedItem().toString())){		
//			removeType2 = true;
//			Vector tiersOfType1 = transcription.getTiersWithLinguisticType(columnType1ComboBox.getSelectedItem().toString());
//			List<TierImpl> parentTiers = new ArrayList<TierImpl>();
//			TierImpl tier = (TierImpl) tiersOfType1.get(0);
//			for(int i=0; i < tiersOfType1.size(); i++){
//				TierImpl t = (TierImpl) tiersOfType1.get(i);
//				if(t.hasParentTier()){						
//					if(parentTiers.contains(t.getParentTier())){
//						removeType2 = false; 
//						break;
//					} else {
//						parentTiers.add((TierImpl) t.getParentTier());
//					}
//				}
//			}
//		}		
			
		columnType2ComboBox.addItem(TranscriptionManager.COMBOBOX_DEFAULT_STRING);		
		
		for(int i = 0; i < linkedTypes.size(); i++){
			columnType2ComboBox.addItem(linkedTypes.get(i));
		}
		
//		if(removeType2){
//			columnType2ComboBox.removeItem(columnType1ComboBox.getSelectedItem());
//		}
		
		if(columnType2ComboBox.getItemCount() > 1){			 
			 columnType2ComboBox.setEnabled(true);
			 columnType2ComboBox.addActionListener(this);	
//			 if(columnType2 != null){
//				columnType2ComboBox.setSelectedItem(columnType2);
//				if(columnType2ComboBox.getSelectedIndex() < 0 ){
//					columnType2ComboBox.setSelectedIndex(0);
//				}
//			}
		} else{
			if(columnType2ComboBox.getItemCount() == 1){			 
				columnType2ComboBox.removeAllItems();
			}
			columnType3ComboBox.removeAllItems();
			columnType2ComboBox.setEnabled(false);
		}
	}	
	
    /**
	 * Loads the columnType3ComboBox values according to 
	 * the values selected in the columnType1ComboBox &
	 * columnType2ComboBox
	 */
	private void initializecolumnType3ComboBox(){
		columnType3ComboBox.removeActionListener(this);
		columnType3ComboBox.removeAllItems();		
		columnType3ComboBox.addItem(TranscriptionManager.COMBOBOX_DEFAULT_STRING);
		 
		for(int i = 0; i < linkedTypes.size(); i++){
			columnType3ComboBox.addItem(linkedTypes.get(i));
		}		
		
		Vector linkedTiers = transcription.getTiersWithLinguisticType(columnType2ComboBox.getSelectedItem().toString());
		List<TierImpl> parentTiers = new ArrayList<TierImpl>();
		boolean removeType2 = true;		
		
		// if a type1 == type2, check whether the third type can be the same 
		if(columnType2ComboBox.getSelectedItem().toString().equals(columnType1ComboBox.getSelectedItem().toString())){
			if(linkedTiers !=null){
				for(int i=0; i < linkedTiers.size(); i++){
					TierImpl tier = (TierImpl) linkedTiers.get(i);
					
					// if type1 == type2 then the types are symbolically associated
					TierImpl parentTier = (TierImpl) tier.getParentTier();
					while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
						parentTier = (TierImpl) parentTier.getParentTier();
					}
					
					if(parentTiers.contains(parentTier)){					
						continue;
					} else {
						parentTiers.add(parentTier);
						List childTiers = parentTier.getChildTiers();
						int n = 0;
						for(int x = 0; x < childTiers.size(); x++){
							TierImpl childTier = (TierImpl) childTiers.get(x);
							if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
								if(childTier.getLinguisticType().getLinguisticTypeName().equals(columnType2ComboBox.getSelectedItem().toString())){
									n++;
								}
								
								// get all the types of the tiers depending on this child tier
								List dependentTiers = childTier.getDependentTiers();
								for(int y=0; y < dependentTiers.size(); y++) {
									TierImpl dependantTier = (TierImpl) dependentTiers.get(y);	
									if(dependantTier.getLinguisticType().getLinguisticTypeName().equals(columnType2ComboBox.getSelectedItem().toString())){
										n++;
									}
								}
							}							
							
							if(n==3){
								removeType2 = false;								
								break;
							}
						}
					}
				}
			}								
		} else {
			
			//check whether the type for column3 can be the same as the type for second column 
			if(linkedTiers !=null){
				for(int i=0; i < linkedTiers.size(); i++){
					TierImpl tier = (TierImpl) linkedTiers.get(i);
					
					TierImpl parentTier = (TierImpl) tier.getParentTier();
					while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
						parentTier = (TierImpl) parentTier.getParentTier();
					}
					
					if(parentTiers.contains(parentTier)){					
						continue;
					} else {
						parentTiers.add((TierImpl) tier.getRootTier());
						List childTiers = parentTier.getChildTiers();
						int n = 0;
						for(int x = 0; x < childTiers.size(); x++){
							TierImpl childTier = (TierImpl) childTiers.get(x);
							if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
								if(childTier.getLinguisticType().getLinguisticTypeName().equals(columnType2ComboBox.getSelectedItem().toString())){
									n++;
								}
								
								// get all the types of the tiers depending on this child tier
								List dependentTiers = childTier.getDependentTiers();
								for(int y=0; y < dependentTiers.size(); y++) {
									TierImpl dependantTier = (TierImpl) dependentTiers.get(y);	
									if(dependantTier.getLinguisticType().getLinguisticTypeName().equals(columnType2ComboBox.getSelectedItem().toString())){
										n++;
									}
								}
							}							
							
							if(n==2){
								removeType2 = false;								
								break;
							}
						}
					}
				}
			}
		}		

//		// if a same type is selected for both the column, check made whether the third type for column3 can be the same or not
//		if(columnType2ComboBox.getSelectedItem().toString().equals(columnType1ComboBox.getSelectedItem().toString())){
//			if(!linkedTypes.contains(columnType1ComboBox.getSelectedItem().toString())){
//				removeType2 = false;
//				removeType1 = false ;
//			} else {
//				if(linkedTiers !=null){
//					for(int i=0; i < linkedTiers.size(); i++){
//					TierImpl tier = (TierImpl) linkedTiers.get(i);
//					if(tier.hasParentTier()){
//						if(parentTiers.contains(tier.getRootTier())){					
//							continue;
//						} else {
//							parentTiers.add((TierImpl) tier.getRootTier());
//							Vector childTiers = ((TierImpl)tier.getRootTier()).getDependentTiers();
//							if(childTiers != null){
//								int n = 0;
//								for(int x = 0; x < childTiers.size(); x++){
//									TierImpl childTier = (TierImpl) childTiers.get(x);
//									if(childTier.getLinguisticType().getLinguisticTypeName().equals(columnType2ComboBox.getSelectedItem().toString())){
//										n++;
//									}									
//									if(n==3){
//										removeType2 = false;
//										removeType1 = false;
//										break;
//									}
//								}								
//								if(n==3){
//									break;
//								}
//							}
//						}
//					}
//				}
//			}
//			}
//		} else {
//			//check made whether the third type for column3 can be the same as the type for second column or not
//			if(linkedTiers !=null){
//				for(int i=0; i < linkedTiers.size(); i++){
//					TierImpl tier = (TierImpl) linkedTiers.get(i);
//					if(tier.hasParentTier()){
//						if(parentTiers.contains(tier.getParentTier())){						
//							removeType2 = false;
//							break;
//						}else {
//							parentTiers.add((TierImpl) tier.getParentTier());
//						}
//					}
//				}				
//			}
//			
//			//check made whether the third type for column3 can be the same as the type for first column or not
//			linkedTiers = transcription.getTiersWithLinguisticType(columnType1ComboBox.getSelectedItem().toString());		
//			parentTiers.clear();
//			if(linkedTiers !=null){
//				for(int i=0; i < linkedTiers.size(); i++){
//					TierImpl tier = (TierImpl) linkedTiers.get(i);
//					if(tier.hasParentTier()){
//						if(parentTiers.contains(tier.getParentTier())){						
//							removeType1 = false;
//							break;
//						}else {
//							parentTiers.add((TierImpl) tier.getParentTier());
//						}
//					}
//				}				
//			}
//		}		
		
		if(removeType2){
			columnType3ComboBox.removeItem(columnType2ComboBox.getSelectedItem());			
		}
		
		if(columnType3ComboBox.getItemCount() > 1){			
			columnType3ComboBox.setEnabled(true);
			columnType3ComboBox.addActionListener(this);	
//			if(columnType3 != null){
//				columnType3ComboBox.setSelectedItem(columnType3);
//				if(columnType3ComboBox.getSelectedIndex() < 0 ){
//					columnType3ComboBox.setSelectedIndex(0);
//				}
//			}	
		} else {
			columnType3ComboBox.removeAllItems();
			columnType3ComboBox.setEnabled(false);
		}
	}
	
	/**
	 * Enables or disables the option to select the tiers button
	 * according to the types selected
	 * 
	 * @param type, the selected type
	 */
	private void enableOrDisableSeletTierButton(String type){		
		if(butSelectTiers.isEnabled()){
			return;
		}
		butSelectTiers.setEnabled(false);
		Vector linkedTiers = transcription.getTiersWithLinguisticType(type);
		List<TierImpl> parentTiers = new ArrayList<TierImpl>();
				
		for(int i=0; i < linkedTiers.size(); i++){
			TierImpl tier = (TierImpl) linkedTiers.get(i);			
			
			//if type selected for the first column is not symbolic associated type
			if(tier.getLinguisticType().getConstraints() == null || tier.getLinguisticType().getConstraints().getStereoType() != Constraint.SYMBOLIC_ASSOCIATION ){
				return;
			} 
				
			TierImpl parentTier = (TierImpl) tier.getParentTier();
			while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
				parentTier = (TierImpl) parentTier.getParentTier();
			}				
			if(parentTiers.contains(parentTier)){					
				continue;
			}	
			
			List childTiers = parentTier.getChildTiers();			
			int n = 0;
			for(int x = 0; x < childTiers.size(); x++){
				TierImpl childTier = (TierImpl) childTiers.get(x);
				
				if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
					if(childTier.getLinguisticType().getLinguisticTypeName().equals(type)){
						n++;
					}
					
					// get all the types of the tiers depending on this child tier
					List dependentTiers = childTier.getDependentTiers();
					for(int y=0; y < dependentTiers.size(); y++) {
						TierImpl dependantTier = (TierImpl) dependentTiers.get(y);	
						if(dependantTier.getLinguisticType().getLinguisticTypeName().equals(type)){
							n++;
						}
					}										
					
					if(n > 1){
						butSelectTiers.setEnabled(true);
						break;
					} 
				}			
			}
		}
//		// gets all the parent tiers of the tiers of selected type
//		if(linkedTiers !=null && linkedTiers.size() > 0){
//			TierImpl parentTier = (TierImpl) linkedTiers.get(0);
//			if(parentTier.hasParentTier()){
//				for(int i=0; i < linkedTiers.size(); i++){
//					TierImpl tier = (TierImpl) linkedTiers.get(i);
//					if(tier.hasParentTier()){						
//						if(!parentTiers.contains(tier.getParentTier())){
//							parentTiers.add((TierImpl) tier.getParentTier());
//						}
//					} 
//				}
//			}
//		} 
//		
//		// checks if the parent tier has more than one tiers of the selected type
//		for(int x= 0; x< parentTiers.size(); x++){
//			int n=0;
//			Vector dependingTiers = ((TierImpl)parentTiers.get(x)).getDependentTiers();				 			
//			if(dependingTiers != null){
//				for(int i=0; i< dependingTiers.size();i++){
//					TierImpl dependentTier = (TierImpl)dependingTiers.get(i); 				
//					if(dependentTier.getLinguisticType().getLinguisticTypeName().equals(type)){
//						if(dependentTier.hasParentTier()){
//							if(dependentTier.getParentTier() == (TierImpl)parentTiers.get(x)){
//								n++;
//								if(n > 1){
//									butSelectTiers.setEnabled(true);
//									break;
//								} 
//							}
//						}
//					}
//				}
//			}
//		}	
	}
	
	/**
	 * Getter method to get the type for the column1
	 * 
	 * @return columnType1, the type for column1
	 */
	public String getTypeForFirstColumn(){
		return columnType1;
	}
	
	/**
	 * Getter method to get the type for the column2
	 * 
	 * @return columnType2, the type for column2
	 */
	public String getTypeForSecondColumn(){
		return columnType2;
	}
	
	/**
	 * Getter method to get the type for the column3
	 * 
	 * @return columnType3, the type for column3
	 */
	public String getTypeForThirdColumn(){
		return columnType3;
	}
	
	/**
	 * Returns the tierMap for the first column
	 * 
	 * @return tierMapColumn1, the tierMap  for column1
	 */
	public HashMap<TierImpl, TierImpl> getTierMapColumn1(){
	   	return tierMapColumn1;
	}
	
	/**
	 * Returns the tierMap for the second column
	 * 
	 * @return tierMapColumn2, the tierMap  for column3
	 */
	public HashMap<TierImpl, TierImpl> getTierMapColumn2(){
	   	return tierMapColumn2;
	}
	    
	/**
	 * Returns the tierMap for the third column
	 * 
	 * @return tierMapColumn3, the tierMap  for column3
	 */
	public HashMap<TierImpl, TierImpl> getTierMapColumn3() {
	   	return tierMapColumn3;		
	}
	
	/**
	 * Returns the font size for the table
	 * 
	 * @return the font size
	 */
	public Integer getFontSize(){
		return (Integer)fontSizeComboBox.getSelectedItem();
	}
    
    /**
     * Closes this dialog
     */
    private void doClose() {
		setVisible(false);
		dispose();
	} 
    
    public boolean isValueChanged(){
    	return valueChanged;
    }
    
    public void actionPerformed(ActionEvent e) {
		if(e.getSource() == applyButton){
			columnType1 = (String) columnType1ComboBox.getSelectedItem();
			columnType2 = (String) columnType2ComboBox.getSelectedItem();
			columnType3 = (String) columnType3ComboBox.getSelectedItem();
			doClose();			
		}else if (e.getSource() == cancelButton){	
			columnType1 = null;
			columnType2 = null;
			columnType3 = null;
			doClose();
		}
		else if(e.getSource() == columnType1ComboBox){			
			columnType3ComboBox.removeActionListener(this);
			columnType3ComboBox.removeAllItems();
			columnType3ComboBox.addActionListener(this);
			columnType3ComboBox.setEnabled(false);		
			
			if(columnType1ComboBox.getSelectedIndex() <= 0){
				columnType2ComboBox.removeActionListener(this);
				columnType2ComboBox.removeAllItems();
				columnType2ComboBox.addActionListener(this);
				columnType2ComboBox.setEnabled(false);
				return;
			} else if(columnType1ComboBox.getSelectedIndex() > 0){
				butSelectTiers.setEnabled(false);
				enableOrDisableSeletTierButton(columnType1ComboBox.getSelectedItem().toString());
				applyButton.setEnabled(true);
			}
			getLinkedTypes();
			initializecolumnType2ComboBox();
		} 
		else if ( e.getSource() == columnType2ComboBox){	
			if(columnType2ComboBox.getSelectedIndex() <= 0){
				columnType3ComboBox.removeActionListener(this);
				columnType3ComboBox.removeAllItems();
				columnType3ComboBox.addActionListener(this);
				columnType3ComboBox.setEnabled(false);
				return;
			} else if(columnType2ComboBox.getSelectedIndex() > 0){
				enableOrDisableSeletTierButton(columnType2ComboBox.getSelectedItem().toString());
			}
			
			initializecolumnType3ComboBox();
		} 
		else if ( e.getSource() == columnType3ComboBox){	
			if(columnType3ComboBox.getSelectedIndex() > 0){
				enableOrDisableSeletTierButton(columnType3ComboBox.getSelectedItem().toString());
			}
		} else if(e.getSource() == butSelectTiers){
			String columnType1 = (String) columnType1ComboBox.getSelectedItem();
			String columnType2 = (String) columnType2ComboBox.getSelectedItem();
			String columnType3 = (String) columnType3ComboBox.getSelectedItem();
			
			if(columnType2 != null && columnType2.equals(TranscriptionManager.COMBOBOX_DEFAULT_STRING)){
				columnType2 = null;
			}
			
			if(columnType3 !=  null && columnType3.equals(TranscriptionManager.COMBOBOX_DEFAULT_STRING)){
				columnType3 = null;
			}
		
			SelectChildTiersDlg dialog = new SelectChildTiersDlg(layoutManager, columnType1, columnType2, columnType3);
			dialog.setVisible(true);	
			valueChanged = dialog.isValueChanged();
			
			tierMapColumn1 = dialog.getTierMapColumn1();
			tierMapColumn2 = dialog.getTierMapColumn2();
			tierMapColumn3 = dialog.getTierMapColumn3();		
		} else if(e.getSource() == helpButton){
			//this.
			showHelpDialog();
		}
	}
    
    private void showHelpDialog() {
    	try {
    		HTMLViewer helpViewer = new HTMLViewer("/mpi/eudico/client/annotator/resources/transcriptionModeHelpDoc.html", 
    			false, ElanLocale.getString("TranscriptionModeSettingsDlg.Help"));
    		JDialog dialog = helpViewer.createHTMLDialog(this);
    		dialog.pack();
    		dialog.setSize(500, 600);
    		dialog.setVisible(true);
    	} catch (IOException ioe) {
    		// message box
    		JOptionPane.showMessageDialog(this, ("Unable to load the Transcription mode help file " + ioe.getMessage()), 
    				ElanLocale.getString("Message.Warning"), JOptionPane.WARNING_MESSAGE, null);
    	}
    }
}



