package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.Color;
import java.awt.Component;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import java.util.Iterator;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.util.TableSubHeaderObject;
import mpi.eudico.server.corpora.clom.Annotation;

/**
 * JTable specially made for easy transcription and translation,
 * in the transcription mode
 * 
 * @author Aarthy Somasundaram
 *
 */
public class TranscriptionTable extends JTable {
	
	private HashMap prefTierFontColors;
	private HashMap<String, Color> prefTierBrighterFontColors;
	private HashMap<String, Color> prefTierBrightestFontColors;
	private HashMap prefTierFonts;		
	/** stores width of parent component */
	private int width = -1;
	/** Default height of a row */	
	private final int rowHeight;
	
	private int currentRowIndex = -1;
	private int currentColumnIndex = -1;
	
	private boolean moveViaColumn = false;	
	private boolean scrollToCenter = false;		
	private boolean thirdColumnActivated = false;
	private boolean storeColumnOrder = true;
	
	/**
	 * Creates an instance of TranscriptionTable
	 */
	public TranscriptionTable( ) {	
		rowHeight = getRowHeight() + 5;
		prefTierFontColors = new HashMap();
		prefTierBrighterFontColors = new HashMap<String, Color>();
		prefTierBrightestFontColors = new HashMap<String, Color>();
		prefTierFonts = new HashMap();		
		
		// table settings
		setFont(Constants.DEFAULTFONT);
		setCellSelectionEnabled(true);		
		setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);		
		setShowGrid(true);
	    setGridColor(Color.BLACK);
	    setSurrendersFocusOnKeystroke(true);	    
	    getTableHeader().setReorderingAllowed(true);
	    	    
	    // column model
	    this.setColumnModel(new DefaultTableColumnModel( ){	
			   public void moveColumn(int columnIndex, int newIndex){				   
				   if(columnIndex == 0 || newIndex== 0) {
					  return;
				   }
				   
				   String columnName = TranscriptionTable.this.getColumnName(currentColumnIndex);
				   super.moveColumn(columnIndex, newIndex);					   
				  
				   if(storeColumnOrder){
					   if(columnIndex == currentColumnIndex || newIndex == currentColumnIndex){
						   currentColumnIndex = TranscriptionTable.this.getColumnModel().getColumnIndex(columnName);
					   }
					   TranscriptionViewer viewer = (TranscriptionViewer) TranscriptionTable.this.getParent().getParent().getParent();
					   viewer.storeColumnOrder();
					   boolean playBack = viewer.isAutoPlayBack();
					   viewer.setAutoPlayBack(false);					   
					   startEdit(null);
					   viewer.setAutoPlayBack(playBack);
				   }	
			   }
			   
			   protected void fireColumnMarginChanged(){
				   if(!storeColumnOrder){
					   return;
				   }
				   super.fireColumnMarginChanged();			   
				   reCalculateRowHeight();	
//				   	if(TranscriptionTable.this.getParent() !=null){
//				   		TranscriptionViewer viewer = (TranscriptionViewer) TranscriptionTable.this.getParent().getParent().getParent();
//				   		boolean playBack = viewer.isAutoPlayBack();
//				   		viewer.setAutoPlayBack(false);				   		
//				   		//startEdit(null);
//				   		viewer.setAutoPlayBack(playBack);
//				   	}
				   	
			   }
		});
	    
	  
	    
	    
	    
		
		// remove the default behavior of TAB and ENTER
		Action doNothing = new AbstractAction() {
		    public void actionPerformed(ActionEvent e) { }
		};		
		getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), "doNothing");
		getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),	"doNothing");
		getActionMap().put("doNothing", doNothing);		
	}
	
	public void columnMarginChanged(ChangeEvent e) {
		
		TableColumn resizingColumn = this.getTableHeader().getResizingColumn();
		// Need to do this here, before the parent's
		// layout manager calls getPreferredSize().
		if (resizingColumn != null && autoResizeMode == AUTO_RESIZE_OFF) {
		    resizingColumn.setPreferredWidth(resizingColumn.getWidth());
		}
		resizeAndRepaint();
	    }

	
	/**
	 * Sets a flag whether the column order should be
	 * stored or not
	 * 
	 * @param store, if true, stores the column order
	 *                  else not
	 */
	public void setStoreColumnOrder(boolean store){
		storeColumnOrder = store;
	}
	
	/**
	 * Sets s flag, whether "enter" moves via column or not
	 * 
	 * @param selected, if true moves via column
	 * 		if false, moves by row in the current column
	 */
	public void moveViaColumn(boolean selected) {
		moveViaColumn = selected;			
	}
	
	/**
	 * Sets s flag, whether the editing cell should 
	 * always be in the center or not
	 * 
	 * @param selected, if true scrolls the current editing cell to the center of the table
	 * 		if false, has the default behavior of the table
	 */
	public void scrollActiveCellInCenter(boolean selected) {
		scrollToCenter = selected;
	}
	
	/**
	 * Activates the third column for the extra tier 
	 * 
	 * @param activated, if true adds a extra column
	 * 					for the new tier
	 */
	public void activateThirdColumn(boolean activated) {
		thirdColumnActivated = activated;			
	}
	
	/**
	 * Returns whether the third column
	 * is activated or not
	 * 
	 * @return true if activated else false
	 */
	public boolean isThirdColumnActive() {
		return thirdColumnActivated;	
	}	
	
	protected boolean processKeyBinding(KeyStroke ks, KeyEvent e, int condition, boolean pressed) {		
		int code = e.getKeyCode();		
		if(code == KeyEvent.VK_ENTER  && condition==0 && pressed){	
			if(!isEditing()){
				int selectedRow = getSelectedRow();
				int selectedColumn = getSelectedColumn();
				if(editCellAt(selectedRow,selectedColumn)){
					currentRowIndex = selectedRow;
					currentColumnIndex = selectedColumn;
				}else { 
					goToNextEditableCell();	
				}
			}	
			startEdit(e);
		}		
		return false;
	}
	
	/**
	 * Returns the current row in the table
	 * 
	 * @return currentRowIndex.
	 */
	public int getCurrentRow() {		
		return currentRowIndex;
	}
	
	/**
	 * Returns the current column in the table
	 * 
	 * @return currentColumnIndex.
	 */
	public int getCurrentColumn() {
		return currentColumnIndex;
	}
	
	/**
	 * Jumps to the editable cell Up
	 */
	public void goToEditableCellUp(){
		int rowBefore = currentRowIndex -1;
   		
   		before:   	
   			while(true){
   				if(rowBefore < 0){
   					rowBefore = 0;
   				}
   				
   				if(getValueAt(rowBefore,currentColumnIndex) instanceof TableSubHeaderObject){	
   					if(rowBefore == 0){
   						rowBefore = currentRowIndex;
   						break before;
   					} else {
   						rowBefore = rowBefore-1;   									
   					} 
   				}   	
   				
   				if (getValueAt(rowBefore,currentColumnIndex) == null ){
   					if(rowBefore == 0){
   						rowBefore = currentRowIndex;
   						break before;
   					} else {
						rowBefore = rowBefore-1;
					} 
   					continue before;
   				}   	
   					
   				break before;
   			}
		
		if(rowBefore != currentRowIndex){
			changeSelection(rowBefore, currentColumnIndex, false, false);	
			if(this.isEditing()){
				editingStopped(null);
			}
			startEdit(null);
		}
  	}
	
	/**
	 * Jumps to the editable cell down
	 */
	public void goToEditableCellDown(){
		int rowNext = currentRowIndex + 1;
   		
   		next:   	
   			while(true){
   				if(rowNext >= getRowCount()){
   					rowNext = currentRowIndex;
   				}
   				
   				if(getValueAt(rowNext,currentColumnIndex) instanceof TableSubHeaderObject){
   					if(rowNext >= getRowCount()){
   						rowNext = currentRowIndex;
   						break next;
   					} else {
   						rowNext = rowNext+1;
   					}
   				}   	
   				
   				if (getValueAt(rowNext,currentColumnIndex) == null ){
   					if(rowNext >= getRowCount()){
   						rowNext = currentRowIndex;
   						break next;
   					} else {
   						rowNext = rowNext+1;
   					}
   					continue next;
   				}   
   					
   				break next;
   			}
		
		if(rowNext != currentRowIndex){
			changeSelection(rowNext, currentColumnIndex, false, false);		
			if(this.isEditing()){
				editingStopped(null);
			}
			startEdit(null);
		}
  	}
	
	/**
	 * Jumps to the editable cell left
	 */
	public void goToEditableCellLeft(){
		int leftColumn = currentColumnIndex - 1;
		
		left :
			while(true){
				if(leftColumn == 0){
		   			leftColumn = currentColumnIndex;
		   			break left;
		   		}
				
				if (getValueAt(currentRowIndex,leftColumn) == null ){
					leftColumn = leftColumn - 1;
					continue left;
		   		}
				
				break left;
			} 
   		
		if(leftColumn != currentColumnIndex){
			changeSelection(currentRowIndex, leftColumn, false, false);	
			if(this.isEditing()){
				editingStopped(null);
			}
			startEdit(null);
		}
  	}
	
	/**
	 * Jumps to the editable cell right
	 */
	public void goToEditableCellRight(){
		int rightColumn = currentColumnIndex + 1;
		
		if(rightColumn < getColumnCount()){
			if(this.getColumnModel().getColumn(rightColumn).getMaxWidth()==0){
				rightColumn = currentColumnIndex;
			}
		} else {
			rightColumn = currentColumnIndex;
		}
		
		right :
			while(true){
				if(rightColumn < getColumnCount()){
					if(this.getColumnModel().getColumn(rightColumn).getMaxWidth()==0){
						rightColumn = currentColumnIndex;
						break right;
					}
				} else {
					rightColumn = currentColumnIndex;
					break right;
				}
				
				if (getValueAt(currentRowIndex,rightColumn) == null ){
					rightColumn = rightColumn + 1;
					continue right;
		   		}
				
				break right;
			} 
   		
		if(rightColumn != currentColumnIndex){
			changeSelection(currentRowIndex, rightColumn, false, false);	
			if(this.isEditing()){
				editingStopped(null);
			}
			startEdit(null);
		}			
  	}
	
	/**
	 * Jumps to the next editable cell
	 */
	public void goToNextEditableCell(){
		int nextRow = currentRowIndex;
   		int nextColumn = currentColumnIndex;  
   		
   		if(moveViaColumn){
   			next:   	
   				while(true){
   					if(nextColumn == (getColumnCount()-1)){
   						if(nextRow == (getRowCount()-1)){
   							nextRow = 0;
   						}else {
   							nextRow = nextRow+1;
   						}
   						nextColumn = 1;
   					} else {
   						nextColumn = nextColumn+1;
   					}
   		
   					if (!thirdColumnActivated && nextColumn == (getColumnCount()-1) ){
   						if(nextRow == (getRowCount()-1)){
   							nextRow = 0;
   						}else {
   							nextRow = nextRow+1;
   						}
   						nextColumn = 1;
   					}
   					
   					if(nextRow < 0){
   						nextRow = nextRow +1;
   					}
   					
   					if(getValueAt(nextRow,nextColumn) instanceof TableSubHeaderObject){	
   						nextRow = nextRow+1;   						
   					} 
   					
   					if (getValueAt(nextRow,nextColumn) == null ){
   						continue next;
   					}	   	
   					break next;
   				}
   			changeSelection(nextRow, nextColumn, false, false);
  		} else {
  			next:
   				while(true){
   					if(nextRow == (getRowCount()-1)){
   						nextRow = 0;
   					} else {
   						nextRow = nextRow+1;
   					}
   					
   					if(nextColumn < 0){
   						nextColumn = nextColumn+2;
   					}
	   		
   					if(getValueAt(nextRow,nextColumn) instanceof TableSubHeaderObject){	
   						nextRow = nextRow+1;   						
   					}
   					
   					if (getValueAt(nextRow,nextColumn) == null ){
   						continue next;
   					}
   					break next;
   				}
   			changeSelection(nextRow, nextColumn, false, false);
  		}   		
   		
  	}
	
	/**
	 * Scrolls the current editing row to
	 * the center of the table if needed
	 * 
	 */
	private void scrollIfNeeded(){		
		if(!scrollToCenter){
			return;
		}
		
		JViewport viewport = (JViewport)getParent();

		// This rectangle is relative to the table where the
		// northwest corner of cell (0,0) is always (0,0).
		Rectangle rect = getCellRect(currentRowIndex, currentColumnIndex, true);		   
		Rectangle viewRect = viewport.getViewRect();

		// Translate the cell location so that it is relative
		// to the view, assuming the northwest corner of the
		// view is (0,0).
		rect.setLocation(rect.x-viewRect.x, rect.y-viewRect.y);

		// Calculate location of rect if it were at the center of view
		int centerX = (viewRect.width-rect.width)/2;
		int centerY = (viewRect.height-rect.height)/2;

		// Fake the location of the cell so that scrollRectToVisible
		// will move the cell to the center
		if (rect.x < centerX) {
			centerX = -centerX;
		}
		
		if (rect.y < centerY) {
			centerY = -centerY;
		}
		rect.translate(centerX, centerY);

		// Scroll the area into view.
		viewport.scrollRectToVisible(rect);
	}   		
	
	/**
	 * Starts editing  the current cell
	 * 
	 * @param e
	 */
	public void startEdit(KeyEvent e){			
		if (!isEditing()) {
			if (!editCellAt(currentRowIndex, currentColumnIndex, e)) {
				return ;
			}
		}	
		Component editorComponent = getEditorComponent();
				
	    if (editorComponent instanceof JComponent) {
	    	if (getSurrendersFocusOnKeystroke()) {
	    		editorComponent.requestFocus();
            }
	    }
	}
    
	/**
	 * Deletes all the rows in the table
	 */
    public void clearRows(){	
    	if(isEditing()){
    		TranscriptionTableCellEditor editor = (TranscriptionTableCellEditor)getCellEditor(currentRowIndex, currentColumnIndex);
    		editor.commitChanges(true);
    	}
    	while(getRowCount() > 0){
    		((TranscriptionTableModel)getModel()).removeRow(0);
    	}    	
    	revalidate();
    	currentRowIndex = 0; 
    	currentColumnIndex = 1;   
    }    
    
    public void changeSelection(int row,  int column, boolean  toggle, boolean extend) {    	    	
    	if(column == 0){
    		startEdit(null);    		
	    	return;  
	    }     	
    	currentRowIndex = row; 
    	currentColumnIndex = column; 
    
    	scrollIfNeeded();
        super.changeSelection(row, column, toggle, extend);	      
    }

   	public void editingStopped(ChangeEvent e) {	   	
   		if((TranscriptionTableCellEditor)this.getCellEditor() != null){
   			((TranscriptionTableCellEditor)this.getCellEditor()).commitChanges(false);
   		}
		super.editingStopped(e);		
		recalculateRowHeightAt(currentRowIndex);	
	}

	public void editingCanceled(ChangeEvent e) {
		super.editingCanceled(e);
	}
	
	/**
	 * Set the font color of the tiers 
	 * 
	 * @param fontColors
	 */
	public void setFontColorForTiers(HashMap fontColors) {
		//prefTierFontColors.clear();
		if (fontColors != null) {
			prefTierFontColors.putAll(fontColors);
			
			Object key;
			Object val;
			Iterator keyIt = fontColors.keySet().iterator();
			while (keyIt.hasNext()) {
				key = keyIt.next();
				val = fontColors.get(key);
				if (val instanceof Color) {
					prefTierBrighterFontColors.put((String) key, brighter((Color) val));
					prefTierBrightestFontColors.put((String) key, brightest((Color) val));
				}
			}
		}
		
	}
	
	/**
	 * Calculates a brighter version of the color, comparable to alpha of 40.
	 * 
	 * @param orig
	 * @return brightest color
	 */
	private Color brightest(Color orig) {
		//return new Color(orig.getRed(), orig.getGreen(), orig.getBlue(), 40);
        double FACTOR = 0.16;

        return new Color((int) (255 - ((255 - orig.getRed()) * FACTOR)),
        		(int) (255 - ((255 - orig.getGreen()) * FACTOR)),
        		(int) (255 - ((255 - orig.getBlue()) * FACTOR)));
	}
	
	/**
	 * Calculates a brighter version of the color, comparable to alpha of 70.
	 * 
	 * @param orig
	 * @return brightest color
	 */
	private Color brighter(Color orig) {
		//return new Color(orig.getRed(), orig.getGreen(), orig.getBlue(), 70);
        double FACTOR = 0.3;

        return new Color((int) (255 - ((255 - orig.getRed()) * FACTOR)),
        		(int) (255 - ((255 - orig.getGreen()) * FACTOR)),
        		(int) (255 - ((255 - orig.getBlue()) * FACTOR)));
	}
	
	/**
	 * Returns the font color of the tier
	 * 
	 * @param tierName, tier for which the font color is returned
	 * @return font color 
	 */
	public Color getFontColorForTier(String tierName) {
		return (Color) prefTierFontColors.get(tierName);
	}
	
	/**
	 * Returns the brighter font color of the tier
	 * 
	 * @param tierName, tier for which the brighter font color is returned
	 * @return brighter font color 
	 */
	public Color getBrighterFontColorForTier(String tierName) {
		return (Color) prefTierBrighterFontColors.get(tierName);
	}
	
	
	/**
	 * Returns the brightest version of font color of the tier
	 * 
	 * @param tierName, tier for which the brightest font color is returned
	 * @return brightest font color 
	 */
	public Color getBrightestFontColorForTier(String tierName) {
		return (Color) prefTierBrightestFontColors.get(tierName);
	}

	public HashMap getFontColorTierMap() {
		return prefTierFontColors;
	}

	/**
	 * Sets the font of the tiers
	 * 
	 * @param fontNames
	 */
	public void setFontsForTiers(HashMap fontNames) {
		prefTierFonts.clear();
		if (fontNames != null) {
			Iterator keyIt = fontNames.keySet().iterator();
			String key;
			String fn;
			while (keyIt.hasNext()) {
				key = (String) keyIt.next();
				fn = ((Font) fontNames.get(key)).getName();
				if (fn != null) {
					prefTierFonts.put(key, new Font(fn, Font.PLAIN, getFontSize()));
				} 
			}
		}	
	}
	
	/**
	 * Returns the font size of the font
	 * 
	 * @return
	 */
	public int getFontSize() {
		return getFont().getSize();
	}
	
	/**
	 * Returns the font used for the tier
	 * 
	 * @param tierName, tier for which the font is returned
	 * @return font
	 */
	public Font getFontForTier(String tierName) {
		return (Font) prefTierFonts.get(tierName);
	}

	/**
	 * Recalculates the height of the given row	 
	 * 
	 * @param row, rowIndex of the row which is to be
	 * 			   recalculated
	 */
	private void recalculateRowHeightAt(int row){
		int fontHeight = 0;
		int rowHeightValue = 0;
		FontMetrics fm = null;
	 	Graphics g = getGraphics();
	 	 
		for(int i = 1; i< getColumnCount(); i++){
			 String value = null;					 
			 Object c  = getValueAt(row, i);
			 if(c instanceof Annotation){
				 Annotation ann = (Annotation)c;
				 value = ann.getValue();	
				 if(value != null) {
					 Font f = getFontForTier(ann.getTier().getName());
					 if(f != null){
				 		f = new Font(f.getFontName(), f.getStyle(), getFontSize());
				 		fm = getFontMetrics(f);
				 	 } else {		
				 		f = new Font(getFont().getFontName(), getFont().getStyle(), getFontSize());
				 		fm = getFontMetrics(f);
				 	 }
			 		 int height = fm.getAscent()+ fm.getDescent() + 3;
		 			 if(height > fontHeight){
		 				 fontHeight = height;
		 			 }
					 Rectangle2D r = fm.getStringBounds(value, g);
					 Rectangle rt = r.getBounds();
					 double width = rt.getWidth() + 20; 
					 int coulumnWidth = getColumnModel().getColumn(i).getWidth();
												 
					 int newRowHeight = 0;
					 if(width > coulumnWidth){      
						 int y = (int)(width /coulumnWidth);						 				 
						 newRowHeight = (1 + y) * fontHeight;						 
					 } else {
						 newRowHeight = fontHeight;
					 }
					 
					 if(newRowHeight > rowHeightValue){
						 rowHeightValue = newRowHeight;
					 }					 
					 
					 if(rowHeightValue > rowHeight){
						 setRowHeight(row, rowHeightValue+10); 	
					 }else {
						 setRowHeight(row, rowHeight+10);  
					 }
				 }
			 }
		}
		revalidate();
	}
	
	/**
	 * Recalculates the row height for all the rows,
	 * according to the content of each row
	 */
	public void reCalculateRowHeight(){		
 
		 FontMetrics fm = null;
	 	 Graphics g = getGraphics();
		 
		 for(int x =0 ; x < getRowCount(); x++){	
			 int fontHeight = 0;
			 int rowHeightValue = 0;
			 for(int i = 1; i< getColumnCount(); i++){
				 String value = null;					 
				 Object c  = getValueAt(x, i);
				 if(c instanceof Annotation){
					 Annotation ann = (Annotation)c;
					 value = ann.getValue();	
					 if(value != null) {
						 Font f = getFontForTier(ann.getTier().getName());						 
				 		 if(f != null){
				 			f = new Font(f.getFontName(), f.getStyle(), getFontSize());
				 			fm = getFontMetrics(f);
				 		 } else {		
				 			f = new Font(getFont().getFontName(), getFont().getStyle(), getFontSize());
				 			fm = getFontMetrics(f);
				 		 }
				 		 int height = fm.getAscent()+ fm.getDescent() + 3;
			 			 if(height > fontHeight){
			 				 fontHeight = height;
			 			 }
						 Rectangle2D r = fm.getStringBounds(value, g);
						 Rectangle rt = r.getBounds();
						 double width = rt.getWidth() + 20; 
						 int coulumnWidth = getColumnModel().getColumn(i).getWidth();													 
						 int newRowHeight = 0;
						 if(width > coulumnWidth){      
							 int y = (int)(width /coulumnWidth);										 
							 newRowHeight = (1 + y) * fontHeight;							 
						 } else {
							 newRowHeight = fontHeight;
						 }
						 
						 if(newRowHeight > rowHeightValue){
							 rowHeightValue = newRowHeight;
						 }		
						 if(rowHeightValue > rowHeight){
							 setRowHeight(x, rowHeightValue+10); 	
						 }else {
							 setRowHeight(x, rowHeight+10);  
						 }
				 	 }
				 } else if(c instanceof TableSubHeaderObject){
					 fm = getFontMetrics(getFont());
					 int height = fm.getAscent()+ fm.getDescent() + 3;
			 		 if(height > rowHeight){
			 			setRowHeight(x, height); 
			 		 }else {
			 			setRowHeight(x, rowHeight); 
			 		 }
				} else {
					if(rowHeightValue > rowHeight){
						 setRowHeight(x, rowHeightValue+10); 	
					 }else {
						 setRowHeight(x, rowHeight+10);  
					 }
				}
			 }
		 } 
		 revalidate();
	}
}
