package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.util.HashMap;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.util.TableSubHeaderObject;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;

/**
 * Cell Editor for the transcription table
 * 
 * @author Aarthy Somasundaram
 *
 */
public class TranscriptionTableCellEditor extends DefaultCellEditor {	
	
	private static final String EMPTY = "";
	private Annotation annotation;
	private TranscriptionTableEditBox inlineEditBox;
	private TranscriptionViewer viewer;
	private int startEditInOneClick = 1;
	
	/**
	 * Creates an instance of TranscriptionTableCellEditor
	 * 
	 * @param viewer
	 * @param dataModel
	 */
	public TranscriptionTableCellEditor(TranscriptionViewer viewer) {
		super(new JTextField());
		getComponent().setEnabled(false);
		this.viewer =viewer;
		setClickCountToStart(startEditInOneClick);
	}

	public Component getTableCellEditorComponent(
		JTable table,
		Object value,
		boolean isSelected,
		int row,
		int column) {	
		annotation = null;
		if (inlineEditBox == null) {
			inlineEditBox = new TranscriptionTableEditBox(viewer, (TranscriptionTable)table);
		}
		
		if (value instanceof Annotation) {
			annotation = (Annotation) value;
			configureEditBox(table, row, column);
			viewer.updateMedia(annotation.getBeginTimeBoundary(),annotation.getEndTimeBoundary());
			if(viewer.isAutoPlayBack()){
				viewer.playInterval(annotation.getBeginTimeBoundary(),annotation.getEndTimeBoundary(),false);
			}
			inlineEditBox.startEdit();
			return inlineEditBox.getEditorComponent();
		} else if (value instanceof String && value.toString().contains(TranscriptionViewer.CREATE_ANN)) {
			try {				
				annotation = createAnnotation( table, row, column);
				if (annotation != null) {					
					table.setValueAt(annotation, row, column);
					configureEditBox(table, row, column);
					viewer.updateMedia(annotation.getBeginTimeBoundary(),annotation.getEndTimeBoundary());
					if(viewer.isAutoPlayBack()){
						viewer.playInterval(annotation.getBeginTimeBoundary(),annotation.getEndTimeBoundary(),false);
					}					
					
//					SwingUtilities.invokeLater(new Runnable() {
//						public void run() {
//							if (inlineEditBox != null) {
//								inlineEditBox.startEdit();
//							}
//						}
//					});
					inlineEditBox.startEdit();
					return inlineEditBox.getEditorComponent();
				}
			}
			catch (Exception ex) {
				ex.printStackTrace();
				//LOG.warning(LogUtil.formatStackTrace(ex));
				return getComponent();
			}
		}
			
		return getComponent();
	}
	
	 public TranscriptionTableEditBox getEditorComponent(){
	       if (inlineEditBox != null) {
	            return inlineEditBox;
	        }
	        return null;
	 }
	
	/**
	 * Creates a new annotation on the current selected cell
	 * 
	 * @param table 
	 * @param row
	 * @param column
	 * 
	 * @return newAnnotation, the new created annotation, can be null
	 */
	private Annotation createAnnotation(JTable table, int row, int column){		
		String columnName = table.getColumnName(column);	
		AbstractAnnotation ann = null;
		Annotation newAnnotation = null;
		TierImpl currentTier = null;
		long beginTime = 0L;
		long endTime = 0L;
		
		TranscriptionTableModel tableModel = (TranscriptionTableModel) table.getModel();
		
		String column1Name = tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX);
		String column2Name = tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX);
		String column3Name = tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX);
		
		//if the current column is not the column1
		if(!columnName.equals(column1Name)){
			// Check whether column1 has a annotation or if its also empty
			// if it has a annotation, get the time info from that annotation
			// else check for annotation on the other column
			Object val =  table.getValueAt(row, table.getColumnModel().getColumnIndex(column1Name));
			if(val instanceof Annotation){
				 ann = (AbstractAnnotation)val;
			} else {
				if(!columnName.equals(column2Name) && table.getColumn(column2Name).getMaxWidth() > 0){
					val =  table.getValueAt(row, table.getColumnModel().getColumnIndex(column2Name));
					if(val instanceof Annotation){
						 ann = (AbstractAnnotation)val;
					}
				} else if(!columnName.equals(column3Name) && table.getColumn(column3Name).getMaxWidth() > 0){
					val =  table.getValueAt(row, table.getColumnModel().getColumnIndex(column3Name));
					if(val instanceof Annotation){
						 ann = (AbstractAnnotation)val;
					}
				}
			}	
		} else {
			// if its current column is column1, check for annotations in other columns
			if(table.getColumn(column2Name).getMaxWidth() > 0){
				Object val =  table.getValueAt(row, table.getColumnModel().getColumnIndex(column2Name));
				if(val instanceof Annotation){
					 ann = (AbstractAnnotation)val;
				} else {
					if(table.getColumn(column3Name).getMaxWidth() > 0){
						val =  table.getValueAt(row, table.getColumnModel().getColumnIndex(column3Name));
						if(val instanceof Annotation){
							ann = (AbstractAnnotation)val;
						}
					}
				}
			}
		} 
		
		// if there is no annotation for reference, get the time info from the
		// string value of the current cell
			
		if(ann == null){				
			// structure of value is  " CREATE_ANN;tierName;beginTime : endTime";
			String value = (String) table.getValueAt(row, column);
			//int index = value.indexOf(';');
			String s[] = value.split(";");
			//String timeValue = value.substring(index+1);
			String timeValue[] = s[2].split(":");
			if(timeValue != null){
				for(int i=0; i < timeValue.length; i++){
					String time =timeValue[i];
					try {
						long t = Long.parseLong(time.trim());
						if(i==0){
							beginTime = t;
						} else if(i==1){
							endTime = t;
						}
					} catch (NumberFormatException e) {
						System.out.println(e);
					}
				}
				
				if(viewer.isTierNamesShown()){
					for(int i = row-1; i>=0; i--){
						Object obj = table.getValueAt(i, column);
						if(obj instanceof TableSubHeaderObject){
							String name = obj.toString();
							currentTier = (TierImpl) (viewer.getViewerManager().getTranscription()).getTierWithId(name);
						}					
					}
				} else {
					String tierName = s[1];
					currentTier = (TierImpl) (viewer.getViewerManager().getTranscription()).getTierWithId(tierName);					
				}
			}
		} else {
			beginTime = ann.getBeginTimeBoundary();
			endTime = ann.getEndTimeBoundary();
			TierImpl linkedTier = (TierImpl) ann.getTier();			
			if(linkedTier.getLinguisticType().getConstraints() != null &&
					linkedTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){				
				TierImpl parentTier = (TierImpl) linkedTier.getParentTier();
				while(parentTier.getLinguisticType().getConstraints() != null && 
						parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
					parentTier = (TierImpl) parentTier.getParentTier();
				}
				currentTier = viewer.getTierMap(columnName).get(parentTier);
			}else {
				currentTier =  viewer.getTierMap(columnName).get(linkedTier);
			}
		}
			
		if(currentTier != null){
			if(currentTier.isTimeAlignable()){
				newAnnotation =  currentTier.createAnnotation(beginTime, endTime);
			} else {				
				long time =	(beginTime + endTime) / 2;
				newAnnotation = ((TierImpl) currentTier).createAnnotation(time, time);
			}
		}
		
		return newAnnotation;
	}

	/**
	 * Configures the edit box, for the active cell
	 * in the table
	 * 
	 * @param table
	 * @param row
	 * @param column
	 */
	private void configureEditBox(JTable table, int row, int column) {			
		inlineEditBox.setAnnotation(annotation);
		//table.setRowHeight(row, (int) (1.5 * table.getRowHeight(row)));
		table.setRowHeight(row, (int) (table.getRowHeight(row)));
		Font ff = null;
		if (table instanceof TranscriptionTable) {
			ff = ((TranscriptionTable) table).getFontForTier( ((Annotation)table.getValueAt(row, column)).getTier().getName() );
			if(ff == null){
				ff = table.getFont();
			}
		}
		
		if (inlineEditBox.isUsingControlledVocabulary()) {
			inlineEditBox.configureEditor(
				JComboBox.class, ff,
				table.getCellRect(row, column, true).getSize());
		} else {
			int i = table.getRowHeight(row);
			inlineEditBox.configureEditor(
				JScrollPane.class,	ff,
				table.getCellRect(row, column, true).getSize());
		}
	}
	
	public Object getCellEditorValue() {
		if (annotation != null) {
			return annotation;
		}
		else {
			return EMPTY;
		}
	}
	
	public void showPopUp(Component comp, int x, int y) {
		if (inlineEditBox != null)
			inlineEditBox.showPopUp(comp, x, y);
	}

	public void updateLocale() {
		if (inlineEditBox != null)
			inlineEditBox.updateLocale();
	}
	
	public void commitChanges(boolean bool) {
		if (inlineEditBox != null)
			inlineEditBox.commitChanges(bool);
	}

	public void cancelCellEditing() {
		//super.cancelCellEditing();
		if (inlineEditBox != null) {
			inlineEditBox.cancelEdit();
		}
	}
}
