package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.DefaultTableCellRenderer;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.util.TableSubHeaderObject;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;

/**
 * Cell Renderer for the transcription table
 * 
 * @author aarsom
 */
public class TranscriptionTableCellRenderer extends DefaultTableCellRenderer {
	
	private JTextArea area;
	private boolean showTierNames = true;
	private boolean colorOnlyOnNoColumn = false;
	private Transcription transcription;
	private final Color NO_ANN_BG = new Color(230,230,250);
	private final JLabel EMPTY_LABEL = new JLabel();
	private final Color DEF_LABEL_BG;
	private final Color DEF_AREA_BG;
	
	public TranscriptionTableCellRenderer(Transcription transcription){
		this.transcription = transcription;
		EMPTY_LABEL.setOpaque(true);
		DEF_LABEL_BG = new Color (EMPTY_LABEL.getBackground().getRed(), EMPTY_LABEL.getBackground().getGreen(), 
				EMPTY_LABEL.getBackground().getBlue());
		area = new JTextArea();
		DEF_AREA_BG = area.getBackground();
    	area.setLineWrap(true);
    	area.setMargin(new Insets(0,3,0,3));
    	area.setAlignmentX(CENTER);
    	area.setAlignmentY(CENTER);
	}
	
	public Component getTableCellRendererComponent(JTable table,
       Object value, boolean isSelected, boolean hasFocus, int row,
       int column)
    {   			
    	// for the No column
    	if(table.getColumnName(column).equals(ElanLocale.getString(ELANCommandFactory.TRANS_TABLE_CLM_NO))){
    		//JLabel label = new JLabel(); //HS
    		// JLabel label = this; //HS
    		setForeground(Color.BLACK);
        	/*label.*/setFont(table.getFont().deriveFont(Font.PLAIN, table.getFont().getSize()));
        	/*label.*/setText(value.toString());
        	/*label.*/setHorizontalAlignment(CENTER);
        	/*label.*/setVerticalAlignment(TOP);        	     	
        	/*label.*/setOpaque(true); 
        	if(table.getSelectedRow() == row){        			
        		/*label.*/setBackground(Color.LIGHT_GRAY);
        	} else {
        		/*label.*/setBackground(DEF_LABEL_BG);
        	}
        	
        	if(!showTierNames && colorOnlyOnNoColumn){
        		Color  c = null;
        	    column = column +1;
        		value = table.getValueAt(row, column);
        		TierImpl parentTier = null;        		
        		while (true){
        			if (value instanceof Annotation) {
        				TierImpl tier = (TierImpl) ((Annotation)value).getTier();        				
        				if(tier.getLinguisticType().getConstraints() == null || tier.getLinguisticType().getConstraints().getStereoType() != Constraint.SYMBOLIC_ASSOCIATION){
        					parentTier = tier;
        				} else {
        					parentTier = (TierImpl) tier.getParentTier();
        					while( parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
        						parentTier = (TierImpl) parentTier.getParentTier();
        					}
        				}
        				//label.setToolTipText(getRenderedText(value));
        				break;
        			} else if (value == null || value instanceof String) {
        				if(value instanceof String){
        					// structure of value is  " CREATE_ANN;tierName;beginTime : endTime";
        					String values[] = ((String) value).split(";");
        					if(values.length >= 2 ){
        						String tierName = values[1];
        						TierImpl tier = (TierImpl) transcription.getTierWithId(tierName.trim());        						
        						parentTier = (TierImpl) tier.getParentTier();
            					while( parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
            						parentTier = (TierImpl) parentTier.getParentTier();
            					}
        					}
        						
        				}
        			
        				column++;
        				if(column == 4){
        					break;
        				}
        				if(table.getColumnModel().getColumn(column).getMaxWidth() > 0){
        					value = table.getValueAt(row, column);
        					continue;
        				} else {
        					break;
        				}
        			}
        		} 
        		if(parentTier != null){
        			c = (Color)((TranscriptionTable)table).getFontColorForTier(parentTier.getName()); 
        			/*label.*///setBackground(new Color(c.getRed(), c.getGreen(), c.getBlue(), 40));
        			setBackground((Color)((TranscriptionTable)table).getBrightestFontColorForTier(parentTier.getName()));
        			if(parentTier.getParticipant() != null){
        				/*label.*/setToolTipText("participant : " + parentTier.getParticipant());
        			}
        		}
        		
        		if(table.getSelectedRow() == row){  
        			if(c != null){
        				/*label.*/ //setBackground(new Color(c.getRed(), c.getGreen(), c.getBlue(), 70));
        				setBackground((Color)((TranscriptionTable)table).getBrighterFontColorForTier(parentTier.getName()));
        			} 
        		}
        	}
        	//return label;
        	return this;
    	}
    	
    	// for the tierName
    	if (value instanceof TableSubHeaderObject) {
    		//JLabel label = new JLabel(); //HS
    		//JLabel label = this; //HS
    		/*label.*/setFont(table.getFont().deriveFont(Font.PLAIN, table.getFont().getSize() + 2));
    		/*label.*/setText(value.toString());
    		setHorizontalAlignment(LEFT);
    		setVerticalAlignment(EMPTY_LABEL.getVerticalAlignment());
    		/*label.*/setOpaque(true);
    		if(value.toString().trim().length() > 0){
    			Color  c = (Color)((TranscriptionTable)table).getFontColorForTier(value.toString());
    			/*label.*/setForeground(c);
    			/*label.*/// setBackground(new Color(c.getRed(), c.getGreen(), c.getBlue(), 40));
    			setBackground((Color)((TranscriptionTable)table).getBrightestFontColorForTier(value.toString()));
    		}
            //return label;
    		return this;
        } 
    	
    	// for the empty cell when there is no child tier available, 
    	if(value == null){
    		// JLabel label = new JLabel(); //HS    		
    		// label.setOpaque(true); //HS    		
    		// return label; //HS
    		return EMPTY_LABEL;
    	}
    	
    	// else
    	// area = new JTextArea();  //HS  	
    	area.setText(getRenderedText(value));    	    	
    	area.setToolTipText(getToolTipText(value));  
//     	area.setLineWrap(true); //HS
//    	area.setMargin(new Insets(0,3,0,3)); //HS
//    	area.setAlignmentX(CENTER); //HS
//    	area.setAlignmentY(CENTER); //HS
    	area.setBackground(DEF_AREA_BG);//HS
    	
    	if (value instanceof Annotation) {       		
    		Font f = ((TranscriptionTable)table).getFontForTier(((Annotation)value).getTier().getName());    		
    		if(f != null){
    			area.setFont(new Font(f.getFontName(), f.getStyle(), table.getFont().getSize()));
    		}else{
    			area.setFont(table.getFont());
    		} 
    		
    		if(!showTierNames && !colorOnlyOnNoColumn){
    			Color  c = (Color)((TranscriptionTable)table).getFontColorForTier(((Annotation)value).getTier().getName());    	
    			//area.setBackground(new Color(c.getRed(), c.getGreen(), c.getBlue(), 40));
    			area.setBackground((Color)((TranscriptionTable)table).getBrightestFontColorForTier(((Annotation)value).getTier().getName()));
    		}
    	} else if (value instanceof String) { 
    		if(showTierNames ){
    			//area.setBackground(new Color(230,230,250));
    			area.setBackground(NO_ANN_BG);
    		}else {
    			if(colorOnlyOnNoColumn){   
    				//area.setBackground(new Color(230,230,250));
    				area.setBackground(NO_ANN_BG);
    			} 
    		}
    	} 
        return area;
    }
    
    public void setShowTierNames(boolean bool){
    	showTierNames = bool;
    }
    
    public void showColorOnlyOnNoColumn(boolean bool){
    	colorOnlyOnNoColumn = bool;
    }
    
    private String getRenderedText(Object value) {
        return ((value instanceof Annotation)
        ? ((Annotation) value).getValue()
        : ((value instanceof String) ? "" : null));
    }
    
    private String getToolTipText(Object value) {   
    	String toolTipText = null;
    	if(value instanceof Annotation){    
    		if(!showTierNames){
    			String participant = ((TierImpl)((Annotation) value).getTier()).getParticipant();
    			toolTipText = ((Annotation) value).getTier().getName() ;    		
    			if(participant !=null){
    				toolTipText += " : " + participant;
    			}
    		}else {
    			toolTipText = ((Annotation) value).getBeginTimeBoundary()+" - " + ((Annotation) value).getEndTimeBoundary();
    		}
    	} else if (value instanceof String){
    		// structure of value is  " CREATE_ANN;tierName;beginTime : endTime";
    		if(!showTierNames){
    			String values[] = ((String) value).split(";");
    			if(values.length >= 2){
    				TierImpl tier = (TierImpl) transcription.getTierWithId(values[1]);
    				String participant = tier.getParticipant();
    				toolTipText = tier.getName() ;    		
    				if(participant !=null){
    					toolTipText += " : " + participant;
    				}
    			}    			
    		} else {
    			long beginTime = -1;
    			long endTime = -1;
    			String values[] = ((String) value).split(";");
    			if(values.length >= 3){
    				String timeValue[] = values[2].split(":");    			
    				if(timeValue != null){
    					for(int i=0; i < timeValue.length; i++){
    						String time =timeValue[i];
    						try {
    							long t = Long.parseLong(time.trim());
    							if(i==0){
    								beginTime = t;
    							} else if(i==1){
    								endTime = t;
    							}
    						} catch (NumberFormatException e) {
    							System.out.println(e);
    						}
    					}
    				}
    				if(beginTime > -1 && endTime > -1){
    					toolTipText = beginTime+" - " + endTime;
    				}
    			}
    		}
    	}else {
    		return null;
    	}
      return toolTipText;
    }
 }    

