package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.util.Locale;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.ElanLocaleListener;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.im.ImUtil;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.ExternalReference;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.ExternalReferenceGroup;
import mpi.eudico.server.corpora.clomimpl.abstr.ExternalReferenceImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.util.CVEntry;
import mpi.eudico.util.ControlledVocabulary;
import mpi.eudico.util.ExternalCV;
import mpi.eudico.util.ExternalCVEntry;
import mpi.util.LogUtil;


/** A class that provides and configures components for  editing
 * annotation values, specially made for the transcription table 
 * in Transcription mode.
 *  
 * @author aarsom
 *
 */
public class TranscriptionTableEditBox extends JPanel implements ActionListener,
    MouseListener, KeyListener, ElanLocaleListener {

	/** action command constant */
    private static final String EDIT_MENU_CMT = "Commit Changes";

    /** action command constant */
    private static final String EDIT_MENU_CNL = "Cancel Changes";
    
    /** action command constant */
    private static final String EDIT_IN_ANN_MODE = "Anno mode";

    /** A logger to replace System.out calls. */
    private static final Logger LOG = Logger.getLogger(TranscriptionTableEditBox.class.getName());

    final private JTextArea textArea = new JTextArea("", 2, 1);
    
    final private JScrollPane textAreaScrollPane = new JScrollPane(textArea);
    
    private JPopupMenu popupMenu = new JPopupMenu("Select Language");
    private Locale[] allLocales;
    private int numberOfLocales;
    private String oldText;
    private Annotation annotation;
    private TranscriptionTable table = null;
    private TranscriptionViewer viewer;
    private Locale annotationLocale;
    private boolean isUsingControlledVocabulary = false;
    private JMenuItem commitMI;
    private JMenuItem cancelMI;
    private JMenuItem mergeBeforeMI;
    private JMenuItem mergeNextMI;
    private JMenuItem deleteAnnotationMI;
    private JMenuItem editInAnnModeMI;
	private JMenuItem selectAllMI;
    private JMenuItem cutMI;
    private JMenuItem copyMI;
    private JMenuItem pasteMI;
    
    /** a JList in a scrollpane */    
    private CVEntryComponent cvEntryComp;
    
    private int caretPosition = -1;
   
    /**
     * this field can be either a JScrollPane(with JTextArea)
     *  or a JComboBox 
     */
    private JComponent editorComponent;
    //temp
    private Font uniFont = Constants.DEFAULTFONT;
    
    private boolean isEditing = false;
  
    /**
     * a focus listener that lets the textarea request the
     * keyboard focus
     */
    final private FocusListener intFocusListener = new FocusAdapter() {
    	public void focusGained(FocusEvent e) {
    		if (!isUsingControlledVocabulary) {
    			textArea.requestFocus();
                textArea.getCaret().setVisible(true);
            } else {
                if (cvEntryComp != null) {
                   cvEntryComp.grabFocus();
                 }
            }
        }
            
        public void focusLost(FocusEvent e) {            	
        	if (!isEditing) {
				transferFocusUpCycle();
			}
        }
    };

    /**
     * Creates a instance of TranscriptionTableEditBox
     * 
     * @param viewer
     * @param table
     */
    public TranscriptionTableEditBox(TranscriptionViewer viewer, TranscriptionTable table) {
        this.table = table;
        this.viewer = viewer;
        init();
    }
    
    /**
     * Initializes the components  
     */
    public void init() {    	  
        setLayout(new BorderLayout());

        try {
            allLocales = ImUtil.getLanguages();
            numberOfLocales = (allLocales == null) ? 0 : allLocales.length;
        } catch (java.lang.NoSuchMethodError nsme) {
            // The SPI extensions have not been present at startup.
            //String msg = "Setup incomplete: you won't be able to set languages for editing.";
            String msg = ElanLocale.getString("InlineEditBox.Message.SPI") +
                "\n" + ElanLocale.getString("InlineEditBox.Message.SPI2");
            JOptionPane.showMessageDialog(null, msg, null,
                JOptionPane.ERROR_MESSAGE);
        } catch (Exception exc) {
            LOG.warning("InlineEditBox::init::ParentIMBug::FIXME");
            LOG.warning(LogUtil.formatStackTrace(exc));
        }

        textArea.addMouseListener(this);
        textArea.setLineWrap(true);
        textArea.setMargin(new Insets(0,3,0,3));
        textArea.setWrapStyleWord(true);
        textAreaScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        textAreaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        add(textAreaScrollPane, BorderLayout.CENTER);
        textArea.getCaret().setVisible(true);        
        textArea.addKeyListener(this);
        textArea.addFocusListener(new FocusAdapter() {
        	public void focusGained(FocusEvent e) {
               	if (annotationLocale != null && !annotationLocale.equals(Locale.getDefault()) && 
               			!annotationLocale.equals(textArea.getLocale())) {
               		ImUtil.setLanguage(textArea, annotationLocale);
               		textArea.setFont(uniFont);
               	}
            }
			
        	public void focusLost(FocusEvent e) {
				if (!isEditing) {						
					transferFocusUpCycle();
				}					
			}
        });

        createPopupMenu();       
        textAreaScrollPane.addFocusListener(intFocusListener);
        addFocusListener(intFocusListener);
        
		// remove the default behavior of TAB 
		Action doNothing = new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
		        //do nothing
		    }
		};	
		textArea.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0),
		                            "doNothing");
    }

    /**
     * Creates a popup menu.
     */
    public void createPopupMenu() {               
        commitMI = new JMenuItem(ElanLocale.getString("InlineEditBox.Commit"));
        commitMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0));
        commitMI.setActionCommand(EDIT_MENU_CMT);
        commitMI.addActionListener(this);
        popupMenu.add(commitMI);
        cancelMI = new JMenuItem(ElanLocale.getString("InlineEditBox.Cancel"));
        cancelMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0));
        cancelMI.setActionCommand(EDIT_MENU_CNL);
        cancelMI.addActionListener(this);
        popupMenu.add(cancelMI); 
        editInAnnModeMI = new JMenuItem(ElanLocale.getString("TranscriptionTableEditBox.EditInAnnotationMode"));
        editInAnnModeMI.setActionCommand(EDIT_IN_ANN_MODE);
        //editInAnnModeMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,Event.SHIFT_MASK));       
        editInAnnModeMI.addActionListener(this); 
        popupMenu.add(editInAnnModeMI); 
        mergeBeforeMI = new JMenuItem(ElanLocale.getString(ELANCommandFactory.MERGE_ANNOTATION_WB));
        mergeBeforeMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, Event.CTRL_MASK));       
        mergeBeforeMI.addActionListener(this); 
        popupMenu.add(mergeBeforeMI);        
        mergeNextMI = new JMenuItem(ElanLocale.getString(ELANCommandFactory.MERGE_ANNOTATION_WN));
        mergeNextMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, Event.CTRL_MASK));       
        mergeNextMI.addActionListener(this); 
        popupMenu.add(mergeNextMI);
        deleteAnnotationMI = new JMenuItem(ElanLocale.getString(ELANCommandFactory.DELETE_ANNOTATION));
        deleteAnnotationMI.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,Event.SHIFT_MASK));       
        deleteAnnotationMI.addActionListener(this); 
        popupMenu.add(deleteAnnotationMI);
        
        popupMenu.addSeparator();
        cutMI = new JMenuItem(ElanLocale.getString("InlineEditBox.Edit.Cut"));
        cutMI.setActionCommand("cut");
        cutMI.addActionListener(this);
        popupMenu.add(cutMI);
        copyMI = new JMenuItem(ElanLocale.getString("InlineEditBox.Edit.Copy"));
        copyMI.setActionCommand("copy");
        copyMI.addActionListener(this);
        popupMenu.add(copyMI);
        pasteMI = new JMenuItem(ElanLocale.getString("InlineEditBox.Edit.Paste"));
        pasteMI.setActionCommand("paste");
        pasteMI.addActionListener(this);
        popupMenu.add(pasteMI);
        selectAllMI = new JMenuItem(ElanLocale.getString("InlineEditBox.Edit.SelectAll"));
        selectAllMI.setActionCommand("selectAll");
        selectAllMI.addActionListener(this);
        popupMenu.add(selectAllMI);

    }

    /**
     * Returns whether or not the current annotation's value is restricted by a
     * ControlledVocabulary.
     *
     * @return true if a CV has to be used, false otherwise
     */
    public boolean isUsingControlledVocabulary() {
        return isUsingControlledVocabulary;
    }

    /**
     * Overrides setVisible(boolean) in JComponent.
     *
     * @param vis the visibility value
     */
    public void setVisible(boolean vis) {
        super.setVisible(vis);
        
        if (vis == false) {
            //closeIM();
            if (table == null) {
                setLocation(-10, -10);
            }
        }
    }

    /**
     * Overrides setSize(Dimension) in Component by performing a check to
     * guarantee a minimal  size.
     *
     * @param d the requested size
     */
    public void setSize(Dimension d) {
        // set usable sizes for edit region
        if (d.getWidth() < 60) {
            d = new Dimension(60, d.height);
        }

        if (d.getHeight() < 38) {
            d = new Dimension(d.width, 38);
        }

        //AK 16/08/2002 unfortunately all those setSizes are somewhere necessary
        super.setSize(d);
        setPreferredSize(d);
        textAreaScrollPane.setPreferredSize(d);
        textAreaScrollPane.setSize(d);
    }

    /**
     * Overrides setFont(Font) in Component by also setting the font for the
     * textareas.
     *
     * @param font the Font to use
     */
    public void setFont(Font font) {
        super.setFont(font);
        uniFont = font;

        // setFont() is used at initializing superclass - textarea not yet instantiated
        if (textArea != null) {
            textArea.setFont(font);
        }
    }
    
    /**
     * Sets the annotation that is to be edited. 
     * When <code>forceOpenCV</code> is true an 'open' text edit box will be used
     * even if the linguistic type has an associated ControlledVocabulery.
     * 
     * @param ann the annotation to be edited
     * @param forceOpenCV if true the associated CV will be ignored, 
     * editing will be open
     */
    public void setAnnotation (Annotation ann, boolean forceOpenCV) {    
//    	if(annotation != null && annotation==ann){
//    		return;
//    	}
    	//System.out.println("set annotatio");
		annotation = ann;
		oldText = ann.getValue();
		//textArea.setText(oldText.trim());
		textArea.setText(oldText);// don't trim, otherwise it's difficult to remove spaces newlines etc.

		try {
			annotationLocale = ((TierImpl) annotation.getTier()).getDefaultLocale();
			if (forceOpenCV) {
				isUsingControlledVocabulary = false;
			} else {			
				isUsingControlledVocabulary = ((TierImpl) annotation.getTier()).getLinguisticType()
												   .isUsingControlledVocabulary();
			}			
		} catch (Exception e) {
			LOG.warning(
				"Could not establish Default Language of Tier. Using System Default instead.");
			annotationLocale = null;//??
			isUsingControlledVocabulary = false;
		}
		
		if (!isUsingControlledVocabulary) {
			textArea.setEditable(true);
			
			if(caretPosition > 0 && caretPosition <= textArea.getText().length()){
				textArea.setCaretPosition(caretPosition);		
			} else {
				textArea.setCaretPosition(textArea.getText().length());
				caretPosition = -1;
			}
			
			
			//textArea.requestFocus();
		} else {
			textArea.setEditable(false);
			if (cvEntryComp == null) {
				cvEntryComp = new CVEntryComponent(JComboBox.class);
			}
			cvEntryComp.setAnnotation(annotation);
		}
    }

    /**
     * Sets the annotation that is to be edited.
     *
     * @param ann the annotation to be edited
     */
    public void setAnnotation(Annotation ann) {
    	setAnnotation(ann, false);
    }

    /**
     * Checks whether the annotation's value has been edited.
     *
     * @return true if the annotation's value has been edited, false otherwise
     */
    public boolean annotationModified() {
        return !oldText.equals(textArea.getText());
    }    

    /**
     * Resets elements and cleans up without applying any changes in the  value
     * of the annotation.  HB, 11 oct 01, changed from protected to public
     */
    public void cancelEdit() {
    	isEditing = false;
        closeIM();
        
        setVisible(false);
        if (table != null) {
            table.editingCanceled(new ChangeEvent(this));            
        }
    }

   /**
    * Checks for modifications, applies the modification if any, resets ,
    * cleans  up a bit and decide whether to exit the edit mode 
    * 
    * @param exit, if true exit the edit mode
    */
    public void commitChanges(boolean exit) {
		isEditing = false;
        closeIM();
        
        if(annotation == null){
        	return;
        }
        
        Object extRef = null;
        if (isUsingControlledVocabulary && (cvEntryComp != null)) {
        	if (cvEntryComp.getSelectedEntry() != null) {
        		textArea.setText(cvEntryComp.getSelectedEntryValue());
	            
	            extRef = cvEntryComp.getSelectedEntry().getExternalRef();
	            if(cvEntryComp.getSelectedEntry() instanceof ExternalCVEntry) {
	            	String entryId = ((ExternalCVEntry) cvEntryComp.getSelectedEntry()).getId();
	            	ExternalReference extRefCv = new ExternalReferenceImpl(entryId, ExternalReference.CVE_ID);
	            	ExternalReferenceGroup tmpExtRefGrp = new ExternalReferenceGroup();
	            	tmpExtRefGrp.addReference(extRefCv);
	            	if (extRef != null) {
						try {
							Object clone = ((ExternalReference) extRef).clone();
							if (clone instanceof ExternalReference) {
								tmpExtRefGrp.addReference((ExternalReference) clone);
							}
							extRef = tmpExtRefGrp;
						} catch (Exception e) {
							// TODO: handle exception
							e.printStackTrace();
						}
					} else {
						extRef = tmpExtRefGrp;
					}
	            }
        	}
        }
      
        // remove an ExternalCV reference by passing a null value.
        if (extRef == null && ((AbstractAnnotation) annotation).getExtRef() != null) {
        	extRef = new ExternalReferenceImpl(null, ExternalReference.CVE_ID);
        }
        
        String newText = "";
        boolean modified = annotationModified();

        if (modified) {
        	
        	newText = textArea.getText();        	
        	setVisible(false);     
        	
            Command c = ELANCommandFactory.createCommand(((Transcription) annotation.getTier() .getParent()),
                            ELANCommandFactory.MODIFY_ANNOTATION);
            Object[] args = new Object[] { oldText, newText, extRef };
            c.execute(annotation, args);
        }

        if (exit && table != null) {
            table.editingStopped(new ChangeEvent(this));
        }
    }
   
    /**
     * Restores the default locale (because of InputMethod stuff) of the
     * textareas, unless the language hasn't been changed when the annotation was set.
     */
    private void closeIM() {
    	if (annotationLocale != null) {
    		if (!textArea.getLocale().equals(Locale.getDefault())) {
				ImUtil.setLanguage(textArea, Locale.getDefault());
	        } 
    	}
    }

    /**
     * Forwards the cut action to the <code>textArea</code> 
     */
    private void doCut() {
    	 textArea.cut();
    }

    /**
     * Forwards the copy action to the <code>textArea</code> 
     */
    private void doCopy() {
    	textArea.copy();
    }

    /**
     * Forwards the paste action to the <code>textArea</code> 
     */
    private void doPaste() {
    	textArea.paste();
    }
    
	/**
	 * Forwards the select all action to the <code>textArea</code> 
	 */
    private void doSelectAll() {
    	textArea.selectAll();
    }
    
    /**
      * Merges the given annotation with the annotation before it     
      */
    private void mergeWithAnnBefore() {  
    	if(mergeBeforeMI.isEnabled()){
    		Annotation ann = (Annotation) table.getValueAt(table.getCurrentRow()-1, table.getCurrentColumn());
    		caretPosition = ann.getValue().length() + textArea.getCaretPosition();        	
    		commitChanges(true);
    		if(((TierImpl)annotation.getTier()).hasParentTier()){    			
    			viewer.mergeBeforeAnn(annotation.getParentAnnotation());
    		}else {
    			viewer.mergeBeforeAnn(annotation);
    		}     		
    	}
    }
    
    /**
     * Merges the given annotation with the annotation next to it
     */
    private void mergeWithNextAnn() {    	
    	if(mergeNextMI.isEnabled()){
    		caretPosition = textArea.getCaretPosition();
    		commitChanges(true);    		
    		if(((TierImpl)annotation.getTier()).hasParentTier()){    			
    			viewer.mergeNextAnn(annotation.getParentAnnotation());
    		}else {
    			viewer.mergeNextAnn(annotation);
    		}    		
    	}
    }
    
    /**
     * Deletes the annotation
     */
    private void deleteAnnotation() {
    	commitChanges(true);
    	viewer.deleteAnnotation(annotation);		
	}

    /**
     * Enables/disables edit menu items in the popup menu.<br>
     * Check the contents of the <code>textArea</code> component and the
     * system clipboard.
     */
    private void updatePopup() {   	
        if ((textArea.getSelectedText() == null) ||
                (textArea.getSelectedText().length() == 0)) {
            cutMI.setEnabled(false);
            copyMI.setEnabled(false);
        } else {
            cutMI.setEnabled(true);
            copyMI.setEnabled(true);
        }

        if (isTextOnClipboard()) {
            pasteMI.setEnabled(true);
        } else {
            pasteMI.setEnabled(false);
        }
        
		if (textArea.getText() == null || 
				textArea.getText().length() == 0) {
			selectAllMI.setEnabled(false);
		} else {
			selectAllMI.setEnabled(true);
		}
		
		if(annotation == null){
			editInAnnModeMI.setEnabled(false);
		} else {
			editInAnnModeMI.setEnabled(true);
		}		
		
		if(viewer.getMerge()){
			int leadColumn = table.getCurrentColumn();
			int leadRow = table.getCurrentRow();
			Object obj ;
			if(leadRow != 0){
				obj = table.getValueAt(leadRow-1, leadColumn);
				if(obj instanceof Annotation){
					if( annotation.getTier().equals(((Annotation)obj).getTier())){
						mergeBeforeMI.setEnabled(true);
					}else{
						mergeBeforeMI.setEnabled(false);
					}
				} else {
					mergeBeforeMI.setEnabled(false);
				}
			} else {
				mergeBeforeMI.setEnabled(false);
			}			
			
			if(leadRow < (table.getRowCount()-1)){
				obj = table.getValueAt(leadRow+1, leadColumn);
				if(obj instanceof Annotation){
					if( annotation.getTier().equals(((Annotation)obj).getTier())){
						mergeNextMI.setEnabled(true);
					}else{
						mergeNextMI.setEnabled(false);
					}
				} else {
					mergeNextMI.setEnabled(false);
				}
			} else {
				mergeNextMI.setEnabled(false);
			}
		} else {
			mergeBeforeMI.setEnabled(false);
			mergeNextMI.setEnabled(false);
		}
    }    
    
    public void showPopUp(java.awt.Component obj , int x, int y) {
    	//if(obj !=)
		updatePopup();
		popupMenu.show(obj, x, y);
		popupMenu.setVisible(true);
    }

    /**
     * Checks whether the contents of the system clipboard can be paste into  a
     * textcomponent.
     *
     * @return true if there is contents of type text, false otherwise
     */
    private boolean isTextOnClipboard() {
        Transferable contents = null;

        try {
            contents = Toolkit.getDefaultToolkit().getSystemClipboard()
                              .getContents(this);
        } catch (IllegalStateException ise) {
            LOG.warning("Could not access the system clipboard.");
        }

        if (contents != null) {
            DataFlavor[] flavors = contents.getTransferDataFlavors();
            DataFlavor best = DataFlavor.selectBestTextFlavor(flavors);

            if (best != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns the (configured) editor component, ready to be used in a
     * viewer's  layout.
     *
     * @return the editor component
     *
     * @see #configureEditor(Class, Font, Dimension)
     * @see #startEdit()
     */
    public JComponent getEditorComponent() {
        if (editorComponent == null) {
            return this;
        }
        return editorComponent;
    }

    /**
     * Sets up and configures a certain kind of editor component.<br>
     *
     * @param preferredComponent DOCUMENT ME!
     * @param font DOCUMENT ME!
     * @param size DOCUMENT ME!
     *
     * @see #getEditorComponent()
     * @see #startEdit()
     */
    public void configureEditor(Class preferredComponent, Font font,
        Dimension size) {
        if (preferredComponent == JScrollPane.class) {
                if (font != null) {
                    setFont(font);
                }
                if (size != null) {
                    setSize(size);
                }
                editorComponent = textAreaScrollPane;           
        } else if (preferredComponent == JComboBox.class) {
            if (isUsingControlledVocabulary) {
                if (cvEntryComp == null) {
                    cvEntryComp = new CVEntryComponent(preferredComponent);
                    cvEntryComp.setAnnotation(annotation);
                }                 
                if (font != null) {
                    cvEntryComp.setFont(font);
                }
                if (size != null) {
                    cvEntryComp.getEditorComponent().setSize(size);
                }
                editorComponent = cvEntryComp.getEditorComponent();
            }
        }
    }

    /**
     * Makes the editorComponent visible and tries to grabFocus.<br>
     * This should be called after configuring and getting the editor
     * component
     *
     * @see #configureEditor(Class, Font, Dimension)
     * @see #getEditorComponent()
     */
    public void startEdit() {
    	if(isEditing){
    		return;
    	}
		isEditing = true;
		
        if (editorComponent == this) {
            setVisible(true);
            requestFocus();
        } else {
            if (isUsingControlledVocabulary) {            	
                cvEntryComp.grabFocus();                
            } else {
                editorComponent.requestFocus();
            }
        }	
        
        //System.out.println("strat edit");
    }
    
    /**
     * Menu items' ActionPerformed handling.
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        if (command.equals(EDIT_MENU_CNL) || command.equals("close")) {
            cancelEdit();
        } else if (command.equals(EDIT_MENU_CMT)) {
        	commitChanges(true);
        } else if (command.equals(EDIT_IN_ANN_MODE)) {        	
        	commitChanges(true);
        	if(annotation != null){    
        		viewer.getViewerManager().getMasterMediaPlayer().stop();
        		viewer.editInAnnotationMode();
        	}
        } else if (command.equals("cut")) {
            doCut();
        } else if (command.equals("copy")) {
            doCopy();
        } else if (command.equals("paste")) {
            doPaste();
        } else if (command.equals("selectAll")) {
			doSelectAll();
		} else if (e.getSource() == mergeBeforeMI) {
			mergeWithAnnBefore();
		}else if (e.getSource() == mergeNextMI) {
			mergeWithNextAnn();
		} else if (e.getSource() == deleteAnnotationMI){
			deleteAnnotation();
		} else {		
            for (int i = 0; i < numberOfLocales; i++) {
                if (command.equals(allLocales[i].getDisplayName())) {
                    annotationLocale = allLocales[i];
                    ImUtil.setLanguage(textArea, annotationLocale);
                    textArea.setFont(uniFont);
                    break;
                }
            }
        }
    }
    
	/**
     * Mouse event handling for popping up the popup menu.
     *
     * @param e the mouse event
     */
    public void mouseClicked(MouseEvent e) {
    }

    /**
     * Stub
     *
     * @param e the mouse event
     */
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Stub
     *
     * @param e the mouse event
     */
    public void mouseExited(MouseEvent e) {    	
    }

    /**
     * Stub
     *
     * @param e the mouse event
     */
    public void mousePressed(MouseEvent e) {
		if (javax.swing.SwingUtilities.isRightMouseButton(e) ||
			 e.isPopupTrigger()) {
			showPopUp(textArea, e.getX(), e.getY());
		}
    }

    /**
     * Stub
     *
     * @param e the mouse event
     */
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * M_P all e.consume() calls outcommented because the events are needed to
     * deselect a newly  made / edited annotation in EudicoAnnotationFrame
     * with the Escape key M_P 25 june 2003 Just the first outcommented. The
     * last when has to be consumed.
     *
     * @param e the key event
     */
    public void keyPressed(KeyEvent e) {
        // KB Cancel Changes
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
            //e.consume();
            cancelEdit();
        }  		
		//
        else if(e.getKeyCode() == KeyEvent.VK_UP ){
        	e.consume();
        	commitChanges(false);
            table.goToEditableCellUp();
            //table.startEdit(e);           
        }  
		//
        else if(e.getKeyCode() == KeyEvent.VK_DOWN ){
        	e.consume();
        	commitChanges(false);
            table.goToEditableCellDown();
            //table.startEdit(e);
        } 
		//
        else if(e.getKeyCode() == KeyEvent.VK_LEFT  && e.getModifiers() == Event.ALT_MASK) {        
        	e.consume();
        	commitChanges(false);
            table.goToEditableCellLeft();
            //table.startEdit(e);           
        } 		
		//
        else if(e.getKeyCode() == KeyEvent.VK_RIGHT  && e.getModifiers() == Event.ALT_MASK) {        
        	e.consume();
        	commitChanges(false);
            table.goToEditableCellRight();
            //table.startEdit(e);           
        } 
		//
        else if((e.getModifiers() == Event.SHIFT_MASK) && e.getKeyCode() == KeyEvent.VK_TAB ){
        	e.consume();
        	viewer.playIntervalFromBeginTime(annotation.getBeginTimeBoundary(), annotation.getEndTimeBoundary());
        }  
        // play the media for the time interval
        else if (e.getKeyCode() == KeyEvent.VK_TAB) {
        	e.consume();
        	viewer.playInterval(annotation.getBeginTimeBoundary(), annotation.getEndTimeBoundary(), true);
        }            
        // delete annotation
        else if (e.getKeyCode() == KeyEvent.VK_DELETE && (e.getModifiers() == Event.SHIFT_MASK)) {  
        	e.consume();      
            deleteAnnotation();
        } 
        // merge with before
        else if (e.getKeyCode() == KeyEvent.VK_B && (e.getModifiers() == Event.CTRL_MASK)) {
        	e.consume();
        	mergeWithAnnBefore();
        }
        // merge with Next
        else if (e.getKeyCode() == KeyEvent.VK_A && (e.getModifiers() == Event.CTRL_MASK)) {  
        	e.consume();
        	mergeWithNextAnn();
        }
        // KB Confirm
        else if ((e.getKeyCode() == KeyEvent.VK_ENTER) ) {
            e.consume();
            commitChanges(true);
            table.goToNextEditableCell();
            table.startEdit(e);
        }  else if(e.getKeyCode() == KeyEvent.VK_SPACE && e.getModifiers() == Event.CTRL_MASK){
        	e.consume();
        	viewer.playInterval(annotation.getBeginTimeBoundary(), annotation.getEndTimeBoundary(), true);
        } else if(e.getKeyCode() == KeyEvent.VK_L && (e.getModifiers() & Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()) != 0){
        	e.consume();
        	viewer.toggleLoopMode();
        }
        // June 2010 capture the standard undo/redo events to prevent undo/redo being called in the enclosing program
        // while the edit box is active. Ideally an UndoManager should be installed for the edit box.
        // maybe better to use the user defined shortcuts for undo and redo
        else if ( (e.getKeyCode() == KeyEvent.VK_Z || e.getKeyCode() == KeyEvent.VK_Y) && 
    			(e.getModifiers() & Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()) != 0) {
        	// do nothing unless UndoManager is active 
        	e.consume();
        }		
        else {
        	// consume the event to prevent actions with keyboard shortcuts to be triggered?
        	// no, that disables e.g. the backspace key on a mac
        	//e.consume();
        }
    }

	/**
     * Stub
     *
     * @param e the key event
     */
    public void keyReleased(KeyEvent e) {
    }

    /**
     * Stub
     *
     * @param e the key event
     */
    public void keyTyped(KeyEvent e) {
    }
    
    /**
     * Updates UI elements after a change in the selected Locale.
     */
    public void updateLocale() {      
        commitMI.setText(ElanLocale.getString("InlineEditBox.Commit"));
        cancelMI.setText(ElanLocale.getString("InlineEditBox.Cancel"));
        cutMI.setText(ElanLocale.getString("InlineEditBox.Edit.Cut"));
        copyMI.setText(ElanLocale.getString("InlineEditBox.Edit.Copy"));
        pasteMI.setText(ElanLocale.getString("InlineEditBox.Edit.Paste"));
    }

    //////////////////////////
    // inner class: a component for selecting an entry from a list of entries of a cv
    //////////////////////////

    /**
     * A class that provides a component for the selection of an entry from  a ControlledVocabulary.<br>
     * The current possible delegate components are a JScrollPane containing
     * a JList or a JComboBox
     *
     * @author Han Sloetjes
     */
    class CVEntryComponent { 

        /** a combo box editor component */
        private JComboBox box;

        /** a model for the combo box */
        private DefaultComboBoxModel entryBoxModel;
        
        /**
         * the array of CV entries from the ControlledVocabulary referenced by
         * the  LinguisticType in use by the Tier containing the current
         * annotation
         */
        private CVEntry[] entries;

        /** the annotation to edit */
        private Annotation annotation;
        
        /** the length in numbers of characters of the longest entry **/
        private int maxEntryLength = 0;	
        Object temp;

        /**
         * Creates a new entrylist and initializes components.<br>
         * Components are being initialized depending on the type of the
         * argument.
         *
         * @param componentClass the type of component to use for edit
         *        operations
         */
        public CVEntryComponent(Class componentClass) {
            initComponents(componentClass);
        }

        /**
         * Returns the current delegate component.
         *
         * @return the delegate component for editing actions
         */
        public JComponent getEditorComponent() {
            return box;
        }
             
        /**
         * Initializes a combo box.  Adds listeners.
         *
         * @param component the type of component to use for editing
         */
        private void initComponents(Class component) {
        	if (component == JComboBox.class) {
                if (box == null) {
                    entryBoxModel = new DefaultComboBoxModel();
                    
                    box = new JComboBox(entryBoxModel);
                    
                    KeyAdapter keyList = new KeyAdapter() {
                        public void keyReleased(KeyEvent e) {
                        	
                        	if (e.getKeyCode() == KeyEvent.VK_ENTER) { 
                            		if (box.getSelectedItem() != null) {
                            			TranscriptionTableEditBox.this.keyPressed(e);
                            		} else {
                            			TranscriptionTableEditBox.this.cancelEdit();
                            		}
                            } else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
                            	TranscriptionTableEditBox.this.cancelEdit();
                            } 
                        }    
                    };
                    
                  box.addKeyListener(keyList);
                }
            } 
        }

        /**
         * Sets the font for the entry list component.
         *
         * @param f the font
         */
        public void setFont(Font f) {          
                box.setFont(f);
        }

        /**
         * Gets the entry array with the entries in the cv referenced by the
         * linguistic type of the tier.
         * HS Jan 2011: try to re-use the (potentially long) list of cv entries
         *
         * @param annotation the active annotation
         */
        public void setAnnotation(Annotation annotation) {
        	ControlledVocabulary oldCV = null;
        	
        	if (this.annotation != null) {
        		TierImpl tier = (TierImpl) this.annotation.getTier();
                TranscriptionImpl trans = (TranscriptionImpl) tier.getParent();
                oldCV = trans.getControlledVocabulary(tier.getLinguisticType()
                                  .getControlledVocabylaryName());
        	}
        	
            this.annotation = annotation;
            ControlledVocabulary cv = null;            
            if (annotation != null) {
                TierImpl tier = (TierImpl) annotation.getTier();
                TranscriptionImpl trans = (TranscriptionImpl) tier.getParent();
                cv = trans.getControlledVocabulary(tier.getLinguisticType()
                                .getControlledVocabylaryName());
            }            
            if (cv != null) {
            	// reload local CV's anyway
            	if (cv != oldCV || !(cv instanceof ExternalCV)) {
            		entries = cv.getEntries();
                    
                    if (entryBoxModel != null) {
                    	entryBoxModel.removeAllElements();
                    }
                    fillModel(true);
            	} else {
            		// else reuse existing list, currently only for external CV because there is 
            		// no notification of changes in local CV's yet
                    fillModel(false);
            	}               
            } else { // cv == null
            	entries = new CVEntry[]{};                

                if (entryBoxModel != null) {
                    entryBoxModel.removeAllElements();
                }
                fillModel(false);
            }
        }

        /**
         * Fills the model of either the combo box or the list with the entries
         * of the current Controlled Vocabulary.
         */
        private void fillModel(boolean reload) {
            String value = null;

            if (annotation != null) {
                value = annotation.getValue();
            }
          	if (reload) {
                for (int i = 0; i < entries.length; i++) {
                    entryBoxModel.addElement(entries[i]);

                    if ((value != null) && value.equals(entries[i].getValue())) {
                        entryBoxModel.setSelectedItem(entries[i]);
                    }
                    
                    if (entries[i].getValue() != null && entries[i].getValue().length() > maxEntryLength) {
                    	maxEntryLength = entries[i].getValue().length();
                    }
                }
           	} else {
           		if (value != null) {
           			CVEntry entry;
           			for (int i = 0; i < entryBoxModel.getSize(); i++) {
           				entry = (CVEntry) entryBoxModel.getElementAt(i);
           				if (value.equals(entry.getValue())) {
           					box.setSelectedIndex(i);
           					break;
           				}
           			}
           		}
           	}
            if ((entries.length > 0) && (box.getSelectedIndex() < 0)) {
                box.setSelectedIndex(0);
            }
       }

        /**
         * Tries to grant the focus to the component.
         */
        public void grabFocus() {         
                box.requestFocus();
        }

        /**
         * Returns the currently selected entry value.
         *
         * @return the currently selected entry value or null
         */
        public String getSelectedEntryValue() {
            String value = null;
            
            if (box.getSelectedItem() != null) {
                 value = ((CVEntry) box.getSelectedItem()).getValue();
            } else {
                 value = annotation.getValue();
            }
            return value;
        }
        
        /**
         * Returns the currently selected entry.
         *
         * @return the currently selected entry or null
         */
        public CVEntry getSelectedEntry() {
        	CVEntry value = null;
        	
        	if (box.getSelectedItem() != null) {
        		value = (CVEntry) box.getSelectedItem();
            }
        	
            return value;
        }
        
        /**
         * Returns the number of characters of the longest entry value.
         * 
         * @return the length of the longest entry
         */
        public int getMaxEntryLength() {
        	return maxEntryLength;
        }
        /**
         * Action handling.
         *
         * @param ae the action event
         */
        public void actionPerformed(ActionEvent ae) {
            if (ae.getSource() == box) {
                if ((ae.getID() == ActionEvent.ACTION_PERFORMED) &&
                        (ae.getModifiers() == InputEvent.BUTTON1_MASK  )) {
                    // prevent that the first click / doubleclick on the combo box
                    // causes a commit
                    if (box.isPopupVisible()) {
                    	TranscriptionTableEditBox.this.commitChanges(true);
                    }
                }
            } 
        }
    }

	
}
