package mpi.eudico.client.annotator.transcriptionMode;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JViewport;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.Selection;
import mpi.eudico.client.annotator.SelectionPanel;
import mpi.eudico.client.annotator.ViewerManager2;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.layout.TranscriptionManager;
import mpi.eudico.client.annotator.player.NativeMediaPlayerWindowsDS;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.SignalViewer;
import mpi.eudico.client.mediacontrol.ControllerEvent;
import mpi.eudico.client.mediacontrol.StartEvent;
import mpi.eudico.client.mediacontrol.StopEvent;
import mpi.eudico.client.util.TableSubHeaderObject;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.event.ACMEditEvent;
import mpi.eudico.server.corpora.event.ACMEditListener;

/**
 * Viewer for the transcription table in the transcription mode layout
 * with added functions, which allows easy transcription for the selected type of tiers
 * 
 * @author aarsom
 *
 */
public class TranscriptionViewer  extends AbstractViewer implements ListSelectionListener, ACMEditListener{	

    /** Holds value of property DOCUMENT ME! */
    public static final String CREATE_ANN = "create";
    
    private TranscriptionManager layoutManager;
    private ViewerManager2 viewerManager;
    //private ElanMediaPlayer viewerManager.getMasterMediaPlayer();
    private SignalViewer signalViewer;
    
    private JScrollPane scroller;
    private TranscriptionTable table;
    private TranscriptionTableModel tableModel;
    
    private String column1Type;
    private String column2Type;
    private String column3Type;

 	private HashMap<TierImpl,TierImpl> tierMapColumn1;
 	private HashMap<TierImpl,TierImpl> tierMapColumn2;
 	private HashMap<TierImpl,TierImpl> tierMapColumn3;
 	private HashMap<String,Integer> columnOrder;
 	private List<Color> tierColorsList;
 	
 	private boolean merge = false;
 	private boolean showTierNames = true;
 	private boolean autoPlayBack = true; 	 
 	private boolean secondColumnActivated = false;
 	
 	/**
 	 * Creates a instance of TranscriptionViewer
 	 * 
 	 * @param viewerManager
 	 * @param transManager
 	 */
    public TranscriptionViewer(ViewerManager2 viewerManager){     	
    	this.viewerManager = viewerManager; 
    	//viewerManager.getMasterMediaPlayer() = viewerManager.getMasterMediaPlayer();
   	 	signalViewer = viewerManager.getSignalViewer();    
    }    
    
    /**
     * Initializes this viewer with the necessary components
     * 
     * @param transManager
     */
    public void intializeViewer(TranscriptionManager transManager){
    	layoutManager = transManager;
	
    	tierColorsList = new ArrayList<Color>();
		tierMapColumn1 = new HashMap<TierImpl,TierImpl>();
		tierMapColumn2 = new HashMap<TierImpl,TierImpl>();
		tierMapColumn3 = new HashMap<TierImpl,TierImpl>();
		columnOrder = new HashMap<String, Integer>();
		initializeTable();
    }
    
    public Transcription getTranscription(){
    	return viewerManager.getTranscription();
    }
    
    /**
     * Set the maps selected by the users, 
     * to load the table
     * 
     * @param tierMap, map with parent and first column dependent tier
     * @param tierMap1, map with parent and second column dependent tier
     */
    public void setTierMaps(HashMap<TierImpl,TierImpl> tierMap1, HashMap<TierImpl,TierImpl> tierMap2, HashMap<TierImpl,TierImpl> tierMap3){
    	if(tierMap1 != null){
    		tierMapColumn1.putAll(tierMap1);
    	}
    	if(tierMap2 != null){
    		tierMapColumn2.putAll(tierMap2);
    	}
    	
    	if(tierMap3 != null){
    		tierMapColumn3.putAll(tierMap3);
    	}
    	
    	setPreference("TranscriptionTable.TierMap1", changeToStorableMap(tierMap1), viewerManager.getTranscription());
    	setPreference("TranscriptionTable.TierMap2", changeToStorableMap(tierMap2), viewerManager.getTranscription());
    	setPreference("TranscriptionTable.TierMap3", changeToStorableMap(tierMap3), viewerManager.getTranscription());
    }
    
    /**
     * Changes the format of the given map to a 
     * new format such that the map can be stored by the 
     * PrefernceWriter
     * 
     * @param map, map which is can be stored
     * @return
     */
    private HashMap<String, TierImpl> changeToStorableMap(HashMap<TierImpl, TierImpl> map){
    	if(map != null){
    		HashMap<String, TierImpl> newMap = new HashMap();
    	
    		TierImpl keyObj;
    		Iterator<TierImpl> keyIt = map.keySet().iterator();

    		while (keyIt.hasNext()) {
    			keyObj = keyIt.next();
    			newMap.put(keyObj.getName(), map.get(keyObj));
    		}
    		return newMap;
    	}    	
    	return null;
    }
    
    /**
     * Changes the format of the given map back
     * to the original format
     * 
     * @param map, original map
     * @return
     */
    private HashMap<TierImpl, TierImpl> changeToUsableMap(HashMap<String, TierImpl> map){
    	if(map != null){
    		HashMap<TierImpl, TierImpl> newMap = new HashMap();
    	
    		String keyObj;
    		Iterator<String> keyIt = map.keySet().iterator();

    		while (keyIt.hasNext()) {
    			keyObj = keyIt.next();
    			newMap.put((TierImpl)viewerManager.getTranscription().getTierWithId(keyObj), map.get(keyObj));
    		}
    		return newMap;
    	}    	
    	return null;
    }

    public HashMap<TierImpl,TierImpl> getTierMap(String columnName){
    	if (columnName.startsWith(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX))){
    		return tierMapColumn1;
    	} else if (columnName.startsWith(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX))){
    		return tierMapColumn2;
    	}  else if (columnName.startsWith(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX))){
    		return tierMapColumn3;
    	}   	
    	return null;
    }
    
    /**
	 * Initialize the table
	 */
	private void initializeTable() {
		tableModel = new TranscriptionTableModel();	 
		table = new TranscriptionTable();  
		table.setModel(tableModel);
	    table.setDefaultEditor(Object.class, new TranscriptionTableCellEditor(this)); 
	    table.setDefaultRenderer(Object.class, new TranscriptionTableCellRenderer(getTranscription()));
	    table.getColumnModel().getColumn(0).setMinWidth(40);
	    table.getColumnModel().getColumn(0).setPreferredWidth(40);
	    table.getColumnModel().getColumn(0).setMaxWidth(40);
	    
	    // hide child tier columns in the beginning 
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMinWidth(0);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setPreferredWidth(0);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMaxWidth(0);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setResizable(false);	
	    
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMinWidth(0);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setPreferredWidth(0);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMaxWidth(0);		
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setResizable(false);	
	    
	    scroller = new JScrollPane(table);
	    
		table.addMouseListener(new MouseAdapter() {		    
	        public void mouseReleased(MouseEvent e) {	        	
	        	if ( javax.swing.SwingUtilities.isRightMouseButton(e)){	   
	        		if(table.isEditing()){
	        			((TranscriptionTableCellEditor)table.getCellEditor()).commitChanges(true);
	        		}
	        		
	        		int r = table.rowAtPoint(e.getPoint());
	        		int c = table.columnAtPoint(e.getPoint());
	        		if (r >= 0 && r < table.getRowCount()) {
		               table.changeSelection(r, c, false, false);
		            }	        	
	        		
	        		table.startEdit(null);
	        		if(table.isEditing()){
	        			((TranscriptionTableCellEditor)table.getCellEditor()).showPopUp(table, e.getX(), e.getY());
	        		}
	        	}
	        }
		});	
	    
	    scroller.addComponentListener(new ComponentAdapter() {			
			public void componentResized(ComponentEvent e) {	
				//table.reCalculateRowHeight();
				long selectionBeginTime = 0L;
				long selectionEndTime = 0L;
				
				
				boolean playback = isAutoPlayBack();
				setAutoPlayBack(false);
				long mediaTime = viewerManager.getMasterMediaPlayer().getMediaTime();
				table.startEdit(null);
				setAutoPlayBack(playback);
				viewerManager.getMasterMediaPlayer().setMediaTime(mediaTime);	
				
				if(viewerManager.getSelection() != null){
					selectionBeginTime = viewerManager.getSelection().getBeginTime();
					selectionEndTime = viewerManager.getSelection().getEndTime();
				}
				
				if(selectionBeginTime > 0L && selectionEndTime > 0L && signalViewer != null){
					//signalViewer.repaint();
					signalViewer.setSelection(selectionBeginTime, selectionEndTime);
					((SelectionPanel) viewerManager.getMediaPlayerController().getSelectionPanel()).setBegin(selectionBeginTime);
					((SelectionPanel) viewerManager.getMediaPlayerController().getSelectionPanel()).setEnd(selectionEndTime);
				}
				
			}
		});
	    	   
		setLayout(new BorderLayout());		
		add(scroller, BorderLayout.CENTER);	
	    table.getSelectionModel().addListSelectionListener(this);	
	    
	    setPreferredFontAndColorSettings();
	}
	
	/**set the preferred for the scroll pane
	 * 
	 * @param width
	 * @param height
	 */
	public void setScrollerSize(int width, int height) {
		scroller.setPreferredSize(new Dimension(width, height));		
	}
	
	/**
	 * Sets the preferred font and color setting for the tiers
	 *                         							
	 */
	private void setPreferredFontAndColorSettings(){			
		//preferred Font Color
		Object colorMap = Preferences.get("TranscriptionMode.Temp.TierColors", viewerManager.getTranscription());
		if (colorMap instanceof HashMap) {
			table.setFontColorForTiers((HashMap) colorMap);			
		}		
		
		colorMap = Preferences.get("TierColors", viewerManager.getTranscription());
		if (colorMap instanceof HashMap) {
			table.setFontColorForTiers((HashMap) colorMap);			
		}
		
		//preferred fonts
		Object fo = Preferences.get("TierFonts", viewerManager.getTranscription());
		if (fo instanceof HashMap) {
			table.setFontsForTiers((HashMap) fo);
		}
	}
	
	/**
	 * Sets the font size of the table
	 * 
	 * @param size
	 */
	public void setFontSize(Integer size) {
		table.setFont(new Font(table.getFont().getFontName(), table.getFont().getStyle(), size));
	}
	
	/**
	 * Returns the current font size of the table
	 * @return
	 */
	public Integer getFontSize() {
		// TODO Auto-generated method stub
		return table.getFontSize();
	}
	
	/**
	 * Sets s flag, whether "enter" moves via column or not
	 * 
	 * @param selected, if true moves via column
	 * 		if false, moves by row in the current column
	 */
	public void moveViaColumn(boolean selected) {
		table.moveViaColumn(selected);			
	}
	
	/**
	 * Sets s flag, whether the editing cell should 
	 * always be in the center or not
	 * 
	 * @param selected, if true scrolls the current editing cell to the center of the table
	 * 		if false, has the default behaviour of the table
	 */
	public void scrollActiveCellInCenter(boolean selected) {
		table.scrollActiveCellInCenter(selected);
	}
	
	/**
	 * Sets whether the media should automatically
	 * played when start editing a cell
	 * 
	 * @param selected, if true, the autoplaymaode is set true
	 */
	public void setAutoPlayBack(boolean selected) {
		autoPlayBack = selected;		
	}
	
	/**
	 * Returns whether the autoplayBack mode is on or off
	 * @return
	 */
	public boolean isAutoPlayBack() {
		return autoPlayBack;
	}
	
	public boolean isTierNamesShown() {
		return showTierNames;
	}
	
	public void showColorOnlyOnNoColumn(boolean selected) {
		((TranscriptionTableCellRenderer)table.getDefaultRenderer(Object.class)).showColorOnlyOnNoColumn(selected);
		if(!layoutManager.isInitialized() || table.getRowCount() <= 0){
			return;
		} 
		
		table.revalidate();
		table.repaint();
	}
	
	/**
	 * Method to show/ hide the tier names from the table
	 * 
	 * @param selected, if true, shows the tierNames in the table,
	 * 					if false, then the tierNames are hidden
	 */
	public void showTierNames(boolean selected) {
		showTierNames = selected;
		((TranscriptionTableCellRenderer)table.getDefaultRenderer(Object.class)).setShowTierNames(showTierNames);
		
		if(!layoutManager.isInitialized() || table.getRowCount() <= 0){
			return;
		}
		
		int currentRow = table.getCurrentRow();
		int currentColumn = table.getCurrentColumn();
		int annNumber = 0;
		boolean plaBack = isAutoPlayBack();	
		setAutoPlayBack(false);
		if(currentRow < 0 || currentRow >= table.getRowCount()){
			annNumber = 1;
		}else{
			Object val = table.getValueAt(currentRow, 0);
			if(val instanceof Integer){ 
				annNumber = ((Integer)val).intValue();
			}
		}		
		
		if(layoutManager.isInitialized()){
			loadTable();
		}
		
		if(showTierNames){
			int n = 0;
			for(int i=0; i < table.getRowCount(); i++){
				Object val = table.getValueAt(i, 0);
				if(val instanceof Integer){
					n = ((Integer)val).intValue();
					if(n == annNumber){
						table.changeSelection(i, currentColumn, false, false);
						if(table.editCellAt(i, currentColumn)){
							table.startEdit(null);
						}
						break;
					}
				}
			}
		}else{			
			if(annNumber > 0 && (table.getRowCount()-1) >= (annNumber-1)){
				table.changeSelection(annNumber-1, currentColumn, false, false);
				if(table.editCellAt(annNumber-1, currentColumn)){
					table.startEdit(null);
				}
			}
		}
		
		setAutoPlayBack(plaBack);
	}
	
	/**
	 * Sets the type for first column
	 * 
	 * @param type
	 */
	public void setColumn1Type(String type){
		column1Type = type;
	}
	
	/**
	 * Sets the type for second column
	 * 
	 * @param type
	 */
	public void setColumn2Type(String type){
		column2Type = type;
	}
	
	/**
	 * Sets the type for third column
	 * 
	 * @param type
	 */
	public void setColumn3Type(String type){
		column3Type = type;
	}
	
	/**
	 * Gets the type for first column
	 * 
	 * @return column1Type
	 */
	public String getColumn1Type(){
		return column1Type;
	}
	
	/**
	 * Gets the type for second column
	 * 
	 * @return column2Type
	 */
	public String getColumn2Type(){
		return column2Type;
	}
	
	/**
	 * Gets the type for third column
	 * 
	 * @return column3Type
	 */
	public String getColumn3Type(){
		return column3Type;
	}
	
	/**
	 * Checks whether merging of annotations is 
	 * possible for the selected type of parent
	 * tiers
	 */
	public void checkForMerge(){
		merge = false;
		if(column1Type != null){
			Vector tierList = viewerManager.getTranscription().getTiersWithLinguisticType(column1Type);
			if(tierList != null && tierList.size() > 0){
				//to set whether merging should be possible of the selected type of tiers
				for(int i =0; i< tierList.size(); i++){
					if(tierMapColumn1.containsValue(tierList.get(i))){
						if(((TierImpl)tierList.get(i)).hasParentTier()){
							TierImpl parentTier = (TierImpl) ((TierImpl)tierList.get(i)).getParentTier();
							if(parentTier.getLinguisticType().getConstraints() == null && ((TierImpl)tierList.get(i)).getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
								merge = true;
								break;
							}
						} else {
							merge = true;
							break;
						}
					} 
				}
			}
		}
	}
	
	/**
	 * Focuses the table
	 */
	public void focusTable(){
		if(table.isEditing()){
			table.getEditorComponent().requestFocusInWindow();
		}else{
			table.requestFocusInWindow();
		}
	}
	
	/**
	 * Return whether merging is possible of not
	 * 
	 * @return merge
	 */
	public boolean getMerge(){
		return merge;
	}
	
	/**
	 * Clears the table
	 */
	public void clearTable() {
		table.getSelectionModel().removeListSelectionListener(this);
		table.clearRows();
		table.repaint();
		table.getSelectionModel().addListSelectionListener(this);
	}

	/**
	 * Switch on/off the loop mode
	 */
	public void toggleLoopMode() {
		layoutManager.toggleLoopMode();
		
	}
	
	/**
	 * stores the column order in the map
	 */
	public void storeColumnOrder(){		
		columnOrder.put(TranscriptionTableModel.COLUMN1_PREFIX, table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX)));
		columnOrder.put(TranscriptionTableModel.COLUMN2_PREFIX, table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)));
		columnOrder.put(TranscriptionTableModel.COLUMN3_PREFIX, table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)));
		setPreference("TranscriptionTable.ColumnOrder", columnOrder, viewerManager.getTranscription());
	}
	
	private void updateColumnHeaders(){
//		String headerValue;
//		String columnName;
		
//		if(column1Type != null){
//			columnName = tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX) ;
//			headerValue = tableModel.COLUMN1_PREFIX + " : " + column1Type;
//			table.getColumn(columnName).setIdentifier(headerValue);
//			//table.getColumnModel().get
//			//table.getColumn(columnName).setHeaderValue(headerValue);			
//			tableModel.setColumnName(tableModel.COLUMN1_PREFIX, headerValue);			
//		}
//		
//		if(column2Type != null){
//			columnName = tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX) ;
//			headerValue = tableModel.COLUMN2_PREFIX + " : " + column2Type;
//			//table.getColumn(columnName).setIdentifier(headerValue);
//			table.getColumn(columnName).setHeaderValue(headerValue);	
//			tableModel.setColumnName(tableModel.COLUMN2_PREFIX, headerValue);		
//		}
//		
//		if(column3Type != null){
//			columnName = tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX) ;
//			headerValue = tableModel.COLUMN3_PREFIX + " : " + column3Type;
//			//table.getColumn(columnName).setIdentifier(headerValue);
//			table.getColumn(columnName).setHeaderValue(headerValue);	
//			tableModel.setColumnName(tableModel.COLUMN3_PREFIX, headerValue);		
//		}
		
		String headerValue1 = null;
		String headerValue2= null;
		String headerValue3= null;
		
		if(column1Type != null){			
			headerValue1 = tableModel.COLUMN1_PREFIX + " : " + column1Type;
		}
		
		if(column2Type != null){			
			headerValue2 = tableModel.COLUMN2_PREFIX + " : " + column2Type;		
		}
		
		if(column3Type != null){		
			headerValue3 = tableModel.COLUMN3_PREFIX + " : " + column3Type;	
		}
		
		 String columnsNames[] = {headerValue1, headerValue2, headerValue3};
		tableModel.setNewIdentifiers(columnsNames);
	}
	
	private List<TierImpl> getLinkedTiersOfType(TierImpl refTier, String type){
    	List childTiers = refTier.getChildTiers();
    	List<TierImpl> linkedTiers = new ArrayList<TierImpl>();
    	for(int i=0; i < childTiers.size(); i++){
			TierImpl childTier = (TierImpl) childTiers.get(i);
			if(childTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
				if(childTier.getLinguisticType().getLinguisticTypeName().equals(type)){	
					linkedTiers.add(childTier);
				}
				// get all the types of the tiers depending on this child tier
				List dependentTiers = childTier.getDependentTiers();
				for(int y=0; y < dependentTiers.size(); y++) {
					TierImpl dependantTier = (TierImpl) dependentTiers.get(y);	
					if(dependantTier.getLinguisticType().getLinguisticTypeName().equals(type)){								
						linkedTiers.add(dependantTier);	
					}
				}
			}
		}
    	return linkedTiers;	   
    }
	
	/**
	 * load the table with values
	 */
	public void loadTable(){
		table.getSelectionModel().removeListSelectionListener(this);
		table.clearRows();		
		
		List tiers  = new ArrayList();
		List <TierImpl> parentTierListType = new ArrayList<TierImpl>();	 	
		List annotationsList = new ArrayList();
		if(column1Type != null){
			tiers = viewerManager.getTranscription().getTiersWithLinguisticType(column1Type);	
		} else{
			return;
		}
		
		if(tiers != null && tiers.size() > 0){
			TierImpl tierC1 = (TierImpl)tiers.get(0);	
			
			// if columnType1 is symbolic Associatedtype
	 		if(tierC1.getLinguisticType().getConstraints() != null && tierC1.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){ 
	 			for(int x = 0; x < tiers.size(); x++){						
					TierImpl tier = (TierImpl)tiers.get(x);	
					TierImpl parentTier = (TierImpl) tier.getParentTier();
					while(parentTier != null && (parentTier.getLinguisticType().getConstraints() != null && parentTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
						parentTier = (TierImpl) parentTier.getParentTier();
					}
					
					if(parentTierListType.contains(parentTier)){
						continue;
					} 		
					
					List<TierImpl> matchedTiersType1 = new ArrayList<TierImpl>(); 	
	 				List<TierImpl> matchedTiersType2 = new ArrayList<TierImpl>(); 	
	 				List<TierImpl> matchedTiersType3 = new ArrayList<TierImpl>(); 	
					
					parentTierListType.add(parentTier);
					matchedTiersType1.addAll(getLinkedTiersOfType(parentTier, column1Type));
					
					TierImpl obj1 = tierMapColumn1.get(parentTier);
					TierImpl obj2 = tierMapColumn2.get(parentTier);
					TierImpl obj3 = tierMapColumn3.get(parentTier);
					
					if(obj1 != null){
						if(matchedTiersType1.contains(obj1)){
							matchedTiersType1.remove(obj1);
						} else {
							obj1 = null;
						}
					}
				
					if(obj2 != null && column2Type != null ){
						if(column2Type.equals(column1Type) && matchedTiersType1.contains(obj2)){
							matchedTiersType1.remove(obj2);
						} else {
							obj2 = null;
						}
					} else{
						obj2 = null;
					}
					
					if(obj3 != null && column3Type != null){
						if( column3Type.equals(column1Type)  && matchedTiersType1.contains(obj3)){
							matchedTiersType1.remove(obj3);
						}else {
							obj3 = null;
						}
					}else{
						obj3 = null;
					}
					
					if(obj1 == null){			 							
						if(matchedTiersType1.size() > 0){
							obj1 =  matchedTiersType1.get(0);							
							matchedTiersType1.remove(0);
						} else if(obj2 != null){
							obj1 = obj2;
							obj2 = null;							
						}else if(obj3 != null){
							obj1 = obj3;
							obj3 = null;	
						}else{
							obj1 = null;
						}
					}
					tierMapColumn1.put(parentTier, obj1);
					
					//if first column is not empty
					if(obj1 != null){					
						if(obj2 == null && column2Type != null){	
							if(column1Type.equals(column2Type)){					
								if(matchedTiersType1.size() > 0){
									obj2 = matchedTiersType1.get(0);
									matchedTiersType1.remove(0);
								}else {	 
									obj2 = null;								
								}
							} else {
								matchedTiersType2.addAll(getLinkedTiersOfType(parentTier, column2Type));
								
								obj2 = tierMapColumn2.get(parentTier);
								obj3 = tierMapColumn2.get(parentTier);
								
								if(obj2 != null){
									if(matchedTiersType2.contains(obj2)){
										matchedTiersType2.remove(obj2);
									} else {
										obj2 = null;
									}
								}
								
								if(obj3 != null && column3Type != null){
									if( column3Type.equals(column2Type)  && matchedTiersType2.contains(obj3)){
										matchedTiersType2.remove(obj3);
									} else {
										obj3 = null;
									}
								} else {
									obj3 = null;
								}
													
								if(obj2 == null){
									if(matchedTiersType2.size() > 0){
										obj2 = matchedTiersType2.get(0);											
										matchedTiersType2.remove(0);
									} else if(obj3 != null){
										obj2 = obj3;
										obj3 = null;							
									} else {	
										obj2 = null;																	
									}
								}								
							}	
						} 
						
						if(column2Type != null){
							tierMapColumn2.put(parentTier, obj2);	
							
							///type 3
							if( obj3 == null && column3Type != null){								
								if(column3Type.equals(column1Type)){					
									if(matchedTiersType1.size() > 0){
										obj3 = matchedTiersType1.get(0);
										matchedTiersType1.remove(0);
									}else {	 						
										obj3 = null;
									}
								} else if (column3Type.equals(column2Type)){														
									if(matchedTiersType2.size() > 0){
										obj3 = matchedTiersType2.get(0);
										matchedTiersType2.remove(0);
									}else {	 						
										obj3 = null;
									}
								} else{
									matchedTiersType3.addAll(getLinkedTiersOfType(parentTier, column3Type));
										
									obj3 = tierMapColumn3.get(tier.getParentTier());
										
									if(obj3 != null){
										if(matchedTiersType3.contains(obj3)){												
											matchedTiersType3.remove(0);
										} else {	 						
											obj3 = null;												
										}
									}
											
									if(obj3 == null){
										if(matchedTiersType3.size() > 0){
											obj3 = matchedTiersType3.get(0);
											matchedTiersType3.remove(0);
										} else {	 						
											obj3 = null;										
										}
									}
								}
							}
								
							if(column3Type !=null){
								tierMapColumn3.put((TierImpl) tier.getParentTier(), obj3);
							} else{
								tierMapColumn3.put((TierImpl) tier.getParentTier(), null);
							}						 
						} else{
							tierMapColumn2.put((TierImpl) tier.getParentTier(), null);
							tierMapColumn3.put((TierImpl) tier.getParentTier(), null);
						}
					} else{
						tierMapColumn2.put((TierImpl) tier.getParentTier(), null);
						tierMapColumn3.put((TierImpl) tier.getParentTier(), null);
					}					
	 			}
	 		} else {
	 			List<TierImpl> parentTiers = new ArrayList<TierImpl>();
	 			for(int x = 0; x < tiers.size(); x++){
	 				TierImpl tier = (TierImpl)tiers.get(x);
	 				parentTierListType.add(tier);
	 				tierMapColumn1.put(tier, tier);	 					

	 				List<TierImpl> matchedTiersType2 = new ArrayList<TierImpl>(); 	
	 				List<TierImpl> matchedTiersType3 = new ArrayList<TierImpl>(); 	
	 				if(column2Type != null){		
	 					matchedTiersType2.addAll(this.getLinkedTiersOfType(tier, column2Type));
	 					
	 					TierImpl obj2 = tierMapColumn2.get(tier);
						TierImpl obj3 = tierMapColumn3.get(tier);
						
						if(obj2 != null){
							if(matchedTiersType2.contains(obj2)){
								matchedTiersType2.remove(obj2);
							} else {  
								obj2 = null;
							}
						}
						
						if(obj3 != null){
							if(column3Type != null && matchedTiersType2.contains(obj3) && column2Type.equals(column3Type)){
								matchedTiersType2.remove(obj3);
							} else {  
								obj3 = null;
							}
						}
						
						if(obj2 == null){
							if(matchedTiersType2.size() > 0){
								obj2 = matchedTiersType2.get(0);											
								matchedTiersType2.remove(0);
							} else if(obj3 != null){
								obj2 = obj3;
								obj3 = null;							
							} else {	
								obj2 = null;																	
							}
						}
						
						tierMapColumn2.put(tier, obj2);
						
						
						if(obj3 == null && column3Type != null){
							if(column2Type.equals(column3Type)){
								if(matchedTiersType2.size() > 0){
									obj3 = matchedTiersType2.get(0);
									matchedTiersType2.remove(0);
								}else {	 						
									obj3 = null;
								}
							} else{
								matchedTiersType3.addAll(this.getLinkedTiersOfType(tier, column3Type));
								obj3 = tierMapColumn3.get(tier);
									
								if(obj3 != null){
									if(matchedTiersType3.contains(obj3)){
										matchedTiersType3.remove(obj3);
									} else {  
										obj3 = null;
									}
								}
									
								if(obj3 == null){
									if(matchedTiersType3.size() > 0){
										obj3 = matchedTiersType3.get(0);
										matchedTiersType3.remove(0);
									}else {	 						
										obj3 = null;
									}
								} 
							}
						} 
						
						if(column3Type !=null){
							tierMapColumn3.put( tier, obj3);
						} else{
							tierMapColumn3.put(tier, null);
						}
						
	 				} else{
	 					tierMapColumn2.put(tier, null);
	 					tierMapColumn3.put(tier, null);
	 				}
		 		}
		 	}
		}
		
		for(int x = 0; x < parentTierListType.size(); x++){
			TierImpl tier = (TierImpl)parentTierListType.get(x);	
			if(tier.getLinguisticType().getConstraints()== null || tier.getLinguisticType().getConstraints().getStereoType() != Constraint.SYMBOLIC_ASSOCIATION){
				if(tierMapColumn1.get(tier) != null){
					annotationsList.addAll(tier.getAnnotations());
				}
			}
	 	}
		
	 	if(column1Type != null && annotationsList.size() ==0 ){
	 		String message = ElanLocale.getString("TranscriptionManager.Message.NoSegments");
			JOptionPane.showMessageDialog(table, message,
			            ElanLocale.getString("Message.Warning"), JOptionPane.WARNING_MESSAGE);	
			return;
	 	}
	 	
		AnnotationComparator annComparator = new AnnotationComparator();
		Collections.sort(annotationsList, annComparator);
		
		HashMap annotationMap1 = new HashMap();
		HashMap annotationMap2 = new HashMap();
		HashMap annotationMap3 = new HashMap();
		
		for(int i=0; i < annotationsList.size(); i++){	
			AbstractAnnotation ann = (AbstractAnnotation) annotationsList.get(i);			
			TierImpl parentTier = (TierImpl) ann.getTier();
			TierImpl tier1 = (TierImpl) tierMapColumn1.get(parentTier);
			TierImpl tier2 = (TierImpl) tierMapColumn2.get(parentTier);
			TierImpl tier3 = (TierImpl) tierMapColumn3.get(parentTier);	
			
			annotationMap1.put(ann, getLinkedAnnotation(ann, tier1));
			annotationMap2.put(ann, getLinkedAnnotation(ann, tier2));			
			annotationMap3.put(ann, getLinkedAnnotation(ann, tier3));	
		}
			
		int column1MaxWidth = table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX)).getMaxWidth();
		int column2MaxWidth = table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).getMaxWidth();
		int column3MaxWidth = table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).getMaxWidth();	
	    
		updateColumnHeaders();		
		table.setStoreColumnOrder(false);
		table.getColumnModel().moveColumn(table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX)), 1);
		table.getColumnModel().moveColumn(table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)), 2);
		table.getColumnModel().moveColumn(table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)), 3);	
			
		int n = 1;
		String tier1Name =  null;
		String tier2Name =  null;
		String tier3Name =  null;
		
		String tierName =  null;
		
		// loads all the annotations in the table
		TranscriptionTableModel tableModel = (TranscriptionTableModel) table.getModel();		
		for(int i=0; i < annotationsList.size(); i++){	
			Annotation ann = (Annotation) annotationsList.get(i);
			Object obj1 = annotationMap1.get(ann);	
			Object obj2 = annotationMap2.get(ann);		
			Object obj3 = annotationMap3.get(ann);	
			
			TierImpl tier1 = (TierImpl) tierMapColumn1.get(ann.getTier());
			TierImpl tier2 = (TierImpl) tierMapColumn2.get(ann.getTier());
			TierImpl tier3 = (TierImpl) tierMapColumn3.get(ann.getTier());	
			
			if(tier1 != null){				
				tier1Name = tier1.getName();		
				checkColorForTier(tier1Name);				
			} else{
				tier1Name = null;
			}
			
			if(tier2 != null){				
				tier2Name = tier2.getName();		
				checkColorForTier(tier2Name);			
			} else{
				tier2Name = null;
			}
			
			if(tier3 != null){				
				tier3Name = tier3.getName();		
				checkColorForTier(tier3Name);			
			} else{
				tier3Name = null;
			}
			
			if(showTierNames){
				if(i == 0){
					tierName = ann.getTier().getName();
					tableModel.addRow( new Object[]{new TableSubHeaderObject(""), new TableSubHeaderObject(tier1Name), 
							new TableSubHeaderObject(tier2Name),new TableSubHeaderObject(tier3Name)});	
				}else if(!tierName.equals(ann.getTier().getName())){	
					tierName = ann.getTier().getName();
					tableModel.addRow( new Object[]{new TableSubHeaderObject(""), new TableSubHeaderObject(tier1Name), 
							new TableSubHeaderObject(tier2Name),new TableSubHeaderObject(tier3Name)});	
				}
			}
		    tableModel.addRow(new Object[]{n, obj1 , obj2, obj3 });
		    n++;
		}
		
		// sets the width of the column "No" according to the number of annotations
		if(n > 999){
			table.getColumnModel().getColumn(0).setMaxWidth(50);	 
			table.getColumnModel().getColumn(0).setMinWidth(50);
		}else if( n > 99){
			table.getColumnModel().getColumn(0).setMaxWidth(45);	
			table.getColumnModel().getColumn(0).setMinWidth(45);
		} else if(n < 100){
			table.getColumnModel().getColumn(0).setMinWidth(40);
			table.getColumnModel().getColumn(0).setMaxWidth(40);
		}

		if(columnOrder.size() > 0){
			table.getColumnModel().moveColumn(table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX)), columnOrder.get(TranscriptionTableModel.COLUMN1_PREFIX));
			table.getColumnModel().moveColumn(table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)), columnOrder.get(TranscriptionTableModel.COLUMN2_PREFIX));
			table.getColumnModel().moveColumn(table.getColumnModel().getColumnIndex(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)), columnOrder.get(TranscriptionTableModel.COLUMN3_PREFIX));
		}
		
		if(column2MaxWidth == 0){
			table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMinWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setPreferredWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMaxWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setResizable(false);	
		    
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMinWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setPreferredWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMaxWidth(0);		
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setResizable(false);	
			
		}else if(column3MaxWidth == 0){		    
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMinWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setPreferredWidth(0);
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMaxWidth(0);		
		    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setResizable(false);				
		}

		table.reCalculateRowHeight();		
		table.revalidate();
		table.requestFocusInWindow();
		table.getSelectionModel().addListSelectionListener(this);
		
		table.setStoreColumnOrder(true);
	} 
	
	private Object getLinkedAnnotation(AbstractAnnotation refAnnotation, TierImpl tier){
		Annotation annotation = null;
		if(tier != null){
			if(tier.hasParentTier() && tier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){		
				TierImpl refTier = (TierImpl) refAnnotation.getTier();		
				List childAnnotations = refAnnotation.getParentListeners();	
				String create_Ann =  CREATE_ANN + ";" + tier.getName()+";"+ refAnnotation.getBeginTimeBoundary() + " : " + refAnnotation.getEndTimeBoundary();	
				TierImpl parentTier = (TierImpl) tier.getParentTier();
				if(parentTier != refTier){
					AbstractAnnotation parentAnn = (AbstractAnnotation) parentTier.getAnnotationAtTime(refAnnotation.getBeginTimeBoundary());
					if(parentAnn != null){
						childAnnotations = parentAnn.getParentListeners();	
						for(int j=0; j< childAnnotations.size();j++){
							if(((Annotation)childAnnotations.get(j)).getTier() == tier){
								annotation = (Annotation)childAnnotations.get(j);							
								break;
							}
						}						
					} else {					
						return null;
					}
				} else {
					for(int j=0; j< childAnnotations.size();j++){
						if(((Annotation)childAnnotations.get(j)).getTier() == tier){
							annotation = (Annotation)childAnnotations.get(j);							
							break;
						}
					}
				}				
				
				if(annotation == null){
					return create_Ann;
				}else {
					return annotation;					
				}
			} else {
				return refAnnotation;
			}
		} else {
			return null;
		}
	}
	/**
	 * Checks if there is a preferred color for the give tier.
	 * Else calls the setColorForTier method
	 * 
	 * @param tierName, tier to be checked
	 */	
	private void checkColorForTier(String tierName){
		Color c = table.getFontColorForTier(tierName);
		if(c == null){
			TierImpl tier = (TierImpl) viewerManager.getTranscription().getTierWithId(tierName);
			if(tier.hasParentTier()){
				TierImpl parentTier = (TierImpl) tier.getParentTier();
				c = table.getFontColorForTier(parentTier.getName());
				if( c==null ){
					setColorForTier(tierName);
				}else {
					HashMap map = new HashMap();
					map.put(tierName, c);
					table.setFontColorForTiers(map);
				}
			} else {
				setColorForTier(tierName);
			}
		} else {
			tierColorsList.add(c);
		}
	}
	
	/**
	 * Sets a random color for the given Tier
	 * 
	 * @param tierName
	 */
	private void setColorForTier(String tierName) {		
		while(true){
			int r = (int)(Math.random()*255);
			int g = (int)(Math.random()*255);
			int b = (int)(Math.random()*255);
			 
			Color c = new Color(r,g,b);
			//new Color(230,230,250) --- color used by renderer
			// to indicate the cells where an annotation can be created
			if(c == Color.BLACK || c == Color.WHITE ||
					c == new Color(238,238,238) || c == new Color(230,230,250)){
				continue;
			}
			
			if(!tierColorsList.contains(c)){
				tierColorsList.add(c);
				HashMap map = new HashMap();
				map.put(tierName, c);
				table.setFontColorForTiers(map);
				break;
			}
		}
	}
	
	public void updateSignalViewer(SignalViewer viewer){
		signalViewer = viewer;
		if(table.isEditing()){
			signalViewer.setEnabled(true);
			Object obj = table.getValueAt(table.getCurrentRow(), table.getCurrentColumn());
			if(obj instanceof Annotation){
				long begin = ((Annotation)obj).getBeginTimeBoundary();
				long end = ((Annotation)obj).getEndTimeBoundary();	
				updateMedia(begin, end);
			}
		}
	}
	
	/**
	 * Updates the signal viewer, the players and the 
	 *
	 * 
	 * @param begin
	 * @param end
	 */
	public void updateMedia(long begin, long end){	
		
		clearSelection();
		
		if(viewerManager.getMasterMediaPlayer() !=null){ 
			if(viewerManager.getMediaPlayerController().getLoopMode()){
				viewerManager.getMediaPlayerController().stopLoop();
			}			
			
			if( viewerManager.getMasterMediaPlayer().isPlaying()){
				viewerManager.getMasterMediaPlayer().stop();
			}
			viewerManager.getMasterMediaPlayer().setMediaTime(begin);
		}	
		
		// updates the signal viewer to show the wave signal for the given time interval
		if(signalViewer != null){
			signalViewer.updateInterval(begin,end);
		}	
		
		((SelectionPanel) viewerManager.getMediaPlayerController().getSelectionPanel()).setBegin(begin);
		((SelectionPanel) viewerManager.getMediaPlayerController().getSelectionPanel()).setEnd(end);
	}	

	/**
	 * Start playing the media
	 */
	public void playMedia(boolean playSelection) {	
		if(table.isEditing() ){
			table.getEditorComponent().requestFocusInWindow();
			Annotation ann = (Annotation) table.getValueAt(table.getEditingRow(),table.getEditingColumn());	
			//updateMedia(ann.getBeginTimeBoundary(),ann.getEndTimeBoundary());		
			playInterval(ann.getBeginTimeBoundary(),ann.getEndTimeBoundary(), playSelection);		
					
		}else{
			//table.startEdit();
			if(table.getSelectedRow() > -1 && table.getSelectedColumn() > -1){
				Annotation ann = (Annotation) table.getValueAt(table.getSelectedRow(),table.getSelectedColumn());
				updateMedia(ann.getBeginTimeBoundary(),ann.getEndTimeBoundary());		
				playInterval(ann.getBeginTimeBoundary(),ann.getEndTimeBoundary(), playSelection);	
			}
			table.requestFocusInWindow();
		}
	}
	
	public void playIntervalFromBeginTime(long beginTime, long endTime){
		if(!layoutManager.isInitialized()){
			return;
		}		
			
		if (viewerManager.getMasterMediaPlayer() == null) {
	       return;
	    }
		
		 viewerManager.getMediaPlayerController().setPlaySelectionMode(true);
    	 viewerManager.getMasterMediaPlayer().playInterval(beginTime, endTime);
		
	}
	
	/**
	 * Plays the media for the given interval
	 * 
	 * @param beginTime, the start time of the media
	 * @param endTime, the end time of the media
	 */
	public void playInterval(long beginTime, long endTime, boolean playSelection){	
		
		if(!layoutManager.isInitialized()){
			return;
		}		
			
		if (viewerManager.getMasterMediaPlayer() == null) {
	       return;
	    }
		
	    //stop if a selection is being played
	    if (viewerManager.getMasterMediaPlayer().isPlaying()) {
	    	viewerManager.getMasterMediaPlayer().stop();	    	
	           
	      	// if playing a selection has been stopped before the end of the selection this 
	        // makes the player jump to the end of the file when the old stoptime is reached
	        // with Windows media framework
	        if (viewerManager.getMasterMediaPlayer() instanceof NativeMediaPlayerWindowsDS) {
	          	// workaround
	        	long t = viewerManager.getMasterMediaPlayer().getMediaTime();
	        	viewerManager.getMasterMediaPlayer().setMediaTime(beginTime);
	        	viewerManager.getMasterMediaPlayer().setStopTime(endTime);
	        	viewerManager.getMasterMediaPlayer().setMediaTime(t);
	        } else {
	        	viewerManager.getMasterMediaPlayer().setStopTime(endTime);
	        }
	        return;
	     }	    
	    
	     long mediaTime = viewerManager.getMasterMediaPlayer().getMediaTime();	 
	     
	     //play selection
	     if(playSelection){
	    	 Selection s = viewerManager.getSelection();		    
	    	 if( s != null && s.getBeginTime() != s.getEndTime() ){
	    		 if(s.getBeginTime() < s.getEndTime() && s.getBeginTime() >= beginTime  && s.getEndTime() <= endTime){
	    			 if(mediaTime > s.getBeginTime() && mediaTime< s.getEndTime()){
	    				 viewerManager.getMasterMediaPlayer().playInterval(mediaTime, s.getEndTime());
	    			 } else {
	    				 viewerManager.getMasterMediaPlayer().playInterval(s.getBeginTime(), s.getEndTime());
	    			 }
	    			 return;
	    		 } 
	    	 }
	     }
	     
	     //if not playing, start playing
	     if ((viewerManager.getMasterMediaPlayer().isPlaying() == false) && (mediaTime >= beginTime) &&
	                (mediaTime < endTime - 5)) {	    	 
	    	 viewerManager.getMediaPlayerController().setPlaySelectionMode(true);
	    	 viewerManager.getMasterMediaPlayer().playInterval(mediaTime, endTime);
	    	 if (viewerManager.getMediaPlayerController().getLoopMode()) {	            
	    		delayedStartLoop(beginTime, endTime);
		     }
	    	 return;
	      }
	     
	     if (viewerManager.getMediaPlayerController().getLoopMode()) {	            
	    	 viewerManager.getMediaPlayerController().startLoop(beginTime, endTime);
	     } else {	        	
	    	 viewerManager.getMasterMediaPlayer().playInterval(beginTime, endTime);
	     }
	}	
	
	public void clearSelection(){
		Selection sel = viewerManager.getSelection();
		if(sel != null && (sel.getBeginTime() != 0 || sel.getEndTime() != 0)){			
			sel.clear();
			if (signalViewer != null) {
				signalViewer.updateSelection();				
			}
		}
	}

	
	/**
     * Starts the loop after a first partial play selection has finished
     */
    private void delayedStartLoop(long beginTime, long endTime) {
        LoopThread loopthread = new LoopThread(beginTime, endTime);
        loopthread.start();
    }
	
	/**
	 * Merge the given annotation with the annotation
	 * before it
	 * 
	 * @param annotation, the annotation to be merged
	 */
	void mergeBeforeAnn(Annotation annotation){
		if(viewerManager.getMasterMediaPlayer()!= null && viewerManager.getMasterMediaPlayer().isPlaying()){
			viewerManager.getMasterMediaPlayer().stop();
		}
		
		setPreference("TranscriptionTable.LastActiveRow", table.getCurrentRow()-1, viewerManager.getTranscription());
		setPreference("TranscriptionTable.LastActiveColumn", table.getColumnName(table.getCurrentColumn()), viewerManager.getTranscription());
   		
   		Transcription transcription = (Transcription)annotation.getTier().getParent();   		
    	Command c = ELANCommandFactory.createCommand(transcription,ELANCommandFactory.MERGE_ANNOTATION_WB);    	
    	Object[] args = new Object[] { annotation, false };
    	c.execute(transcription, args);
	}
   		
	/**
	 * Merge the given annotation with the annotation
	 * next to it
	 * 
	 * @param annotation, the annotation to be merged
	 */	
	void mergeNextAnn(Annotation annotation){
		if(viewerManager.getMasterMediaPlayer()!= null && viewerManager.getMasterMediaPlayer().isPlaying()){
			viewerManager.getMasterMediaPlayer().stop();
		}	
		setPreference("TranscriptionTable.LastActiveRow", table.getCurrentRow(), viewerManager.getTranscription());
		setPreference("TranscriptionTable.LastActiveColumn", table.getColumnName(table.getCurrentColumn()), viewerManager.getTranscription());
		
   		Command c = ELANCommandFactory.createCommand(viewerManager.getTranscription(),ELANCommandFactory.MERGE_ANNOTATION_WN);    	
    	Object[] args = new Object[] { annotation, true };
    	c.execute(viewerManager.getTranscription(), args); 
	}
	
	/**
	 * Deletes the given annotation
	 *
	 * @param annotation, the annotation to be deleted
	 */	
	void deleteAnnotation(Annotation annotation){
		
		if(viewerManager.getMasterMediaPlayer()!= null && viewerManager.getMasterMediaPlayer().isPlaying()){
			viewerManager.getMasterMediaPlayer().stop();
		}
		
		int leadRow = table.getSelectionModel().getLeadSelectionIndex();
   		int leadColumn = table.getColumnModel().getSelectionModel().
	                   getLeadSelectionIndex();
   		
   		if(table.getValueAt(leadRow, leadColumn) instanceof TableSubHeaderObject){
   			return;
   		}
   		
   		Transcription transcription = (Transcription)annotation.getTier().getParent();  
   		Command c = ELANCommandFactory.createCommand(transcription,ELANCommandFactory.DELETE_ANNOTATION);    	
    	Object[] args = new Object[] { viewerManager, annotation};
    	c.execute((TierImpl) annotation.getTier(), args);  
	}
	
	/**
	 * Tries to select the last active cell
	 */
	public void updateTable(){
		Integer lastActiveRow = (Integer) Preferences.get("TranscriptionTable.LastActiveRow", viewerManager.getTranscription());
		String lastActiveColumnName = (String) Preferences.get("TranscriptionTable.LastActiveColumn", viewerManager.getTranscription());
		int lastActiveColumn = -1;

		
		if(lastActiveRow != null && lastActiveColumnName !=null){
			if(table.getColumnName(1).equals(lastActiveColumnName) || table.getColumnName(2).equals(lastActiveColumnName) ||table.getColumnName(3).equals(lastActiveColumnName)){			
				lastActiveColumn = table.getColumnModel().getColumnIndex(lastActiveColumnName);	
			}
			if(lastActiveRow > -1 && lastActiveColumn > -1){
				table.changeSelection(lastActiveRow, lastActiveColumn, false, false);
				if(table.editCellAt(lastActiveRow, lastActiveColumn)){
					boolean playBack = isAutoPlayBack();	
					setAutoPlayBack(false);
					table.startEdit(null);
					
					JViewport viewport = (JViewport)table.getParent();					
					Rectangle rect = table.getCellRect(lastActiveRow, lastActiveColumn, true);		   
					Point pt = viewport.getViewPosition();					
					rect.setLocation(rect.x-pt.x, rect.y-pt.y);
					viewport.scrollRectToVisible(rect);
					setAutoPlayBack(playBack);
					//viewerManager.viewerManager.getMasterMediaPlayer().stop();
				}
			}
		}	
	}
	
	/**
	 * Activates the third column for the extra tier 
	 */
	public void activateThirdColumn() {	
		System.out.println("third column active");
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMinWidth(15);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setPreferredWidth(75);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMaxWidth(Integer.MAX_VALUE);	
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setResizable(true);	
	    table.activateThirdColumn(true);	
	    
	    int width = (table.getWidth() - table.getColumnModel().getColumn(0).getWidth())/3;
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX)).setWidth(width);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setWidth(width);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setWidth(width);
	    
	    layoutManager.setPreference("TranscriptionTable.ThirdColumnActive", true, viewerManager.getTranscription());
	}
	
	/**
	 *  Deactivates the third column for the extra tier 
	 */
	public void deActivateThirdColumn() {		
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMinWidth(0);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setPreferredWidth(0);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setMaxWidth(0);	
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN3_PREFIX)).setResizable(false);	
		table.activateThirdColumn(false);	
		
		 layoutManager.setPreference("TranscriptionTable.ThirdColumnActive", false, viewerManager.getTranscription());
	}
	/**
	 *  Activates the second column 
	 */
	public void activateSecondColumn() {
		
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMinWidth(15);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setPreferredWidth(75);	    
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMaxWidth(Integer.MAX_VALUE);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setResizable(true);	
	    
	    int width = table.getColumnModel().getColumn(1).getWidth() / 2;
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN1_PREFIX)).setWidth(width);
	    table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setWidth(width);
	    
	    secondColumnActivated = true;
	}

	/**
	 *  Deactivates the second column 
	 */
	public void deActivateChildTierColumn() {
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMinWidth(0);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setPreferredWidth(0);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setMaxWidth(0);
		table.getColumn(tableModel.getColumnNameForPrefix(tableModel.COLUMN2_PREFIX)).setResizable(false);
	    secondColumnActivated = false;
	    deActivateThirdColumn();		
	}	

	public void valueChanged(ListSelectionEvent e) {
		int currentRow = table.getSelectionModel().getLeadSelectionIndex();
		int currentColumn = table.getColumnModel().getSelectionModel().getLeadSelectionIndex();

		setPreference("TranscriptionTable.LastActiveRow", currentRow, viewerManager.getTranscription());
		setPreference("TranscriptionTable.LastActiveColumn", table.getColumnName(currentColumn), viewerManager.getTranscription());
			
		if(currentRow > -1 && currentColumn > -1){
			Object obj = null;
			if(table.getRowCount() > currentRow){
				obj =  table.getValueAt(currentRow, currentColumn);
			}
			
			if(obj instanceof Annotation || obj instanceof String ){
				layoutManager.enablePlayButton(true);	
				if(signalViewer  != null){
					//SignalViewer viewer = signalViewer;
					signalViewer.setEnabled(true);					
				}
			} else{				
				layoutManager.enablePlayButton(false);	
				if(signalViewer  != null){
					signalViewer.setEnabled(false);
				}
				updateMedia(0L,0L);
			}
		}
	}
	
	public void reValidateTable(){
		table.repaint();
	}
	
	public void loadPreferences(){
		HashMap<String, TierImpl> map1 = (HashMap<String, TierImpl>) Preferences.get("TranscriptionTable.TierMap1", viewerManager.getTranscription());
		HashMap<String, TierImpl> map2 = (HashMap<String, TierImpl>) Preferences.get("TranscriptionTable.TierMap2", viewerManager.getTranscription());
		HashMap<String, TierImpl> map3 = (HashMap<String, TierImpl>) Preferences.get("TranscriptionTable.TierMap3", viewerManager.getTranscription());
		HashMap<TierImpl, TierImpl> newmap = changeToUsableMap(map1);
		if(newmap != null){
			tierMapColumn1.putAll(newmap);
		}
		newmap = changeToUsableMap(map2);
		if(newmap != null){
			tierMapColumn2.putAll(newmap);
		}
		newmap = changeToUsableMap(map3);
		if(newmap != null){
			tierMapColumn3.putAll(newmap);
		}
		
		HashMap<String, Integer> newColumnOrder = (HashMap<String, Integer>) Preferences.get("TranscriptionTable.ColumnOrder", viewerManager.getTranscription());
		if(newColumnOrder != null){
			columnOrder = newColumnOrder;
		}
	}
	
	public void storePreferences() {		
		layoutManager.setPreference("TranscriptionMode.Temp.TierColors", table.getFontColorTierMap(), viewerManager.getTranscription());
	}
	
	public void controllerUpdate(ControllerEvent event) {
		 if (event instanceof StopEvent) {
			 layoutManager.setPlayPauseButton(true);
	     }

	     if (event instanceof StartEvent) {
	    	 layoutManager.setPlayPauseButton(false);
	    }
	}
	
	public void ACMEdited(ACMEditEvent e) {	
		switch (e.getOperation()){				
			case ACMEditEvent.ADD_ANNOTATION_HERE:
				Annotation ann = (Annotation) e.getModification();
				TierImpl tierObject = (TierImpl) ann.getTier();
				TierImpl refTier = (TierImpl) tierObject.getParentTier();				
				while(refTier != null && (refTier.getLinguisticType().getConstraints() != null && refTier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)){
					refTier = (TierImpl) refTier.getParentTier();
				}
				
				TierImpl linkedTier = tierMapColumn1.get(refTier);
								
				int count = 1;
				while (linkedTier !=null){
					if(linkedTier != tierObject){
						if(linkedTier.getParentTier() == tierObject){
							int currentRow = table.getCurrentRow();
							int currentColumn = table.getCurrentColumn();
							loadTable();							
							table.changeSelection(currentRow, currentColumn, false, false);				
							break;
						}else {
							count++;
						}
					} else {
						count++;
					}
					
					if(count == 2){
						linkedTier = tierMapColumn2.get(refTier);
						continue;
					}
					
					if(count == 3){
						linkedTier = tierMapColumn3.get(refTier);
						continue;
					}					
					break;
				}
				break;
				
			case ACMEditEvent.CHANGE_ANNOTATIONS:
				loadTable();	
				updateTable();
				break;
				
			case ACMEditEvent.REMOVE_ANNOTATION:
				loadTable();
				break;
				
			case ACMEditEvent.ADD_TIER:	
				Object obj  = e.getModification();
				if(obj instanceof TierImpl){
					TierImpl tier = (TierImpl) obj;
					String type = tier.getLinguisticType().getLinguisticTypeName();
					if((column1Type != null && column1Type.equals(type))  || 
							(column2Type != null && column2Type.equals(type))  ||
							(column3Type != null && column3Type.equals(type)) ){
						loadTable();
					}	
				}	
				break;	
			
			case ACMEditEvent.REMOVE_TIER:	
				obj  = e.getModification();
				if(obj instanceof TierImpl){
					TierImpl tier = (TierImpl) obj;
					String type = tier.getLinguisticType().getLinguisticTypeName();
					if( tierMapColumn1.containsKey(tier) || tierMapColumn1.containsValue(tier) ||  
							tierMapColumn2.containsKey(tier) || tierMapColumn2.containsValue(tier) ||
							tierMapColumn3.containsKey(tier) || tierMapColumn3.containsValue(tier) ){
						loadTable();
					}
				}	
				break;	

			case ACMEditEvent.CHANGE_TIER:			
				List tiers = viewerManager.getTranscription().getTiers();
				obj  = e.getSource();			
				if(obj instanceof TierImpl){
					TierImpl tier = (TierImpl) obj;
					if(tiers.contains(tier)){
						setPreferredFontAndColorSettings();
						String type = tier.getLinguisticType().getLinguisticTypeName();
						if( tierMapColumn1.containsKey(tier) || tierMapColumn1.containsValue(tier) ||  
									tierMapColumn2.containsKey(tier) || tierMapColumn2.containsValue(tier) ||
									tierMapColumn3.containsKey(tier) || tierMapColumn3.containsValue(tier) ){
								loadTable();
							}
						}
				}
				break;
		}
							
	}
	
	/**
	 * Updates the selection.
	 */
	public void updateSelection() {
		Selection s = viewerManager.getSelection();
		
		if(table != null){
			if(table.isEditing()){	
				table.getEditorComponent().requestFocusInWindow();
				
				Annotation ann = (Annotation) table.getValueAt(table.getEditingRow(), table.getEditingColumn());
				long beginTime = ann.getBeginTimeBoundary();
				long endTime = ann.getEndTimeBoundary();				
		    
				if( s != null && s.getBeginTime() != s.getEndTime() ){
					if(s.getBeginTime() >= beginTime  && s.getEndTime() <= endTime){
						return;
					} 			
					if(s.getBeginTime() >= beginTime){
						if(signalViewer  != null){		
							signalViewer .setSelection(s.getBeginTime(), endTime);
							signalViewer .repaint();
						}
						return;
					} else if(s.getEndTime() <= endTime){
						if(signalViewer  != null){		
							signalViewer .setSelection(beginTime, s.getEndTime());
							signalViewer .repaint();
						}
						return;
					}
				}
			} else {
				table.requestFocusInWindow();
			}
		} 
	}

	
	public void updateActiveAnnotation() {		
	}
	
	public void editInAnnotationMode(){
		layoutManager.editInAnnotationMode();
	}

	public void setActiveAnnotation() {
		if(table.getCurrentRow() > 0 && table.getCurrentColumn() > 0){
			Object val = table.getValueAt(table.getCurrentRow(), table.getCurrentColumn());
			if(val instanceof Annotation){
				TierImpl tier = (TierImpl) ((Annotation)val).getTier();
				if(tier.getLinguisticType().getConstraints() !=null && tier.getLinguisticType().getConstraints().getStereoType() == Constraint.SYMBOLIC_ASSOCIATION){
					Annotation parentAnn = ((Annotation)val).getParentAnnotation();
					if(parentAnn != null){
						setActiveAnnotation(parentAnn);
					}else {
						setActiveAnnotation((Annotation)val);
					}
				} else {
					setActiveAnnotation((Annotation)val);
				}
			}
		}		
	}

	public void updateLocale() {
	}
	
	public void preferencesChanged() {			
	}	

    /**
     * Calls the media player controllers startloop method after a first, partial selection playback has finished.
     */
    private class LoopThread extends Thread {
    	long beginTime;
    	long endTime;
    	
        public LoopThread(long beginTime, long endTime) {
			super();
			this.beginTime = beginTime;
			this.endTime = endTime;
		}

		/**
         * DOCUMENT ME!
         */
        public void run() {
            while (viewerManager.getMediaPlayerController().isPlaySelectionMode() && viewerManager.getMediaPlayerController().getLoopMode() == true) {
	            try {// give player time to start
	            	Thread.sleep(200);
	            } catch (InterruptedException ie) {
	            	
	            }
	            while (viewerManager.getMasterMediaPlayer().isPlaying()) {// wait until stopped
	            	try {
	            		Thread.sleep(50);
	            	} catch (InterruptedException ie) {
	            		
	            	}
	            }
	            // then start the loop, if player not yet stopped
	            if (viewerManager.getMediaPlayerController().isPlaySelectionMode()) {
	            	viewerManager.getMediaPlayerController().startLoop(beginTime, endTime);
	            }
            }
        }
		
    }
     //end of LoopThread
	
	/**
	 * Class to compare or sort the annotations
	 * according to the start time
	 * 
	 * @author aarsom
	 *
	 */
	private class AnnotationComparator implements Comparator<Annotation>{			
		public int compare(Annotation o1, Annotation o2) {
			Long bt1 = o1.getBeginTimeBoundary();
			Long bt2 = o2.getBeginTimeBoundary();
			if(bt1 != bt2){
				return bt1.compareTo(bt2);
			} else {
				Long et1 = o1.getEndTimeBoundary();
				Long et2 = o2.getEndTimeBoundary();
				return et1.compareTo(et2);	
			}
		 }				
	}

	
}
